/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include "alert.h"

alert_entry *alert_list = NULL;
int alert_list_count = 0;
int alert_max_count = 0;
char *alert_tag = NULL;
int alert_tag_size = 0;
int alert_redraw_on_new_data = 0;

static char *normal_title(char *incoming_title)
{
    static char norm[34];
    char *c_ptr;

    strncpy(norm, incoming_title, 32);
    if ((c_ptr = strstr(norm, "County Warning Area ")) && c_ptr == norm) {
	c_ptr = &norm[strlen("County Warning Area ")];
	strcpy(norm, "CWA");
	strcat(norm, c_ptr);
    }
    while ((c_ptr = strstr(norm, ". "))) memmove(c_ptr, c_ptr+2, strlen(c_ptr));
    if ((c_ptr = strpbrk(norm, " >"))) *c_ptr = '\0';
    while ((c_ptr = strpbrk(norm, "_.-"))) memmove(c_ptr, c_ptr+1, strlen(c_ptr));
    norm[8] = '\0';
    return norm;
}

void alert_print_list(void)
{
    int i;
    char title[100], *c_ptr;

    printf("Alert counts: %d/%d\n", alert_list_count, alert_max_count);
    title[99] = '\0';
    for (i = 0; i < alert_list_count; i++) {
	strncpy(title, alert_list[i].title, 99);
	for (c_ptr = &title[strlen(title)-1]; *c_ptr == ' '; c_ptr--) *c_ptr = '\0';
	printf("Alert:%4d%c,%9s>%9s, Tag: %c%20s, Activity: %9s, Expiration: %lu, Title: %s\n", i,
	     alert_list[i].flags[0], alert_list[i].from, alert_list[i].to, alert_list[i].alert_level,
	       alert_list[i].alert_tag, alert_list[i].activity, alert_list[i].expiration, title);
    }
}

static alert_entry *alert_add_entry(alert_entry *entry)
{
    alert_entry *ptr;
    int i;

    if (strcmp(entry->to, "NWS-SOLAR") == 0) return NULL;

    if (alert_list_count == alert_max_count) {
	ptr = realloc(alert_list, (alert_max_count+10)*sizeof(alert_entry));
	if (ptr) {
	    alert_list = ptr;
	    alert_max_count += 10;
	}
    }
    if (entry->title[0] != '\0' && entry->expiration >= time(NULL)) {
	alert_redraw_on_new_data = 1;
	for (i = 0; i < alert_list_count; i++)
	  if (alert_list[i].title[0] == '\0') {
	      memcpy(&alert_list[i], entry, sizeof(alert_entry));
	      return &alert_list[i];
	  }
	if (alert_list_count < alert_max_count) {
	    memcpy(&alert_list[alert_list_count], entry, sizeof(alert_entry));
	    return &alert_list[alert_list_count++];
	}
    }

    return NULL;
}

static alert_entry *alert_match(alert_entry *alert, int match_level)
{
    int i;
    char *ptr, title_e[33], title_m[33], alert_f[33], filename[33];

    strncpy(title_e, normal_title(alert->title), 32);
    strncpy(filename, alert->filename, 32);
    title_e[32] = title_m[32] = alert_f[32] = filename[32] = '\0';
    if ((ptr = strpbrk(filename, "."))) *ptr = '\0';
    if ((ptr = strrchr(filename, '/'))) {
	ptr++;
	memmove(filename, ptr, strlen(ptr)+1);
    }
    while ((ptr = strpbrk(filename, "_ -"))) memmove(ptr, ptr+1, strlen(ptr));
        
    for (i = 0; i < alert_list_count; i++) {
	strncpy(title_m, normal_title(alert_list[i].title), 32);
	strncpy(alert_f, alert_list[i].filename, 32);
	if ((ptr = strpbrk(alert_f, "."))) *ptr = '\0';
	if ((ptr = strrchr(alert_f, '/'))) {
	    ptr++;
	    memmove(alert_f, ptr, strlen(ptr)+1);
	}
	while ((ptr = strpbrk(alert_f, "_ -"))) memmove(ptr, ptr+1, strlen(ptr));
	if ((match_level < 2 || strcmp(alert_list[i].from, alert->from) == 0) &&
	  (match_level < 1 || strcmp(alert_list[i].alert_tag, alert->alert_tag) == 0) &&
	  (title_m[0] && (strncasecmp(title_e, title_m, strlen(title_m)) == 0 || strcasecmp(title_m, filename) == 0 ||
			  strcasecmp(alert_f, title_e) == 0 || (alert_f[0] && filename[0] && strcasecmp(alert_f, filename) == 0))))
	return &alert_list[i];
    }

    return NULL;
}

void alert_update_list(alert_entry *alert, int match_level)
{
    alert_entry *ptr;
    int i;
    char title_e[33], title_m[33];

    if ((ptr = alert_match(alert, match_level))) {
	if (!ptr->filename[0]) {
	    strncpy(ptr->filename, alert->filename, 32);
	    strncpy(ptr->title, alert->title, 32);
	    ptr->filename[32] = ptr->title[32] = '\0';
	    ptr->top_boundary = alert->top_boundary;
	    ptr->left_boundary = alert->left_boundary;
	    ptr->bottom_boundary = alert->bottom_boundary;
	    ptr->right_boundary = alert->right_boundary;
	}
	ptr->flags[0] = alert->flags[0];
	strncpy(title_e, normal_title(alert->title), 32);
	title_e[32] = title_m[32] = '\0';
	for (i = 0; i < alert_list_count; i++)
	  if ((alert_list[i].flags[0] == '?' || alert_list[i].flags[0] != ptr->flags[0])) {
	      strncpy(title_m, normal_title(alert_list[i].title), 32);
	      if (strcmp(title_e, title_m) == 0) {
		  if (!alert_list[i].filename[0]) {
		      strncpy(alert_list[i].filename, alert->filename, 32);
		      alert_list[i].filename[32] = '\0';
		      alert_list[i].top_boundary = alert->top_boundary;
		      alert_list[i].left_boundary = alert->left_boundary;
		      alert_list[i].bottom_boundary = alert->bottom_boundary;
		      alert_list[i].right_boundary = alert->right_boundary;
		      strcpy(alert_list[i].title, alert->title);
		  }
		  alert_list[i].flags[0] = alert->flags[0];
	      }
	  }
    }
}

int alert_active(alert_entry *alert, int match_level)
{
    alert_entry *a_ptr;
    char l_list[] = {" RYBTGC"};
    int level = 0;
    time_t now;

    time(&now);
    if ((a_ptr = alert_match(alert, match_level))) {
	if (a_ptr->expiration >= now)
      for (level = 0; a_ptr->alert_level != l_list[level] && level < sizeof(l_list); level++);
	else if (a_ptr->expiration < (now - 3600))
	  a_ptr->title[0] = '\0';
	else if (a_ptr->flags[0] == '?')
	  a_ptr->flags[0] = '-';
    }
    return level;
}

static int alert_compare(const void *a, const void *b)
{
    alert_entry *a_entry = (alert_entry *)a;
    alert_entry *b_entry = (alert_entry *)b;
    int a_active, b_active;
    
    if (a_entry->title[0] && !b_entry->title[0]) return -1;
    if (!a_entry->title[0] && b_entry->title[0]) return 1;
    if (a_entry->flags[0] == 'Y' && b_entry->flags[0] != 'Y') return -1;
    if (a_entry->flags[0] != 'Y' && b_entry->flags[0] == 'Y') return 1;
    if (a_entry->flags[0] == '?' && b_entry->flags[0] == 'N') return -1;
    if (a_entry->flags[0] == 'N' && b_entry->flags[0] == '?') return 1;
    a_active = alert_active(a_entry, 2);
    b_active = alert_active(b_entry, 2);
    if (a_active && b_active) {
	if (a_active - b_active) return a_active - b_active;
    }
    else if (a_active)
      return -1;
    else if (b_active)
      return 1;
    return strcmp(a_entry->title, b_entry->title);
}

void alert_sort_active(void)
{
    qsort(alert_list, alert_list_count, sizeof(alert_entry), alert_compare);
    for (; alert_list_count && alert_list[alert_list_count-1].title[0] == '\0'; alert_list_count--);
}

int alert_display_request(void)
{
    int i, alert_count;
    static int last_alert_count;

    for (i = 0, alert_count = 0; i < alert_list_count; i++)
      if (alert_active(&alert_list[i], 2) && (alert_list[i].flags[0] == 'Y' || alert_list[i].flags[0] == '?'))
	alert_count++;
    if (alert_count != last_alert_count) {
	last_alert_count = alert_count;
	return (True);
    }
    return (False);
}

int alert_on_screen(void)
{
    int i, alert_count;

    for (i = 0, alert_count = 0; i < alert_list_count; i++)
      if (alert_active(&alert_list[i], 2) && alert_list[i].flags[0] == 'Y')
	alert_count++;
    return (alert_count);
}

static void alert_build_list(Message *fill)
{
    alert_entry entry[6], *list_ptr;
    int i, j;
    char *ptr;
    DataRow originator;
    
    memset(entry, 0, sizeof(entry));
    sscanf(fill->message_line, "%20[^,],%20[^,],%32[^,],%32[^,],%32[^,],%32[^,],%32[^,],%32[^,]",
	   entry[0].activity, entry[0].alert_tag, entry[0].title, entry[1].title,
	   entry[2].title, entry[3].title, entry[4].title, entry[5].title);
    entry[0].activity[20] = entry[0].alert_tag[20] = '\0';
    if (!isdigit(entry[0].activity[0])) {
	for (j = 5; j >= 0; j--) strcpy(entry[j].title, entry[j-1].title);
	strcpy(entry[0].title, entry[0].alert_tag);
	strcpy(entry[0].alert_tag, entry[0].activity);
    }
    entry[0].expiration = time_from_aprsstring(entry[0].activity);
    memset(entry[0].flags, '?', sizeof(entry[0].flags));
    strncpy(originator.call_sign, fill->from_call_sign, MAX_CALLSIGN);
    if ((i = mem_find_data(originator)) != -1) entry[0].flags[1] = station_data[station_ptr[i]].data_via;
    for (i = 0; i < 6 && entry[i].title[0]; i++) {
	entry[i].title[32] = '\0';
	while ((ptr = strpbrk(entry[i].title, " "))) memmove(ptr, ptr+1, strlen(ptr));
	if ((ptr = strpbrk(entry[i].title, "}>=!:/*+;"))) {
	    fprintf(stderr, "Warning: Wierd Weather Message: %s->%s:%s!\n", fill->from_call_sign, fill->call_sign,
		    fill->message_line);
	    *ptr = '\0';
	}
	strcpy(entry[i].activity, entry[0].activity);
	strcpy(entry[i].alert_tag, entry[0].alert_tag);
	strcpy(entry[i].from, fill->from_call_sign);
	strcpy(entry[i].to, fill->call_sign);
	entry[i].expiration = entry[0].expiration;
	memcpy(entry[i].flags, entry[0].flags, sizeof(entry[0].flags));
	if (strstr(entry[i].alert_tag, "CANCL") || strstr(entry[i].to, "CANCL")) {
	    entry[i].alert_level = 'C';
	    
	}
	else if (!strncmp(entry[i].alert_tag, "TEST", 4) || strstr(entry[i].to, "TEST"))
	  entry[i].alert_level = 'T';
	else if (strstr(entry[i].alert_tag, "WARN") || strstr(entry[i].to, "WARN"))
	  entry[i].alert_level = 'R';
	else if (strstr(entry[i].alert_tag, "WATCH") || strstr(entry[i].to, "WATCH"))
	  entry[i].alert_level = 'Y';
	else if (strstr(entry[i].alert_tag, "ADVIS") || strstr(entry[i].to, "ADVIS"))
	  entry[i].alert_level = 'B';
	else
	  entry[i].alert_level = 'G';
	if ((list_ptr = alert_match(&entry[i], 2))) {
	    list_ptr->expiration = entry[i].expiration;
	    strcpy(list_ptr->activity, entry[i].activity);
	}
	else
	  alert_add_entry(&entry[i]);
	if (alert_active(&entry[i], 2));
    }
}

int alert_message_scan(void)
{
    char *a_ptr;
    int i, j;

    if (!alert_tag) {
	alert_tag = malloc(21);
	alert_tag_size = 21;
    }
    for (i = 0; i < alert_list_count; i++) alert_active(&alert_list[i], 2);
    mscan_file(MESSAGE_NWS, alert_build_list);
    *alert_tag = '\0';
    for (j = 0; j < alert_list_count; j++)
      if (alert_list[j].flags[0] == '?') {
	  for (i = 0; i < strlen(alert_tag); i += 3) {
	      if (strncmp(&alert_tag[i], alert_list[j].title, 2) == 0) {
		  if (alert_list[j].flags[1]==DATA_VIA_TNC || alert_list[j].flags[1]==DATA_VIA_LOCAL)
		    alert_tag[i+2] = alert_list[j].flags[1];
		break;
	  }
	  }
	  if (i == strlen(alert_tag)) {
	      if (i+4 >= alert_tag_size) {
		  a_ptr = realloc(alert_tag, alert_tag_size+21);
		  if (a_ptr) {
		      alert_tag = a_ptr;
		      alert_tag_size += 21;
		  }
	      }
	      if (i+4 < alert_tag_size) {
		  a_ptr = &alert_tag[i];
		  if (alert_list[j].title[0] && alert_list[j].title[1]) {
		  *a_ptr++ = alert_list[j].title[0];
		  *a_ptr++ = alert_list[j].title[1];
		  *a_ptr++ = alert_list[j].flags[1];
		  }
		  *a_ptr = '\0';
	      }
	  }
      }
    return strlen(alert_tag);
}
