/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <Xm/Xm.h>

#include <stdio.h>
#include <math.h>

#include "symbols.h"

#ifndef PI                      /* if not defined in math.h */
#define PI  M_PI
#endif

/* Symbols */
extern SymbolData symbol_data[];
extern int symbols_loaded;

extern int debug_level;
extern char SYMBOLS_DIR[];
extern Widget appshell;
extern Widget da;
extern GC gc;
extern GC gc2;
extern int colors[];
extern long x_long_offset;
extern long y_lat_offset;
extern long size;
extern long screen_width;
extern long screen_height;
extern time_t sec_old;
extern time_t sec_clear; 
extern float x_screen_distance;
extern int symbol_callsign_display;
extern int symbol_alt_display;
extern int symbol_course_display;
extern int symbol_speed_display;
extern int symbol_dist_course_display;
extern int symbol_weather_display;
extern int letter_style;

extern time_t sec_now(void);
extern char *get_line(FILE *f, char *linedata, int maxline);

/*** symbol data ***/

void clear_symbol_data(void)
{int size;
 int i;
 char *data_ptr;


 data_ptr = (char *)symbol_data;
 size=sizeof(SymbolData);
 for(i=0;i<size;i++)
    {*data_ptr++ = 0;}

}


/* draw letters */
void draw_nice_string(Widget w, Pixmap where, int style, long x, long y, char *text, int bgcolor, int fgcolor, int length)
{
  /* this routine need to be made quicker and less efficent but for now this quick and
     gives me the type of look I want */


  if(style==0) {

    /* make outline style */  
    XSetForeground(XtDisplay(w),gc,colors[bgcolor]); 
 
    /* draw sides */
    XDrawString(XtDisplay(w),where,gc,x+1,y-1,text,length);
    XDrawString(XtDisplay(w),where,gc,x+1,y,text,length);
    XDrawString(XtDisplay(w),where,gc,x+1,y+1,text,length);
    XDrawString(XtDisplay(w),where,gc,x-1,y,text,length);
    XDrawString(XtDisplay(w),where,gc,x-1,y-1,text,length);
    XDrawString(XtDisplay(w),where,gc,x-1,y+1,text,length);
    XDrawString(XtDisplay(w),where,gc,x,y+1,text,length);
    XDrawString(XtDisplay(w),where,gc,x,y-1,text,length);
    

  } else {
    /* Draw old way */
    XSetForeground(XtDisplay(w),gc,colors[0xff]);
    XFillRectangle(XtDisplay(w),where,gc,x-1,(y-10),(length*6)+2,11);
    XSetForeground(XtDisplay(w),gc,colors[bgcolor]); 
    XDrawString(XtDisplay(w),where,gc,x+1,y+1,text,length);
  }

  /* draw text */ 
  XSetForeground(XtDisplay(w),gc,colors[fgcolor]); 
  XDrawString(XtDisplay(w),where,gc,x,y,text,length);
}

/* symbol drawing routines */

void load_pixmap_symbol_file(char *filename)
{FILE *f;
 char filen[500];
 char line[40];
 char table_char;
 char symbol_char;
 int done,i,x;
 int color;

 int old_next;

 color=0;
 symbols_loaded=0;
 table_char=0;
 symbol_char=0;
 done=0;
 sprintf(filen,"%s/%s",SYMBOLS_DIR,filename);
 f=fopen(filen,"r");
 if(f!=NULL)
   {while(!feof(f) && !done)
         {get_line(f,line,39);
          if(strncasecmp("TABLE ",line,6)==0)
            {table_char=line[6];
             /*printf("TABLE %c\n",table_char);*/
	    }
            else
            {if(strncasecmp("DONE",line,4)==0)
               {done=1;
                /*printf("DONE\n");*/
	       }
               else
               {if(strncasecmp("APRS ",line,5)==0)
                  {symbol_char=line[5];
                   if(symbols_loaded < MAX_SYMBOLS)
                     {/*printf("SYMBOL CHAR %c TABLE %c\n",symbol_char,table_char);*/
                      
                      symbol_data[symbols_loaded].pix=XCreatePixmap(XtDisplay(appshell),RootWindowOfScreen(XtScreen(appshell)),20,20,DefaultDepthOfScreen(XtScreen(appshell)));
                      symbol_data[symbols_loaded].pix_mask=XCreatePixmap(XtDisplay(appshell),RootWindowOfScreen(XtScreen(appshell)),20,20,1);
                      symbol_data[symbols_loaded].pix_mask_old=XCreatePixmap(XtDisplay(appshell),RootWindowOfScreen(XtScreen(appshell)),20,20,1);

                      x=0;
                      symbol_data[symbols_loaded].active=SYMBOL_ACTIVE;
                      symbol_data[symbols_loaded].table=table_char;
                      symbol_data[symbols_loaded].symbol=symbol_char;                     
                      old_next=0;
                      for(i=0;i<20;i++)
                         {get_line(f,line,39);
                          for(x=0;x<20;x++)
                             {switch(line[x])
                                    {case('#'):/* yellow */
                                               color=0x40;
                                               break;
 
                                     case('a'):/* darkorange2 */
                                               color=0x41;
                                               break;

                                     case('b'):/* purple */
                                               color=0x42;
                                               break;

                                     case('c'):/* gray80*/
                                               color=0x43;
                                               break;

                                     case('d'):/* red3 */
                                               color=0x44;
                                               break;

                                    case('e'):/* brown1 */
                                              color=0x45;
                                              break;

                                    case('f'):/* brown3 */
                                              color=0x46;
                                              break;

                                    case('g'):/* blue4 */
                                              color=0x47;
                                              break;

                                    case('h'):/* Deep sky blue */
                                              color=0x48;
                                              break;

                                    case('i'):/* Dark Green */
                                              color=0x49;
                                              break;

                                    case('j'):/* red2 */
                                              color=0x4a;
                                              break;

                                    case('k'):/* green3*/
                                              color=0x4b;
                                              break;

                                    case('l'):/* mediumblue */
                                             color=0x4c;
                                              break;

                                    case('m'):/* white */
                                              color=0x4d;
                                              break;

                                    case('n'):/* gray59*/
                                              color=0x4e;
                                              break;

                                    case('o'):/* gray53 */
                                              color=0x4f;
                                              break;

                                    case('p'):/* gray27 */
                                              color=0x50;
                                              break;

                                    case('q'):/* black */
                                              color=0x51;
                                              break;

                                   default:color=0xff;
                                           break;
				    }
			     
                             XSetForeground(XtDisplay(da),gc,colors[color]); 
                             XDrawPoint(XtDisplay(da),symbol_data[symbols_loaded].pix,gc,x,i); 
                             if(color!=0xff)
                               {XSetForeground(XtDisplay(appshell),gc2,1);}
                               else
                               {XSetForeground(XtDisplay(appshell),gc2,0);}                             
                             XDrawPoint(XtDisplay(appshell),symbol_data[symbols_loaded].pix_mask,gc2,x,i);
                             
                             old_next++;  
                             if(old_next>1)
                               {old_next=0;
                                XSetForeground(XtDisplay(appshell),gc2,0);
                               }                             
                             XDrawPoint(XtDisplay(appshell),symbol_data[symbols_loaded].pix_mask_old,gc2,x,i);
                             
			     }
                           old_next++;
                           if(old_next>1)
                             {old_next=0;} 
			 }
		     }
                  symbols_loaded++;
		  }
	       }
	    }
	 }
   }
   else
   {printf("Error opening symbol file %s\n",filen);}
if(f!=NULL) 
  {fclose(f);}
}

void draw_phg(long x_long, long y_lat, char *phg, time_t sec_heard, Pixmap where)
{
double power, height, gain, range, rad;
int offx,offy;
long xx,yy,xx1,yy1,fxx,fyy;
long max_x, max_y, x, y, x_origin, y_origin;
double deg,tilt;
double a,b;

xx=0l;
yy=0l;
xx1=0l;
yy1=0l;
fxx=0l;
fyy=0l;
tilt=0.0;
 
/* max off screen values */
max_x = screen_width+800l;
max_y = screen_height+800l;
			                  
if((sec_clear+sec_heard)>sec_now())
  {if((x_long>=0) && (x_long<=129600000l))
     {if((y_lat>=0) && (y_lat<=64800000l))
        {if((x_long>x_long_offset) && (x_long<(x_long_offset+(long)(screen_width*size))))
           {if((y_lat>y_lat_offset) && (y_lat<(y_lat_offset+(long)(screen_height*size))))
              {power = sqrt(((double)phg[3]-'0')/10);
               height= 10.0 * pow(2.0,(double)phg[4]-'0');
               gain = (pow(10.0,((double)(phg[5]-'0') / 10.0))/2.0);
               range = sqrt(2.0 * power * gain * height);
               rad = range/((double)x_screen_distance/(double)50);
               a=rad;
               b=rad/2;
               switch(phg[6]-'0') 
                     {case(0):offx=0;
                              offy=0;
                              break;

                      case(1):/*45*/
                           offx=-1*(rad/3);
                           offy=rad/3;
                           tilt=5.49778;
                           break;

                      case(2):/*90*/
                           offx=-1*(rad/3);
                           offy=0;
                           tilt=0; 
                           break;

                      case(3):/*135*/
                           offx=-1*(rad/3);
                           offy=-1*(rad/3);
                           tilt=.78539;
                           break;

                      case(4):/*180*/
                           offx=0;
                           offy=-1*(rad/3);
                           tilt=1.5707;
                           break;

                       case(5):/*225*/
                           offx=rad/3;
                           offy=-1*(rad/3);
                           tilt=2.3561;
                           break;

                      case(6):/*270*/
                           offx=rad/3; 
                           offy=0;
                           tilt=3.14159;
                           break;

                      case(7):/*315*/
                           offx=rad/3;
                           offy=rad/3;
                           tilt=3.92699;  
                           break;

                      case(8):/*360*/
                           offx=0;
                           offy=rad/3;
                           tilt=4.71238;
                           break;

                      default:offx=0;
                           offy=0;
                           break;

		     }

                 /*printf("PHG=%02f %0.2f %0.2f %0.2f pix %0.2f\n",range,power,height,gain,rad);*/
              if(rad>4.0)
                {XSetLineAttributes(XtDisplay(da), gc, 1, LineSolid, CapButt,JoinMiter);
                 if((sec_old+sec_heard)>sec_now())
                   {XSetForeground(XtDisplay(da),gc,colors[0x0a]);} 
                   else
                   {XSetForeground(XtDisplay(da),gc,colors[0x52]);} 
                  if(phg[6]=='0')
                    {XDrawArc(XtDisplay(da),where,gc,((x_long-x_long_offset)/size)-(rad/2),
                                         ((y_lat-y_lat_offset)/size)-(rad/2),rad,rad,0,64*360);
		    }
                    else
                    {for(deg=0; deg<(PI*2);deg+=.1)
                        {xx=a*cos(deg)*cos(tilt)-(b*sin(deg)*sin(tilt));
                         yy=a*cos(deg)*sin(tilt)+(b*sin(deg)*cos(tilt));
                         if(deg!=0)
                           {x_origin = xx+((x_long-x_long_offset)/size)-offx;
                            y_origin = yy+((y_lat-y_lat_offset)/size)-offy;
                            x = xx1+((x_long-x_long_offset)/size)-offx;
                            y = yy1+((y_lat-y_lat_offset)/size)-offy;
                            if(x_origin > (-800l) && (x_origin< max_x))
			      {if(y_origin > (-800l) && (y_origin< max_y))
                                 {if(x > (-800l) && (x< max_x))
				    {if(y > (-800l) && (y< max_y))			
                                       {XDrawLine(XtDisplay(da),where,gc,x_origin, y_origin, x, y);}
				    }
				 }
			      }
                            }
                           else
                           {fxx=xx;
                            fyy=yy;
			   }
                          xx1=xx;
                          yy1=yy;
			}
                     XDrawLine(XtDisplay(da),where,gc,xx+((x_long-x_long_offset)/size)-offx,yy+((y_lat-y_lat_offset)/size)-offy,fxx+((x_long-x_long_offset)/size)-offx,fyy+((y_lat-y_lat_offset)/size)-offy);
		    }
		}
	      }
	   }
	}
     }
  }
}


void symbol(Widget w, int ghost, char symbol_table, char symbol_id, char symbol_overlay, Pixmap where, int mask, long x_offset, long y_offset)
{int i;
 int found;
 int nosym;
 int alphanum;
 
 found = nosym = alphanum = -1;

 if (symbol_overlay == '\0' || symbol_overlay == ' ') alphanum = 0;
 for (i=0;(i<symbols_loaded);i++) {
     if (symbol_data[i].active==SYMBOL_ACTIVE) {
        if (symbol_data[i].table == symbol_table && symbol_data[i].symbol == symbol_id)
              found=i;
	
        if (symbol_data[i].table=='!' && symbol_data[i].symbol=='#')
           nosym=i;

        if (symbol_data[i].table=='#' && symbol_data[i].symbol==symbol_overlay)
           alphanum=i;

	if (alphanum != -1 && nosym != -1 && found != -1) break;

     }
 }

 if (found == -1) {
    found=nosym;
     if (symbol_table && symbol_id && debug_level & 128) printf("No Symbol Yet! %2x:%2x\n", symbol_table, symbol_id);
 }

 if (mask) {
    if (ghost) {  
       XSetClipMask(XtDisplay(w),gc,symbol_data[found].pix_mask_old);
    } else {
       XSetClipMask(XtDisplay(w),gc,symbol_data[found].pix_mask);
    }
 }

 XSetClipOrigin(XtDisplay(w),gc,x_offset,y_offset);
 XCopyArea(XtDisplay(w),symbol_data[found].pix,where,gc,0,0,20,20,x_offset,y_offset);

 if(alphanum>0) {
   if (ghost) { 
      XSetClipMask(XtDisplay(w),gc,symbol_data[alphanum].pix_mask_old);
   } else {
      XSetClipMask(XtDisplay(w),gc,symbol_data[alphanum].pix_mask);
   }

   XSetClipOrigin(XtDisplay(w),gc,x_offset,y_offset);
   XCopyArea(XtDisplay(w),symbol_data[alphanum].pix,where,gc,0,0,20,20,x_offset,y_offset);
 }
      
 XSetClipMask(XtDisplay(w),gc,None);

}

void draw_symbol(Widget w, char symbol_table, char symbol_id, char symbol_overlay, long x_long,long y_lat, char *callsign_text, char *alt_text, char *course_text, char *speed_text, char *my_distance, char *my_course, char *wx_temp, char* wx_wind, time_t sec_heard, Pixmap where)
{
 long x_offset,y_offset;
 int length;
 int ghost;

if((sec_clear+sec_heard)>sec_now())
  {if((x_long+10>=0) && (x_long-10<=129600000l))
     {if((y_lat+10>=0) && (y_lat-10<=64800000l))
        {if((x_long>x_long_offset) && (x_long<(x_long_offset+(long)(screen_width*size))))
           {if((y_lat>y_lat_offset) && (y_lat<(y_lat_offset+(long)(screen_height*size)))) {
               x_offset=((x_long-x_long_offset)/size)-(10);
               y_offset=((y_lat-y_lat_offset)/size)-(10);

   	       ghost = (sec_old+sec_heard)<sec_now();
 
               symbol(w,ghost,symbol_table,symbol_id,symbol_overlay,where,1,x_offset,y_offset);

               if(symbol_callsign_display)
                 {length=strlen(callsign_text);
                  if(length>0)
                    {x_offset=((x_long-x_long_offset)/size)+12;
                     y_offset=((y_lat-y_lat_offset)/size);

                     draw_nice_string(w,where,letter_style,x_offset,y_offset,callsign_text,0x08,0x0f,length);
		    }
		 }

    
              if(symbol_alt_display)
                {length=strlen(alt_text);
                 if(length>0)
                   {x_offset=((x_long-x_long_offset)/size)+12;
                    y_offset=((y_lat-y_lat_offset)/size)-12;

                    draw_nice_string(w,where,letter_style,x_offset,y_offset,alt_text,0x08,0x48,length); 
		   }
		}

              if(symbol_course_display)
                {length=strlen(course_text);
                 if(length>0)
                   {x_offset=((x_long-x_long_offset)/size)+12;
                    y_offset=((y_lat-y_lat_offset)/size)+12;

                    draw_nice_string(w,where,letter_style,x_offset,y_offset,course_text,0x08,0x52,length);
		   }
		}

             if(symbol_speed_display)
               {length=strlen(speed_text);
                if(length>0)
                  {x_offset=((x_long-x_long_offset)/size)+12;
                   y_offset=((y_lat-y_lat_offset)/size)+24;

                   draw_nice_string(w,where,letter_style,x_offset,y_offset,speed_text,0x08,0x4a,length); 
		  }
	       }

             if(symbol_dist_course_display)
               {length=strlen(my_distance);
                if(length>0)
                  {x_offset=(((x_long-x_long_offset)/size)-(length*6))-12;
                   y_offset=((y_lat-y_lat_offset)/size);

                   draw_nice_string(w,where,letter_style,x_offset,y_offset,my_distance,0x08,0x0f,length);
		  }

               length=strlen(my_course);
               if(length>0)
                 {x_offset=(((x_long-x_long_offset)/size)-(length*6))-12;
                  y_offset=((y_lat-y_lat_offset)/size)+12;

                  draw_nice_string(w,where,letter_style,x_offset,y_offset,my_course,0x08,0x0f,length);
		 }
	       }

             if(symbol_weather_display)
               {length=strlen(wx_temp);
                if(length>0)
                  {x_offset=((x_long-x_long_offset)/size)-20;
                   y_offset=((y_lat-y_lat_offset)/size)+36;

                   draw_nice_string(w,where,letter_style,x_offset,y_offset,wx_temp,0x08,0x40,length);
		  }

                length=strlen(wx_wind);
                if(length>0)
                  {x_offset=((x_long-x_long_offset)/size)-50;
                   y_offset=((y_lat-y_lat_offset)/size)+48;

                   draw_nice_string(w,where,letter_style,x_offset,y_offset,wx_wind,0x08,0x40,length); 
		  }
	       }
	      }
	   }
	}
     }
  }
}

