/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <stdio.h> 
#include <stdlib.h>
#include <sys/ioctl.h> 
#include <ctype.h>
#include <math.h>
#include "xastir.h"

int position_amb_chars;

char *output_lat(char *in_lat)
{int i,j;


j=0;
if(position_amb_chars>0 && position_amb_chars<5)
  {for(i=6;i>(6-position_amb_chars-j);i--)
      {if(i==4)
         {i--;
          j=1;
         }   
      in_lat[i]=' ';
      }
  }
 return(in_lat);
}


char *output_long(char *in_long)
{int i,j;

 j=0;
 if(position_amb_chars>0 && position_amb_chars<5)
   {for(i=7;i>(7-position_amb_chars-j);i--)
       {if(i==5)
          {i--;
           j=1;
          }   
        in_long[i]=' ';
       }
   }
 return(in_long);
}

char *get_line(FILE *f, char *linedata, int maxline)
{char cin;
 int pos;

pos=0;
cin=0;
strcpy(linedata,"");
while(!feof(f) && cin != 13 && cin !=10)
     {if(fread(&cin,1,1,f)==1)
        {if(pos<maxline)
           {if(cin !=13 && cin !=10)
              {linedata[pos++]=cin;}
	   }
	}
        else
        {cin=10;}
     }
linedata[pos]=0;
return(linedata);
}

char *new_get_line(FILE *f, char *linedata, int maxline)
{
    char *ptr;
 int pos;
    static char Buffer[4097];

pos=0;
    *linedata = '\0';
    while((ptr = strpbrk(Buffer, "\r\n")) == NULL && !feof(f) && strlen(Buffer) <= 4096)
      fread(&Buffer[strlen(Buffer)], sizeof(char), sizeof(Buffer)-strlen(Buffer)-1, f);
    if (ptr) *ptr = '\0';
    strncpy(linedata, Buffer, maxline);
    if ((pos = strlen(Buffer)) > maxline)
      fprintf(stderr, "Line length overflow: get_line\n");
    else if (pos < maxline) {
	linedata[pos] = '\0';
    }
    if (ptr) {
	for (ptr++; *ptr == '\r' || *ptr == '\n'; ptr++);
	strcpy(Buffer, ptr);
	   }
    return(linedata);
	}

time_t time_from_aprsstring(char *aprs_time)
{
    int day, hour, minute;
    char tz[20];
    struct tm *time_now, alert_time;
    time_t timenw;
    long zone;

    time(&timenw);
    time_now = localtime(&timenw);
    zone = timezone - 3600 * (daylight != 0);
    tz[0] = tz[1] = '\0';
    switch (sscanf(aprs_time, "%2d%2d%2d%s", &day, &hour, &minute, tz)) {
      case 0:
	day = 0;
      case 1:
	hour = 0;
      case 2:
	minute = 0;
      default:
	break;
    }
    alert_time = *time_now;
    alert_time.tm_sec = 0;
    if (day) {
	alert_time.tm_mday = day;
	if (day < (time_now->tm_mday - 10)) {
	    if (++alert_time.tm_mon > 11) {
		alert_time.tm_mon = 0;
		alert_time.tm_year++;
	    }
	}
	if (day > (time_now->tm_mday + 10)) {
	    if (--alert_time.tm_mon < 0) {
		alert_time.tm_mon = 11;
		alert_time.tm_year--;
	    }
	}	
	alert_time.tm_min = minute;
	alert_time.tm_hour = hour;
	if (tolower(tz[0]) == 'z') {
	    alert_time.tm_hour -= zone/3600;
	    zone %= 3600;
	    if (zone) alert_time.tm_min += zone/60;
	    if (alert_time.tm_min > 60) {
		alert_time.tm_hour++;
		alert_time.tm_min -= 60;
	    }
	    if (alert_time.tm_hour > 23) {
		alert_time.tm_hour -= 24;
		alert_time.tm_mday++;
		if (mktime(&alert_time) == -1) {
		    alert_time.tm_mday = 1;
		    alert_time.tm_mon++;
		    if (mktime(&alert_time) == -1) {
			alert_time.tm_mon = 0;
			alert_time.tm_year++;
		    }
		}
	    }
	    else if (alert_time.tm_hour < 0) {
		alert_time.tm_hour += 24;
		if (--alert_time.tm_mday <= 0) {
		    if (--alert_time.tm_mon < 0) {
			alert_time.tm_year--;
			alert_time.tm_mon = 11;
			alert_time.tm_mday = 31;
		    }
		    else if (alert_time.tm_mon == 3 || alert_time.tm_mon == 5 ||
			     alert_time.tm_mon == 8 || alert_time.tm_mon == 10)
		      alert_time.tm_mday = 30;
		    else if (alert_time.tm_mon == 1)
		      alert_time.tm_mday = (alert_time.tm_year%4 == 0) ? 29: 28;
        else
		      alert_time.tm_mday = 31;
     } 
	    }
	}
    }
    return(mktime(&alert_time));
}

char *compress_posit(const char *input_lat, const char group, const char *input_lon, const char symbol,
		     const int last_course, const int last_speed, const char *phg)
{
    static char pos[100];
    char lat[5], lon[5];
    char c, s, t, ext;
    int temp, deg;
    double minutes;

    sscanf(input_lat, "%2d%lf%c", &deg, &minutes, &ext);
    temp = 380926 * (90 - (deg + minutes / 60.0) * (ext=='N'?1:-1));
    lat[3] = temp%91 + 33; temp /= 91;
    lat[2] = temp%91 + 33; temp /= 91;
    lat[1] = temp%91 + 33; temp /= 91;
    lat[0] = temp + 33;
    lat[4] = '\0';
    ext = input_lon[strlen(input_lon)-1];
    sscanf(input_lon, "%3d%lf%c", &deg, &minutes, &ext);
    temp = 190463 * (180 + (deg + minutes / 60.0) * (ext=='E'?1:-1));
    lon[3] = temp%91 + 33; temp /= 91;
    lon[2] = temp%91 + 33; temp /= 91;
    lon[1] = temp%91 + 33; temp /= 91;
    lon[0] = temp + 33;
    lon[4] = '\0';
    c = s = t = ' ';
    if (last_course > 0 || last_speed > 0) {
	c = last_course/4 + 33;
	s = pow(1.08, last_speed) + 32;
	t = 'C';
    }
    else if (strlen(phg) >= 6) {
	double power, height, gain, range;
	c = '{';
	power = (phg[3]-'0')+1;
	height= 10.0 * pow(2.0,(double)phg[4]-'0');
	gain = pow(10.0,((double)(phg[5]-'0') / 5.0));
	range = sqrt(2.0 * power * gain * height);
	s = 33 + range;
	t = 'C';
    }
    sprintf(pos, "%c%s%s%c%c%c%c", group, lat, lon, symbol, c, s, t);
    return pos;
}

void uncompress_posit(const char *input, char *lat, char *ns, char *lon, char *ew, APRS_Symbol *symbol, char *course, char *speed, char *speed_unit, char *phg, char *altitude, char *alt_unit)
{
    char x1, x2, x3, x4, y1, y2, y3, y4;
    int c, s;
    float lon_float;
    float lat_float;

    /* get symbol info */
    if(input[0] != '/' && input[0] !='\\')
    {/* Symbol overlay */
	symbol->aprs_type = '\\';
	symbol->special_overlay = input[0];
    }
    else {
	symbol->aprs_type = input[0];
	symbol->special_overlay=' ';
    }
    symbol->aprs_symbol = input[9];

    /*********************************************
     Need to handle the lat/lon differently here,
     as it is in a compressed format
    *********************************************/

    y1 = input[1]-33; y2 = input[2]-33;
    y3 = input[3]-33; y4 = input[4]-33;
    x1 = input[5]-33; x2 = input[6]-33;
    x3 = input[7]-33; x4 = input[8]-33;      

    /* This will give decimal degrees for each.  W=-, E=+, N=+, S=- */
    lat_float = 90.0 - ( ( ((y1 * 91 + y2) * 91 + y3) * 91 + y4 ) / 380926.0 );

    lon_float = -180.0 + ( ( ((x1 * 91 + x2) * 91 + x3) * 91 + x4 ) / 190463.0 );

    if(debug_level & 1)
      printf("%f\t%f\n",lat_float,lon_float);

    /* Check for N/S and set variables appropriately */
    if(lat_float >= 0.0)
      *ns = 'N';
    else {
      *ns = 'S';
      lat_float = -lat_float;     /* Make it positive */
    }

    /* Check for E/W and set variables appropriately */
    if(lon_float >= 0.0)
      *ew = 'E';
    else {
      *ew = 'W';
      lon_float = -lon_float;     /* Make it positive */
    }

    /* Longitude DDDMM.MM */
    sprintf(lon, "%.3d%05.2f", (int)lon_float, (lon_float - (int)lon_float) * 60);
    if(debug_level & 1)
      printf("%c%s\t\t", *ew, lon);

    /* Latitude DDMM.MM */
    sprintf(lat, "%.2d%05.2f", (int)lat_float, (lat_float - (int)lat_float)*60);
    if(debug_level & 1)
      printf("%c%s\n", *ns, lat);

    if (input[10] < '{') {
	c=(input[10]-33);
	s=(input[11]-33);
	if (((input[12] -33) & 030) == 2) {
	    /* Altitude */
	    sprintf(altitude, "%06.0f", pow(1.002,(double)(c*91+s))*0.3048);
	    *alt_unit = 'M';
	}
	else {
	    /* get compressed course and speed */
	    /* this is course speed data */
	    *speed_unit='K';
	    /*speed in knots*/
	    sprintf(speed,"%06.2f",pow(1.08,(double)s)-1);
	    sprintf(course,"%03d",(c*4));
	    course[3]=0;
	}
    }
    else {
	s=(input[11]-33);
	if (s <= 13)
	  sprintf(phg, "PHG%s", "000");
	else
	  sprintf(phg, "PHG%s", "111");
    }
}
