/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>

#include "main.h"
#include "xastir.h"
#include "interface.h"

char wx_station_type[100];

char raw_wx_string[800];

extern void fill_wx_data(void);

extern char *get_time(char *time_here);
extern time_t sec_now(void);
extern int redraw_on_new_data;

/* from interface.c */
extern void output_my_data(char *message, int port, int type);
extern pthread_mutex_t port_data_lock; 
extern iface port_data[];

/* from db.c */
void mem_replace_data(DataRow fill, int record_num);

/* from lang.c */
extern char *langcode(char *code);
extern char langcode_hotkey(char *code);

#define MAX_WX_STRING 300
#define WX_TYPE 'z'

/* rain totals */
float rain[24];
float rain_00;
float rain_24;
float rain_base[24];
int rain_first[24];
int rain_check;

/* Other WX station data */
char wx_dew_point[10];
char wx_dew_point_on;
char wx_high_wind[10];
char wx_high_wind_on;
char wx_wind_chill[10];
char wx_wind_chill_on;
char wx_baro_inHg[10];
char wx_baro_inHg_on;
char wx_three_hour_baro[10];
char wx_three_hour_baro_on;
char wx_hi_temp[10];
char wx_hi_temp_on;
char wx_low_temp[10];
char wx_low_temp_on;
char wx_heat_index[10];
char wx_heat_index_on;


/***********************************************************/
/* return the hour of time                                 */
/***********************************************************/
int hour(void)
{struct tm *time_now;
 time_t secs_now;
 char shour[5];

 secs_now=sec_now();
 time_now = localtime(&secs_now);
 strftime(shour,4,"%H",time_now);
 
 return(atoi(shour));
}

/***********************************************************/
/* clear rain data                                         */
/***********************************************************/
void clear_rain_data(void)
{int x;
 for (x=0;x<24;x++) {
     rain[x]=0.0; 
     rain_base[x]=0.0; 
     rain_first[x]=1; 
 }
rain_00=0.0;
rain_24=0.0;
rain_check=0;
}

/***********************************************************/
/* compute_rain - compute rain total from the total rain so*/
/* far                                                     */
/***********************************************************/
    
void compute_rain(float rain_total)
{int h;
 int x;
 int i;
 float r;
 float r_first;

 
 r_first=0.0;
 h=hour();

 /* set base or first rain total for the hour */
 if (rain_first[h]) {
    rain_first[h]=0;
    rain_base[h]=rain_total;
    i=h+1;
    if (i>23)
       i=0;
    rain_first[i]=1;
    rain_check=0;
 } else {
   /* base has been set is it wrong? */
   if (rain_check<3) {
      rain_check++;

      /* is rain less then base? it shouldn't */
      if (rain_total < rain_base[h])
         rain_base[h]=rain_total; 

      /* is the difference greater than 10 inches since last reading? it shouldn't */
      if (fabs(rain_total - rain_base[h]) > 10.0)
         rain_base[h]=rain_total; 

   }
 }
 
 /* set total rain for this hour */
 rain[h]=rain_total-rain_base[h];

 /* set total rain in last 24 hours */
 r=0.0;
 i=h+1;
 if (i>23)
    i=0;
 for (x=0;x<=24;x++) {       
     r+=rain[i++];
     if (i>23)
        i=0;
 }
 rain_24=r;

 /* set rain since midnight */
 r=0.0;
 for (x=1;x<=h;x++) {
     r+=rain[x];
 }
 rain_00=r;
}


/***********************************************************/
/* compute_gust - compute wind gusts                       */
/*                                                         */
/***********************************************************/
float compute_gust(float wx_speed, float last_speed, time_t *last_speed_time)
{float computed_gust;

 /* make sure speed time is not zero */
 if(*last_speed_time==0)
   *last_speed_time=sec_now(); 

 /* set computed to last gust speed */
 computed_gust=last_speed;

 /* check to see if gust is with in 5 min*/
 if ((*last_speed_time+300) > sec_now()) {
    /* speed time within 5 min */

    /* see if gust is higher than last */ 
    if (wx_speed >= last_speed) 
       computed_gust=wx_speed;
     		
 } else { 
   /* speed time over 5 min */

   /* since is was over 5 min reset gust */
   computed_gust=wx_speed;
   *last_speed_time=sec_now(); 
 }

return(computed_gust);
}


/***********************************************************/
/* clear other wx data                                     */
/***********************************************************/
void clear_local_wx_data(void)
{
 memset(wx_dew_point,'\0',sizeof(wx_dew_point));
 wx_dew_point_on=0;

 memset(wx_high_wind,'\0',sizeof(wx_high_wind));
 wx_high_wind_on=0;

 memset(wx_wind_chill,'\0',sizeof(wx_wind_chill));
 wx_wind_chill_on=0;

 memset(wx_baro_inHg,'\0',sizeof(wx_baro_inHg));
 wx_baro_inHg_on=0;

 memset(wx_three_hour_baro,'\0',sizeof(wx_three_hour_baro));
 wx_three_hour_baro_on=0;

 memset(wx_hi_temp,'\0',sizeof(wx_hi_temp));
 wx_hi_temp_on=0;

 memset(wx_low_temp,'\0',sizeof(wx_low_temp));
 wx_low_temp_on=0;

 memset(wx_heat_index,'\0',sizeof(wx_heat_index));
 wx_heat_index_on=0;
}

/**************************************************/
/* Check last WX data received - clear data if old*/
/**************************************************/
void wx_last_data_check(void)
{
 DataRow fill;
 int record;

 /*printf("WX data check\n");*/
 strcpy(fill.call_sign,my_callsign);

 record = mem_find_data(fill);
 if (record != -1l) {
    mem_get_data(&fill,record);

    /* check to see if WX gust data has been updated timely */ 
    if ((fill.wx_speed_sec_time+360)<sec_now()){
       /* clear gust data */
       /*printf("Gust data is old, Time now %ld\n",sec_now());
       printf("WX speed_time %ld\n",fill.wx_speed_sec_time); 
       printf("last Gust %d\n",atoi(fill.wx_gust));*/
       sprintf(fill.wx_gust,"%03d",0);
       mem_replace_data(fill,record);    
    }
 }
}

/**************************************************/
/* RSW num convert                                */
/**************************************************/
int rswnc(unsigned char c)
{
 return(((c>>4)&0x0f)*10+(c&0x0f));
}                      

/***********************************************************/
/* wx fill data field                                      */
/* from: 0=local station, 1=regular decode (other stations)*/
/* type: type of WX packet                                 */
/* data: the packet of WX data                             */
/* fill: the station data to fill                          */
/***********************************************************/
void wx_fill_data(int from, int type, unsigned char *data, DataRow *fill)
{time_t last_speed_time;
 float last_speed;
 float computed_gust;
 int  temp1;
 int  temp2;
 int  temp3;
 char temp[20];
 char temp_data1[10];
 char *temp_conv;
 int len;
 int t2;
 int hidx_temp;
 int rh2;
 int hi_hum;
 int heat_index;
 char format;
 
 last_speed=0.0;
 computed_gust=0.0;
 last_speed_time=0;
 format=0;

 len=strlen(data);

 switch (type) {
    
    case (APRS_WX3):  

        if (debug_level & 1)
           printf("APRS WX3 peet bros U2k (data logging mode) %s:<%s>\n",fill->call_sign,data+2);

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"U2k");

        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed */
        if (data[2]!='-') {
           strncpy(temp_data1,data+2,4);
           temp_data1[4]=0;
           sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
           fill->wx_speed_on=1;
           if (from) {
              fill->wx_speed_sec_time = sec_now();
	   } else {
             /* local station */

             computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
             fill->wx_speed_sec_time = sec_now();
             fill->wx_gust_on=1;
             sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	   }
	} else {
          if (!from) 
             fill->wx_speed_on=0;
	}
                 
        /* wind direction */
        if (data[8]!='-') {
           strncpy(temp_data1,data+8,2);
           temp_data1[2]=0;
           sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/256.0)*360.0));
           fill->wx_course_on=1; 
	} else {
          sprintf(fill->wx_course,"000");
          if (!from) 
             fill->wx_course_on=0;
	}

        /* outdoor temp */
        if (data[10]!='-') {
           strncpy(temp_data1,data+10,4);
           temp_data1[4]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;
	}

        /* rain total long term */
        if (data[14]!='-') {
           strncpy(temp_data1,data+14,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;  
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;      
	   }
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}

        /* baro */
        if (data[18]!='-') {
           strncpy(temp_data1,data+18,4);
           temp_data1[4]=0;
           sprintf(fill->wx_baro,"%0.1f",(float)strtol(temp_data1,&temp_conv,16)/10.0);
           fill->wx_baro_on=1;

           if (!from) {
              sprintf(wx_baro_inHg,"%0.1f",(atof(fill->wx_baro)*0.02953));
              wx_baro_inHg_on=1;
	   }

	}

        /* outdoor humity */
        if (data[26]!='-') {
           strncpy(temp_data1,data+26,4);
           temp_data1[4]=0;
           sprintf(fill->wx_hum,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)/10.0));
           fill->wx_hum_on=1;
	} else {
          if (!from) 
             fill->wx_hum_on=0;
	}

        /* todays rain total */
        if (data[42]!='-') {
           if (from) {
              strncpy(temp_data1,data+42,4);
              temp_data1[4]=0;
              sprintf(fill->wx_prec_00,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
              fill->wx_prec_00_on=1;
	   }
	} else {
          if (!from) 
             fill->wx_prec_00_on=0;
	}

        break;


    case(APRS_WX4):

        if (debug_level & 1)
           printf("APRS WX4 peet bros UII %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"UII");
   
        /* wind direction */
        strncpy(temp_data1,data+1,1);
        temp_data1[1]=0;
        sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/16.0)*360.0));
        fill->wx_course_on=1;

        
        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed */
        strncpy(temp_data1,data+2,2);
        temp_data1[2]=0;
        if (data[0]=='#') {
           /* mph */
           sprintf(fill->wx_speed,"%03d",(int)(float)strtol(temp_data1,&temp_conv,16)); 
	} else {
          /* from kph to mph */
          sprintf(fill->wx_speed,"%03d",(int)(float)(strtol(temp_data1,&temp_conv,16)*0.62137)); 
	}

        fill->wx_speed_on=1;

        if (from) {           
           fill->wx_speed_sec_time = sec_now();
	} else {
          /* local station */

          computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
          fill->wx_speed_sec_time = sec_now();
          fill->wx_gust_on=1;
          sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	}
	
        /* outdoor temp */
        if (data[4]!='-') {
           strncpy(temp_data1,data+4,2);
           temp_data1[2]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)-56));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;
	}

        /* rain div by 100 for readings 0.1 inch */
        if (data[6]!='-') {
           strncpy(temp_data1,data+6,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",(float)rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;    
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;    
	   }
          
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}
        
        fill->wx_baro_on=0;
        fill->wx_hum_on=0;

       break;

    case(APRS_WX5):

        if (debug_level & 1)
           printf("APRS WX5 peet bros U2k Packet (Packet mode) %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"U2k");

        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed peak over last 5 min */
        if (data[5]!='-') {
           strncpy(temp_data1,data+5,4);
           temp_data1[4]=0;
           if (from) {
              sprintf(fill->wx_gust,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
              fill->wx_gust_on=1;
              /* this may be the only wind data */  
              sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
              fill->wx_speed_on=1;
	    } else {
              /* local station and may be the only wind data */
              if (len<56) {
                 sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
                 fill->wx_speed_on=1;

                 computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
                 fill->wx_speed_sec_time = sec_now();
                 fill->wx_gust_on=1;
                 sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	      }
	    }
	} else {
          if (!from) 
             fill->wx_gust_on=0;
	}
        
        /* wind direction */
        if (data[11]!='-') {
           strncpy(temp_data1,data+11,2);
           temp_data1[2]=0;
           sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/256.0)*360.0));
           fill->wx_course_on=1;
	} else {
          sprintf(fill->wx_course,"000");
          if (!from) 
             fill->wx_course_on=0;
	}

        /* outdoor temp */
        if (data[13]!='-') {
           strncpy(temp_data1,data+13,4);
           temp_data1[4]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;     
	}

        /* todays rain total (on some units) */
        if (data[49]!='-') {
           if (from) {
              strncpy(temp_data1,data+49,4);
              temp_data1[4]=0;
              sprintf(fill->wx_prec_00,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
              fill->wx_prec_00_on=1;
	   }
	} else {
          if (!from) 
             fill->wx_prec_00_on=0;
	}

        /* rain total long term */
        if (data[17]!='-') {
           strncpy(temp_data1,data+17,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;    
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;     
	   }
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}

        /* baro */
        if (data[21]!='-') {
           strncpy(temp_data1,data+21,4);
           temp_data1[4]=0;
           sprintf(fill->wx_baro,"%0.1f",(float)strtol(temp_data1,&temp_conv,16)/10.0);
           fill->wx_baro_on=1;

           if (!from) {
              sprintf(wx_baro_inHg,"%0.2f",(atof(fill->wx_baro)*0.02953));
              wx_baro_inHg_on=1;
	   }

	} else {
          if (!from) 
             fill->wx_baro_on=0;
	}

        /* outdoor humity */
        if (data[37]!='-') {
           strncpy(temp_data1,data+37,4);
           temp_data1[4]=0;
           sprintf(fill->wx_hum,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)/10.0));
           fill->wx_hum_on=1;
	} else {
          if (!from) 
             fill->wx_hum_on=0;
	}

        /* 1 min wind speed avg */
        if (len>53 && data[53]!='-') {
           strncpy(temp_data1,data+53,4);
           temp_data1[4]=0;
           sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
           fill->wx_speed_on=1;
           if (from) {
              fill->wx_speed_sec_time = sec_now();
	   } else {
             /* local station */
                
             computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
             fill->wx_speed_sec_time = sec_now();
             fill->wx_gust_on=1;
             sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	   }
	} else {
          if (!from) {
             if (len>53) 
                fill->wx_speed_on=0;
	  }
	}

        break;

    case(QM_WX):

        if (debug_level & 1)
           printf("Qualimetrics Q-Net %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"Q-N");

        sscanf(data,"%s %d %s %d %s %d",temp,&temp1,temp,&temp2,temp,&temp3); 

        /* outdoor temp */
        sprintf(fill->wx_temp,"%03d",(int)(temp2/10.0));
        fill->wx_temp_on=1;

        /* baro */
        sprintf(fill->wx_baro,"%0.1f",((float)temp3/100.0)*33.864);
        fill->wx_baro_on=1;

        if (!from) {
           sprintf(wx_baro_inHg,"%0.2f",(float)temp3/100.0);
           wx_baro_inHg_on=1;
	}

        /* outdoor humity */
        sprintf(fill->wx_hum,"%03d",temp1);
        fill->wx_hum_on=1;

        if (!from) {
           fill->wx_gust_on=0;
           fill->wx_course_on=0;
           fill->wx_rain_on=0;
           fill->wx_prec_00_on=0;
           fill->wx_prec_24_on=0;
           fill->wx_rain_total_on=0;
           fill->wx_gust_on=0;
           fill->wx_speed_on=0;
	}
        break;


    case(PEET_COMPLETE):

        if (debug_level & 1)
           printf("peet bros U2k Packet (Complete Record Mode) %s:<%s>\n",fill->call_sign,data);
          
        if (!from) {
           /* decode only for local station */
           fill->wx_type=WX_TYPE;
           strcpy(fill->wx_station,"U2k");

           if (data[12]!='-') {
              strncpy(temp_data1,data+12,4);
              temp_data1[4]=0;

              sprintf(fill->wx_gust,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
              fill->wx_gust_on=1;

              
	   } else fill->wx_gust_on=0;
	
           /* wind direction */
           if (data[8]!='-') {
              strncpy(temp_data1,data+8,4);
              temp_data1[4]=0;
              sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/256.0)*360.0));
              fill->wx_course_on=1;
	   } else {
             sprintf(fill->wx_course,"000");
             fill->wx_course_on=0;
	   }

          /* outdoor temp */
          if (data[24]!='-') {
             strncpy(temp_data1,data+24,4);
             temp_data1[4]=0;
             sprintf(fill->wx_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
             fill->wx_temp_on=1;
	  } else fill->wx_temp_on=0;

          /* todays rain total (on some units) */
          if (data[28]!='-') {
              strncpy(temp_data1,data+28,4);
              temp_data1[4]=0;
              sprintf(fill->wx_prec_00,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
              fill->wx_prec_00_on=1;

	  } else fill->wx_prec_00_on=0;

          /* rain total long term */
          if (data[432]!='-') {
             strncpy(temp_data1,data+432,4);
             temp_data1[4]=0;
             sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
             fill->wx_rain_total_on=1;

             /* Since local station only */
             compute_rain(atof(fill->wx_rain_total));

             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
             fill->wx_rain_on=1;
             
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;
            
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1; 
	  } else fill->wx_rain_total_on=0;

          /* baro */
          if (data[32]!='-') {
             strncpy(temp_data1,data+32,4);
             temp_data1[4]=0;
             sprintf(fill->wx_baro,"%0.1f",(float)strtol(temp_data1,&temp_conv,16)/10.0);
             fill->wx_baro_on=1;
	  } else fill->wx_baro_on=0;

          /* outdoor humity */
          if (data[52]!='-') {
             strncpy(temp_data1,data+52,4);
             temp_data1[4]=0;
             sprintf(fill->wx_hum,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)/10.0));
             fill->wx_hum_on=1;
	  } else fill->wx_hum_on=0;
	
          /* dew point */
          if (data[60]!='-') {
             strncpy(temp_data1,data+60,4);
             temp_data1[4]=0;
             sprintf(wx_dew_point,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
             wx_dew_point_on=1;
	  }

          if (data[4] !='-'){
             strncpy(temp_data1,data+4,4);
             temp_data1[4]=0;
             sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
             fill->wx_speed_on=1;
	  }
	  
          /*high winds for today*/
          if (data[248]!='-') {
             strncpy(temp_data1,data+248,4);
             temp_data1[4]=0;
             sprintf(wx_high_wind,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
             wx_high_wind_on=1;
	  }

          /*wind chill */
          if (data[20]!='-') {
             strncpy(temp_data1,data+20,4);
             temp_data1[4]=0;
             sprintf(wx_wind_chill,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
             wx_wind_chill_on=1;
	  }

          /* Barometric Pressure inHg */
          if (data[32]!='-') {
             strncpy(temp_data1,data+32,4);
             temp_data1[4]=0;
             sprintf(wx_baro_inHg,"%2.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0/3.38639);
             wx_baro_inHg_on=1;
	  }

          /*3-Hr Barometric Change */
          if (data[36]!='-') {
             strncpy(temp_data1,data+36,4);
	     temp_data1[4]=0;
	     sprintf(wx_three_hour_baro,"%0.2f",(float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/100.0/3.38639);
	     wx_three_hour_baro_on=1;
	  }
		
         /* High Temp for Today*/
         if (data[276]!='-') {
            strncpy(temp_data1,data+276,4);
            temp_data1[4]=0;
            sprintf(wx_hi_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
            wx_hi_temp_on=1;
	 } else wx_hi_temp_on=0;

         /* Low Temp for Today*/
         if (data[100]!='-') {
            strncpy(temp_data1,data+100,4);
            temp_data1[4]=0;
            sprintf(wx_low_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
            wx_low_temp_on=1;
	 } else wx_low_temp_on=0;

         /* Heat Index Calculation*/		
         hi_hum=atoi(fill->wx_hum);
         rh2= atoi(fill->wx_hum);
         rh2=(rh2 * rh2);
         hidx_temp=atoi(fill->wx_temp);
         t2= atoi(fill->wx_temp);
         t2=(t2 * t2);

         if (hidx_temp >= 70) {	
            heat_index=-42.379+2.04901523 * hidx_temp+10.1433127 * hi_hum-0.22475541 * hidx_temp * hi_hum-0.00683783 * t2-0.05481717 * rh2+0.00122874 * t2 * hi_hum+0.00085282 * hidx_temp * rh2-0.00000199 * t2 * rh2;
            sprintf (wx_heat_index,"%03d",heat_index);
            wx_heat_index_on=1;
	 } else wx_heat_index_on=0;
	} 

        break;

    case(RSWX200):
          if (!from) {
             if (debug_level & 1)
                printf("RSWX200 WX (binary)\n"); 

             fill->wx_type=WX_TYPE;
             strcpy(fill->wx_station,"RSW");

             switch (data[0]) {

               case 0x8f: /* humidity */
                 if (rswnc(data[20])<100) {
                    sprintf(fill->wx_hum,"%03d",rswnc(data[20]));
		 } else sprintf(fill->wx_hum,"100");
                 fill->wx_hum_on=1;
                 break;

               case 0x9f: /* temp */
                 /* all data in C ?*/ 
                 sprintf(temp_data1,"%c%d%0.1f",((data[17]&0x08) ? '-' : '+'),(data[17]&0x7),rswnc(data[16])/10.0);
                 /*printf("temp data: <%s> %d %d %d\n", temp_data1,((data[17]&0x08)==0x08),(data[17]&0x7),rswnc(data[16]));*/
                 sprintf(fill->wx_temp,"%03d",(int)(atof(temp_data1)*1.8)+32);
                 fill->wx_temp_on=1; 
                 /*printf("Temp %s C %0.2f %03d\n",temp_data1,atof(temp_data1),(int)atof(temp_data1)); 
                 printf("Temp F %0.2f %03d\n",(atof(temp_data1)*1.8)+32,(int)(atof(temp_data1)*1.8)+32); 
                 */
                 sprintf(temp_data1,"%c%d%d.%d",((data[18]&0x80) ? '-' : '+'),(data[18]&0x70)>>4,(data[18]&0x0f),(data[17] & 0xf0) >> 4); 
                 sprintf(wx_hi_temp,"%03d",(int)(atof(temp_data1)*1.8)+32);
                 wx_hi_temp_on=1; 

                 sprintf(temp_data1,"%c%d%d.%d",((data[23]&0x80) ? '-' : '+'),(data[23]&0x70)>>4,(data[23]&0x0f),(data[22] & 0xf0) >> 4); 
                 sprintf(wx_low_temp,"%03d",(int)(atof(temp_data1)*1.8)+32);
                 wx_low_temp_on=1; 

                 break;

               case 0xaf: /* baro/dewpt */
                 /* local baro pressure in mb?*/
                 /*sprintf(fill->wx_baro,"%02d%02d",rswnc(data[2]),rswnc(data[1]));                  
		 fill->wx_baro_on=1;*/

                 /* Sea Level Adjusted baro in mb */
                 sprintf(fill->wx_baro,"%0d%02d%0.1f",(data[5]&0x0f),rswnc(data[4]),((float)rswnc(data[3])/10.0));              
		 fill->wx_baro_on=1;

                 sprintf(wx_baro_inHg,"%0.2f",(atof(fill->wx_baro)*0.02953));
                 wx_baro_inHg_on=1;

                 /* dew point in C */
                 sprintf(wx_dew_point,"%03d",(int)(rswnc(data[18])*1.8)+32);
                 wx_dew_point_on=1; 
                 break;

               case 0xbf: /* Rain */
                 /* all data in mm ?*/
                 sprintf(temp_data1,"%02d%02d",rswnc(data[6]), rswnc(data[5]));
                 sprintf(fill->wx_rain_total,"%0.2f",atof(temp_data1)*0.03937);
                 fill->wx_rain_total_on=1;

                 /* Since local station only */
                 compute_rain(atof(fill->wx_rain_total));
                 /*last hour rain */
                 sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
                 fill->wx_rain_on=1;
             
                 /*last 24 hour rain */
                 sprintf(fill->wx_prec_24,"%0.2f",rain_24);
                 fill->wx_prec_24_on=1;
            
                 /* rain since midnight */
                 sprintf(fill->wx_prec_00,"%0.2f",rain_00);
                 fill->wx_prec_00_on=1; 
                
                 break;

              case 0xcf: /* Wind w/chill*/

                 /* get last gust speed */
                 if (fill->wx_gust_on) {
                    /* get last speed */
                    last_speed=atof(fill->wx_gust);
                    last_speed_time=fill->wx_speed_sec_time;
		 }

                 /* all data in m/s */                    
                 /* average */
                 sprintf(temp_data1,"%01d%0.1f",(data[5]&0xf),(float)(rswnc(data[4])/10));
                 sprintf(fill->wx_speed,"%03d",(int)(atof(temp_data1)*2.2369));
                 /* gust */
                 sprintf(temp_data1,"%01d%0.1f",(data[2]&0xf),(float)(rswnc(data[1])/10));
                 /*sprintf(fill->wx_gust,"%03d",(int)(atof(temp_data1)*2.2369));*/

                 /* do computed gust */  
                 computed_gust = compute_gust((int)(atof(temp_data1)*2.2369),last_speed,&last_speed_time);
                 fill->wx_speed_sec_time = sec_now();
                 sprintf(fill->wx_gust,"%03d",(int)computed_gust);

                 /* high gust */
                 sprintf(temp_data1,"%01d%0.1f",(data[8]&0xf), (float)(rswnc(data[7])/10));
                 sprintf(wx_high_wind,"%03d",(int)(atof(temp_data1)*2.2369));
                 fill->wx_speed_on=1;
                 fill->wx_gust_on=1;
                 wx_high_wind_on=1;

                 sprintf(fill->wx_course,"%02d%01d",rswnc(data[3]),(data[2]&0xf0)>>4);
                 fill->wx_course_on=1;

                 /* chill in C */
                 sprintf(temp_data1,"%c%d",((data[21]&0x20) ? '-' : '+'),rswnc(data[16])); 
                 sprintf(wx_wind_chill,"%03d",(int)(atof(temp_data1)*1.8)+32);
                 wx_wind_chill_on=1; 
                break; 
              default:break;
	     }

	   if (fill->wx_hum_on==1 && fill->wx_temp_on==1) {
              /* Heat Index Calculation*/		
              hi_hum=atoi(fill->wx_hum);
              rh2= atoi(fill->wx_hum);
              rh2=(rh2 * rh2);
              hidx_temp=atoi(fill->wx_temp);
              t2= atoi(fill->wx_temp);
              t2=(t2 * t2);
             
              if (hidx_temp >= 70) {	
                 heat_index=-42.379+2.04901523 * hidx_temp+10.1433127 * hi_hum-0.22475541 * hidx_temp * hi_hum-0.00683783 * t2-0.05481717 * rh2+0.00122874 * t2 * hi_hum+0.00085282 * hidx_temp * rh2-0.00000199 * t2 * rh2;
                 sprintf (wx_heat_index,"%03d",heat_index);
                 wx_heat_index_on=1;
	      } else wx_heat_index_on=0;
	   }
	  }
 }
}


/***********************************************************/
/* Decode WX data line                                     */
/* wx_line: raw wx data to decode                          */
/***********************************************************/
void wx_decode(unsigned char *wx_line, int port)
{
 DataRow fill;
 int record;
 int decoded;
 int find;
 int i;
 int len;
 char time_data[MAX_TIME];
 unsigned int check_sum;
 int max;

 find=0;
 len=strlen(wx_line);
 if (len>10 || (wx_line[0]!=0 && port_data[port].data_type==1)) {
    strcpy(fill.call_sign,my_callsign);

    record = mem_find_data(fill);
    if (record != -1l) {
       /* current data */
       mem_get_data(&fill,record);
     

       decoded=0;
       /* Ok decode wx data */
       if (wx_line[0]=='!' && wx_line[1]=='!' && is_xnum_or_dash(wx_line+2,40) && port_data[port].data_type==0) {
          /* Found Peet Bros U2k */
          /*printf("Found Peet Bros U2k WX:%s\n",wx_line+2);*/
          strcpy(wx_station_type,langcode("WXPUPSI011"));
          strcpy(raw_wx_string,wx_line);
          strcpy(fill.wx_time,get_time(time_data));
          fill.wx_sec_time=sec_now();
          fill.wx_data=1;
          wx_fill_data(0,APRS_WX3,wx_line,&fill);
          decoded=1; 
          } else { 
            if (((wx_line[0]=='#') || (wx_line[0]=='*')) && is_xnum_or_dash(wx_line+1,13) && port_data[port].data_type==0) {
               /* Found Peet Bros raw U2 data */
               strcpy(wx_station_type,langcode("WXPUPSI012"));
               /*printf("Found Peet Bros raw U2 data WX#:%s\n",wx_line+1);*/
               strcpy(raw_wx_string,wx_line);
               strcpy(fill.wx_time,get_time(time_data));
               fill.wx_sec_time=sec_now();
               fill.wx_data=1;
               wx_fill_data(0,APRS_WX4,wx_line,&fill);
               decoded=1; 
	    } else { 
              if (strncmp("$ULTW",wx_line,5)==0 && is_xnum_or_dash(wx_line+5,44) && port_data[port].data_type==0) {
                 /* Found Peet Bros raw U2 data */
                 strcpy(wx_station_type,langcode("WXPUPSI013"));
                 /*printf("Found Peet Bros Ultmeter Packet data WX#:%s\n",wx_line+5);*/
                 strcpy(raw_wx_string,wx_line);
                 fill.wx_sec_time=sec_now();
                 fill.wx_data=1;
                 wx_fill_data(0,APRS_WX5,wx_line,&fill);
                 decoded=1; 

	      } else {
                if (wx_line[2]==' ' && wx_line[5]==' ' && wx_line[8]=='/' && wx_line[11]=='/' 
                    && wx_line[14]==' ' && wx_line[17]==':' && wx_line[20]==':' 
                    && strncmp(" #0:",wx_line+23,4)==0 && port_data[port].data_type==0
                    ) { 
                    find=0;
                    for (i=len;i>23 && !find;i--) {
                      if (wx_line[i]==0x03) {
                         find=1;
                         wx_line[i]=0l;
		      }
		    }
                    if (find) {   
                       /* found Qualimetrics Q-Net station */
                       strcpy(wx_station_type,langcode("WXPUPSI016"));
                       /*printf("Found Qualimetrics Q-Net station data WX#:%s\n",wx_line+23);*/
                       strcpy(fill.wx_time,get_time(time_data));
                       fill.wx_sec_time=sec_now();
                       fill.wx_data=1;
                       wx_fill_data(0,QM_WX,wx_line+24,&fill);
                       decoded=1; 
		    }

		} else {
                  if (strncmp("&CR&",wx_line,4)==0 && is_xnum_or_dash(wx_line+5,44) && port_data[port].data_type==0) {
                     strcpy(wx_station_type,langcode("WXPUPSI017"));
                     strcpy(raw_wx_string,wx_line);
                     fill.wx_sec_time=sec_now();
                     fill.wx_data=1;
                     wx_fill_data(0,PEET_COMPLETE,wx_line,&fill);
                     decoded=1;
		  } else {
                    if (port_data[port].data_type==1) {
                       /* binary data type */
                       
                       /* clear raw string */
                       memset(raw_wx_string,'\0',sizeof(raw_wx_string));

                       max=0;
                       switch (wx_line[0]) {
                           case 0x8f:max=34;
                                break;
                           case 0x9f:max=33;
                                break;
                           case 0xaf:max=30;
                                break;
                           case 0xbf:max=13;
                                break;
                           case 0xcf:max=26;
                                break;
                           default:break;
		       }
                       
                       if (max>0) {
                          check_sum=0;
                          for (i=0; i<max;i++) 
                              check_sum+=wx_line[i];

			  if (wx_line[max] == (0xff & check_sum)) {
                             /* good RS WX-200 data */
                             /*printf("GOOD %0X data\n",wx_line[0]);*/

                             /* found RS WX-200 */
                             strcpy(wx_station_type,langcode("WXPUPSI025"));
                             strcpy(fill.wx_time,get_time(time_data));
                             fill.wx_sec_time=sec_now();
                             fill.wx_data=1;
                             wx_fill_data(0,RSWX200,wx_line,&fill);
                             decoded=1;
			  }
		       }
                      
		    } else {
                      if (debug_level & 1)
                         printf("Unknown WX DATA:%s\n",wx_line);
		    }
		  }
		}
	      }
	    }
	  }
    
      if (decoded) {
         /* save data back */
         pthread_mutex_lock(&port_data_lock);
         port_data[port].decode_errors=0;
         pthread_mutex_unlock(&port_data_lock);

         XmTextFieldSetString(text,langcode("BBARSTA032"));
         XmUpdateDisplay(text);  
     
         mem_replace_data(fill,record);

         /* redraw now */
         redraw_on_new_data=2;
         fill_wx_data();
      } else {
        /* Undecoded packet */   
        memset(raw_wx_string,'\0',sizeof(raw_wx_string));
        pthread_mutex_lock(&port_data_lock);
        port_data[port].decode_errors++;
        if (port_data[port].decode_errors>10) {
           /* wrong data type? */
           port_data[port].data_type++;
           port_data[port].data_type&=0x01;
           /*if (debug_level & 1)*/
              printf("Data type %d\n",port_data[port].data_type);
           port_data[port].decode_errors=0;
	}
        pthread_mutex_unlock(&port_data_lock);
      }
    }
 }
}


/***********************************************************/
/* fill string with WX data for transmit                   */
/*                            */
/***********************************************************/

time_t wx_tx_data1(char *st)
{
 DataRow fill;
 int record;
 time_t wx_time;
 char temp[100];

 strcpy(st,"");
 wx_time=0;
 strcpy(fill.call_sign,my_callsign);
 
 record = mem_find_data(fill);
 if (record != -1l) {
    /* current data */
    mem_get_data(&fill,record);
    if (fill.wx_data) {
       /* station has wx data */
       if (fill.wx_course_on && fill.wx_speed_on) {
          /* we have enough wx_data */
          wx_time=fill.wx_sec_time;
          sprintf(st,"%s/%s",fill.wx_course,fill.wx_speed);
       } else {
         /* we don't have enough wx_data, must be from a Qualimetrics Q-Net */
          wx_time=fill.wx_sec_time;
          sprintf(st,".../...");
       }

       if (fill.wx_gust_on) {
          sprintf(temp,"g%s",fill.wx_gust);
          if (strlen(temp)>4) 
             strcpy(temp,"g999");
          strcat(st,temp);
	  } else strcat(st,"g...");
          
          
       if (fill.wx_temp_on) {
          sprintf(temp,"t%s",fill.wx_temp);
          strcat(st,temp);
       } else strcat(st,"t...");


       if (fill.wx_rain_on) {
          sprintf(temp,"r%03d",(int)(atof(fill.wx_rain)*100.0));
          if (strlen(temp)>4) 
             strcpy(temp,"r999");
          strcat(st,temp);
       } else strcat(st,"r...");

       if (fill.wx_prec_00_on) {
          sprintf(temp,"P%03d",(int)(atof(fill.wx_prec_00)*100.0));
          if (strlen(temp)>4)
             strcpy(temp,"P999");
          strcat(st,temp);  
       } else strcat(st,"P...");

       if (fill.wx_prec_24_on) {
          sprintf(temp,"p%03d",(int)(atof(fill.wx_prec_24)*100.0));
          if (strlen(temp)>4)
             strcpy(temp,"p999");
          strcat(st,temp);
       } else strcat(st,"p...");

 
       if (fill.wx_hum_on) {
           if (atoi(fill.wx_hum)>99) {
              sprintf(temp,"h00");
	   } else sprintf(temp,"h%02d",atoi(fill.wx_hum));
           strcat(st,temp); 
       } else strcat(st,"h..");
	 
         
       if (fill.wx_baro_on) {
          sprintf(temp,"b%05d",(int)(atof(fill.wx_baro)*10.0));
          if (strlen(temp)>6)
             strcpy(temp,"b99999");
          strcat(st,temp);
       } else strcat(st,"b.....");
         
       sprintf(temp,"%c%s",fill.wx_type,fill.wx_station);
       strcat(st,temp);
    }
     }

return(wx_time);
}


/***********************************************************/
/* transmit raw wx data                                    */
/*                                                         */
/***********************************************************/
void tx_raw_wx_data(void)
{

 if (strlen(raw_wx_string)>10) {
    output_my_data(raw_wx_string,-1,0);
    if (debug_level & 1)
       printf("Sending Raw WX data <%s>\n",raw_wx_string);
 }

}
