/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MenuShell.h>
#include <Xm/MessageB.h>
#include <Xm/MwmUtil.h>
#include <Xm/PanedW.h>
#include <Xm/Protocols.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/SeparatoG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <ctype.h>
#include <unistd.h>
#include <dirent.h>
#include <signal.h>
#include <math.h>
#include <termios.h>
#include <pwd.h>
#include <pthread.h> 
#include <locale.h>

#include "xastir.h"
#include "symbols.h"
#include "main.h"
#include "config.h"
#include "maps.h"
#include "alert.h"
#include "db.h"
#include "interface.h"

#ifndef PI                      /* if not defined in math.h */
#define PI  M_PI
#endif

#define DOS_HDR_LINES 8

#define GRID_MORE 513

FILE *file_wx_test;

int dtr_on=1;

/* language in use */
char lang_to_use[30];

/* version info in main.h */

void da_input(Widget w, XtPointer client_data, XtPointer call_data);
void da_resize(Widget w, XtPointer client_data, XtPointer call_data);
void da_expose(Widget w, XtPointer client_data, XtPointer call_data);

int debug_level;

Widget hidden_shell = (Widget) NULL;
Widget appshell = (Widget) NULL;
Widget form = (Widget) NULL;
Widget da = (Widget) NULL;
Widget text;
Widget text2;
Widget text3;
Widget text4;
Widget iface_da;
Widget menubar;

Widget help_index_dialog = (Widget)NULL;
Widget help_view_dialog = (Widget)NULL;
Widget map_chooser_dialog = (Widget)NULL;
Widget configure_station_dialog = (Widget)NULL;
Widget configure_audio_alarm_dialog = (Widget)NULL;
Widget menu_popup=(Widget)NULL;
Widget station_popup=(Widget)NULL;

int Station_transmit_type;
int Igate_type;

Widget Display_data_dialog  = (Widget)NULL;
Widget Display_data_text;
int Display_packet_data_type;


Widget configure_defaults_dialog = (Widget)NULL;
int old_station_item;
int clear_station_item;

/* help index */
Widget help_list;

extern Widget auto_msg_on, auto_msg_off;
Widget grid_on, grid_off;
Widget map_auto_on, map_auto_off; 
Widget map_levels_on, map_levels_off; 
Widget map_bgcolor[11];
Widget map_station_label0,map_station_label1;
Widget map_wx_alerts_0,map_wx_alerts_1;
Widget tnc_logging_on, tnc_logging_off;
Widget net_logging_on, net_logging_off;
Widget igate_logging_on, igate_logging_off;
Widget wx_logging_on, wx_logging_off;
Widget alt_on, alt_off;
Widget trails_on, trails_off;
Widget phg_on, phg_off;
Widget wx_on, wx_off;
Widget course_on, course_off;
Widget speed_on, speed_off;
Widget dist_course_on, dist_course_off;
Widget units_choice_on, units_choice_off;
Widget read_selection_dialog = (Widget)NULL;

/* config station values */
Widget station_config_call_data, station_config_slat_data_deg, station_config_slat_data_min, 
       station_config_slat_data_ns, station_config_slong_data_deg, station_config_slong_data_min, 
       station_config_slong_data_ew, station_config_group_data, station_config_symbol_data, 
       station_config_comment_data, station_config_power_data, station_config_height_data, 
       station_config_gain_data, station_config_direction_data;

/* defauls*/
Widget raw_wx_tx;
Widget compressed_posit_tx;

/* config audio alarm values */
Widget audio_alarm_config_play_data,
       audio_alarm_config_play_on_new_station, audio_alarm_config_play_ons_data,
       audio_alarm_config_play_on_new_message, audio_alarm_config_play_onm_data,
       audio_alarm_config_play_on_prox, audio_alarm_config_play_onpx_data,
       audio_alarm_config_play_on_bando, audio_alarm_config_play_onbo_data,
       prox_min_data, prox_max_data, bando_min_data, bando_max_data,
       audio_alarm_config_play_on_wx_alert, audio_alarm_config_wx_alert_data;


/* map chooser */
Widget map_list;


extern void create_gc(Widget w);
static void UpdateTime( XtPointer clientData, XtIntervalId id );
void pos_dialog(Widget w);

static void Zoom_in(Widget w, XtPointer clientData, XtPointer calldata);
static void Zoom_out(Widget w, XtPointer clientData, XtPointer calldata);
static void Zoom_level(Widget w, XtPointer clientData, XtPointer calldata);

static void Pan_up(Widget w, XtPointer clientData, XtPointer calldata);
static void Pan_down(Widget w, XtPointer clientData, XtPointer calldata);
static void Pan_left(Widget w, XtPointer clientData, XtPointer calldata);
static void Pan_right(Widget w, XtPointer clientData, XtPointer calldata);

extern void Station_info(Widget w, XtPointer clientData, XtPointer calldata);
extern void Station_List(Widget w, XtPointer clientData, XtPointer calldata);

extern void Window_Quit(Widget w, XtPointer client, XtPointer call);
static void Menu_Quit(Widget w, XtPointer clientData, XtPointer calldata);

static void Grid_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Grid_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Map_auto_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Map_auto_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Map_levels_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Map_levels_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Map_background(Widget w, XtPointer clientData, XtPointer calldata);

static void Map_station_label(Widget w, XtPointer clientData, XtPointer calldata);
static void Map_wx_alerts(Widget w, XtPointer clientData, XtPointer calldata);

static void TNC_Logging_on(Widget w, XtPointer clientData, XtPointer calldata);
static void TNC_Logging_off(Widget w, XtPointer clientData, XtPointer calldata);
static void TNC_Transmit_now(Widget w, XtPointer clientData, XtPointer calldata);

static void Net_Logging_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Net_Logging_off(Widget w, XtPointer clientData, XtPointer calldata);

static void IGate_Logging_on(Widget w, XtPointer clientData, XtPointer calldata);
static void IGate_Logging_off(Widget w, XtPointer clientData, XtPointer calldata);

static void WX_Logging_on(Widget w, XtPointer clientData, XtPointer calldata);
static void WX_Logging_off(Widget w, XtPointer clientData, XtPointer calldata);

void on_off_switch(int switchpos, Widget first, Widget second);

static void Alt_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Alt_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Course_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Course_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Speed_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Speed_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Dist_course_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Dist_course_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Units_choice_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Units_choice_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Station_trails_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Station_trails_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Station_phg_on(Widget w, XtPointer clientData, XtPointer calldata);
static void Station_phg_off(Widget w, XtPointer clientData, XtPointer calldata);

static void WX_on(Widget w, XtPointer clientData, XtPointer calldata);
static void WX_off(Widget w, XtPointer clientData, XtPointer calldata);

static void Help_About(Widget w, XtPointer clientData, XtPointer callData);
static void Help_Index(Widget w, XtPointer clientData, XtPointer callData);

static void Map_chooser(Widget w, XtPointer clientData, XtPointer callData);

static void Configure_station(Widget w, XtPointer clientData, XtPointer callData);

static void Configure_audio_alarms(Widget w, XtPointer clientData, XtPointer callData);

static void Configure_defaults(Widget w, XtPointer clientData, XtPointer callData);

static void Stations_Clear(Widget w, XtPointer clientData, XtPointer callData);

extern void Tracks_All_Clear(Widget w, XtPointer clientData, XtPointer callData);

static void Read_File_Selection(Widget w, XtPointer clientData, XtPointer callData);

static void Display_data(Widget w, XtPointer clientData, XtPointer callData);

extern void Locate_station(Widget w, XtPointer clientData, XtPointer callData);

extern void Display_Wx_Alert(Widget w, XtPointer clientData, XtPointer callData);

/* from popup_gui.c */
extern void clear_popup_message_windows(void);
extern void popup_time_out_check(void);

/* from messages_gui.c */
extern void Send_message(Widget w, XtPointer clientData, XtPointer callData);
extern void Clear_messages(Widget w, XtPointer clientData, XtPointer callData);

extern void Auto_msg_on(Widget w, XtPointer clientData, XtPointer calldata);
extern void Auto_msg_off(Widget w, XtPointer clientData, XtPointer calldata);
extern void Auto_msg_set(Widget w, XtPointer clientData, XtPointer calldata);

static void Test(Widget w, XtPointer clientData, XtPointer callData);

static void Save_Config(Widget w, XtPointer clientData, XtPointer callData); 

/* from track_gui.c */
extern void Track_station(Widget w, XtPointer clientData, XtPointer callData);
extern int track_station_on;
extern int track_case;
extern int track_match;
extern char tracking_station_call[30];

extern void wx_alert_update_list(void);

extern void WX_station(Widget w, XtPointer clientData, XtPointer callData);

/* from color.c */
extern int load_color_file(void);
extern Pixel GetPixelByName(Widget w, char *colorname);

/* from list_gui.c */
extern void update_station_scroll_list(void);

/* from interface_gui.c */
extern void Configure_interface_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData);
extern void Configure_interface(Widget w, XtPointer clientData, XtPointer callData);
extern void output_my_aprs_data(void);
extern unsigned char *data;
extern int data_port;
extern int data_avail;
extern void control_interface(Widget w, XtPointer clientData, XtPointer callData);
extern void dtr_all_set(int dtr);

/* from wx.c */
extern void wx_decode(unsigned char *wx_line, int port);
extern char wx_station_type[];
extern void fill_wx_data(void);
extern void clear_rain_data(void);
extern void tx_raw_wx_data(void);
extern void clear_local_wx_data(void);
extern void wx_last_data_check(void);

/* GLOBAL DEFINES */
GC gc=0;
GC gc2=0;
Pixmap  pixmap;
Pixmap  pixmap_final;

/* database pointers */

char *database_ptr;

#define XmFONTLIST_DEFAULT_MY "fixed"

int delay_time;                 /* used to delay display data */
char user_dir[1000];            /* user directory file */
long mid_x_long_offset;		/* Center of map long offset */ 
long mid_y_lat_offset;		/* Center of map lat offset  */

long x_long_offset;		/* top NW corner long pos of screen map */
long y_lat_offset;		/* top NW corner long pos of screen map */

long size;			/* size or zoom */

long screen_width;		/* map area screen width */
long screen_height;		/* map area screen height */

float d_screen_distance;        /* Diag screen distance */
float x_screen_distance;        /* x screen distance */

int colors[256];		/* screen colors */
int trail_colors[32];           /* station trail colors, duh */
int current_trail_color;        /* what color to draw station trails with */
int max_trail_colors; 

int map_background_color;       /* Background color for maps */
int letter_style;               /* Station Letter style */
int wx_alert_style;             /* WX alert map style */

time_t max_transmit_time; /* max time between transmits */

int symbol_callsign_display;	/* Display Callsign on symbols? */
int symbol_alt_display;		/* Display Altitude on symbols? */
int symbol_course_display;	/* Display course on symbols? */
int symbol_speed_display;	/* Display speed on symbols? */
int symbol_dist_course_display;	/* Display from me distance and course  on symbols? */
int symbol_weather_display;	/* Display wx info on symbols? */

int redraw_on_new_data;         /* data base redraw */
int wait_to_redraw;             /* wait to redraw until till system is up */
int display_up;                 /* display up? */
int display_up_first;           /* display up first */

time_t last_alert_redraw;      /* last time alert caused a redraw */

int log_tnc_data;               /* log data */
int log_net_data;               /* log data */

time_t sec_next_gps;            /* next gps check */
time_t gps_time;                /* gps delay time */  

time_t sec_next_raw_wx;         /* raw wx transmit data */
int transmit_raw_wx;            /* transmit raw wx data? */

int transmit_compressed_posit;  /* transmit location in compressed format? */

int output_station_type;        /* Broadcast station type */

int Configure_station_pos_amb;  /* Broadcast station position ambiguity */

int units_english_metric;       /* units are in english if 1 or metric if 0 */

long max_vectors_allowed;       /* max map vectors allowed */
long max_text_labels_allowed;   /* max map text labels allowed */
long max_symbol_labels_allowed; /* max map symbol labels allowed */

int map_color_levels;           /* toggle use of map_color_levels */
int map_auto_maps;              /* toggle use of auto_maps */

int long_lat_grid;              /* toggle use of map lat and long grid */

time_t net_last_time;           /* reconnect last time in seconds */
time_t net_next_time;           /* reconnect Next update delay time */

time_t last_time;        /* last time in seconds */
time_t next_time;        /* Next update delay time */

time_t next_redraw;      /* Next update time */

char aprs_station_message_type; /* aprs station message type message capable or not */

int transmit_now;               /* set to transmit now (push on moment) */
int operate_as_an_igate;        /* toggle igate operations for net connections */
unsigned igate_msgs_tx;         /* current total of igate messages transmitted */
int log_igate;                  /* toggle to allow igate logging */
int log_wx;                     /* toggle to allow wx logging */

int show_phg;                   /* toggle to show power hight gain circles */

int station_trails;

int WX_ALERTS_REFRESH_TIME;     /* Minimum WX alert map refresh time in seconds */ 

/* button zoom */
int menu_x;
int menu_y;

/* station old after */
time_t sec_old;

/* station cleared after */
time_t sec_clear;

/* gps time out */
time_t GPS_time;

/* file showing */
int read_file;
FILE *read_file_ptr;
int read_file_delay;
int read_file_delay_count;

/* My Info */
char my_callsign[MAX_CALL+1];
char my_lat[MAX_LAT];
char my_long[MAX_LONG];
char my_group;
char my_symbol;
char my_phg[MAX_PHG+1];
char my_comment[MAX_COMMENT+1];
int my_last_course;
int my_last_speed;
long my_last_altitude;
time_t my_last_altitude_time;

/* from symbol.c */
extern void symbol(Widget w, int ghost,char symbol_table, char symbol_id, char symbol_overlay, Pixmap where, int mask, long x_offset, long y_offset);

/* from interface.c */
extern iface port_data[];                   /* shared port data */
extern pthread_mutex_t port_data_lock;      /* Lock for the port data */
extern int port_id[];                       /* shared port id data */
extern pthread_mutex_t output_data_lock;    /* Lock for the data output*/  
extern pthread_mutex_t data_lock;
extern void clear_all_port_data(void);
extern void init_device_names(void);
extern void startup_all_or_defined_port(int port);
extern void shutdown_all_active_or_defined_port(int port);
extern void check_ports(void);

/* from interface_gui.c */
extern void interface_status(Widget w);

/* gps.c */
extern void gps_data_find(char *gps_line_data, int port);
extern int gps_stop_now;


/* from db.c */
extern void packet_data_add(char *from, char *line);
extern void General_query(Widget w, XtPointer clientData, XtPointer calldata);
extern void IGate_query(Widget w, XtPointer clientData, XtPointer calldata);
extern void WX_query(Widget w, XtPointer clientData, XtPointer calldata);
extern int max_stations;

/* Symbols */
SymbolData symbol_data[MAX_SYMBOLS];
int symbols_loaded;

/* sound run */
pid_t last_sound_pid;

/* Default directories */

char AUTO_MAP_DIR[400];
char ALERT_MAP_DIR[400];
char WIN_MAP_DIR[400];
char WIN_MAP_DATA[400];
char SYMBOLS_DIR[400];
char HELP_FILE[400];
char SOUND_DIR[400];

char MDATABASE_FILE[400];
char MDATABASE_POINTER_FILE[400];
char MDATABASE_POINTER_TEMP[400];

char LOGFILE_TNC[400];
char LOGFILE_NET[400];
char LOGFILE_IGATE[400];
char LOGFILE_WX[400];

/* sound data */
char sound_command[90];
int sound_play_new_station;
char sound_new_station[90];
int sound_play_new_message;
char sound_new_message[90];

int sound_play_prox_message;
char sound_prox_message[90];
char prox_min[30];
char prox_max[30];
int sound_play_band_open_message;
char sound_band_open_message[90];
char bando_min[30];
char bando_max[30];
int sound_play_wx_alert_message;
char sound_wx_alert_message[90];

#define LINE_WIDTH 1
static int      input_x = 0;
static int      input_y = 0;

XtAppContext app_context;
Display *display;       /*  Display             */

/* dialog popup last */
int last_popup_x;
int last_popup_y;

void HandlePendingEvents( XtAppContext app)
{XEvent event;

while(XtAppPending(app))
     {XtAppNextEvent(app,&event);
      XtDispatchEvent(&event);
     }
}

Boolean unbusy_cursor(XtPointer clientdata)
{Widget w = (Widget)clientdata;
XUndefineCursor(XtDisplay(w),XtWindow(w));
return(TRUE);
}

void busy_cursor(Widget w)
{static Cursor cs = (Cursor)NULL;
if(!cs)
  {cs=XCreateFontCursor(XtDisplay(w),XC_watch);}
XDefineCursor(XtDisplay(w),XtWindow(w),cs);
XFlush(XtDisplay(w));
XtAppAddWorkProc(XtWidgetToApplicationContext(w),unbusy_cursor,(XtPointer)w);
}


int filethere(char *fn)
{FILE *f;
 int ret;

ret =0;
f=fopen(fn,"r");
if(f != NULL)
  {ret=1;
   fclose(f);
  }
return(ret);
}

time_t file_time(char *fn)
{struct stat nfile;
 
 if(stat(fn,&nfile)==0)
   {return(nfile.st_ctime);}

return(-1);
}

time_t sec_now(void)
{
time_t timenw;
time_t ret;

ret = time(&timenw);
return(ret);
}

void log_data(char *file, char *line)
{FILE *f;

 f=fopen(file,"a");
 if (f!=NULL) {
    fprintf(f,"%s\n",line);
    fclose(f);
 }
  
}



/*** THESE WORK WORLD WIDE *****/

char *convert_long_lat_string(long lat, char *str)
{char ns;
 long computed;
 long deg;
 float min;

strcpy(str,"");
if(lat<=32400000l)
  {/* lat is North */
   ns='N';
   computed=32400000l-lat;
  }
  else
  {/* lat is Sorth */
   ns='S';
   computed=lat-32400000l;
  }
deg=computed/360000;
min = (float)(computed-(deg*360000))/6000.0;
sprintf(str,"%02ld %05.2f%c",deg,min,ns);

return(str);
}


char *convert_long_lat_string_hp(long lat, char *str)
{char ns;
 long computed;
 long deg;
 float min;

strcpy(str,"");
if(lat<=32400000l)
  {/* lat is North */
   ns='N';
   computed=32400000l-lat;
  }
  else
  {/* lat is Sorth */
   ns='S';
   computed=lat-32400000l;
  }
deg=computed/360000;
min = (float)(computed-(deg*360000))/6000.0;
sprintf(str,"%02ld %06.3f%c",deg,min,ns);

return(str);
}


char *convert_long_long_string(long lon, char *str)
{char ew;
 long computed;
 long deg;
 float min;

strcpy(str,"");
if(lon<=64800000l)
  {/* long is North */
   ew='W';
   computed=64800000l-lon;
  }
  else
  {/* long is East */
   ew='E';
   computed=lon-64800000l;
  }
deg=computed/360000;
min = (float)(computed-(deg*360000))/6000.0;
sprintf(str,"%03ld %05.2f%c",deg,min,ew);

return(str);
}

char *convert_long_long_string_hp(long lon, char *str)
{char ew;
 long computed;
 long deg;
 float min;

strcpy(str,"");
if(lon<=64800000l)
  {/* long is North */
   ew='W';
   computed=64800000l-lon;
  }
  else
  {/* long is East */
   ew='E';
   computed=lon-64800000l;
  }
deg=computed/360000;
min = (float)(computed-(deg*360000))/6000.0;
sprintf(str,"%03ld %06.3f%c",deg,min,ew);

return(str);
}

long convert_lat_string(char *lat)
{long ret;

 float degrees;
 float min;
 float t;
 float b;

 char d[4];
 char m[8];

 ret=0l;
 t=90.0;
 b=0.0;
 if(lat[4]=='.' && (toupper(lat[7])=='N' || toupper(lat[8])=='N'))
   {strncpy(d,lat,2);
    d[2]=0;
    if (toupper(lat[7])=='N') {
       strncpy(m,lat+2,5);
       m[5]=0;
    } else {
       /* do high precision */
       strncpy(m,lat+2,6);
       m[6]=0;
    }
    degrees=atof(d);
    min=atof(m);
    t=t-degrees;
    if(min>0.0)
      {b=60.0;
       t=t-1;
       b=b-min;
      }
   
   ret=(long)(t*360000.0);

   ret=ret+(long)(b*6000.0);
   }
   else
   {if(lat[4]=='.' && (toupper(lat[7])=='S' || toupper(lat[8])=='S'))
      {strncpy(d,lat,2);
       d[2]=0;
       if (toupper(lat[7])=='S') {
          strncpy(m,lat+2,5);
          m[5]=0;
       } else {
          /* do high precision */  
          strncpy(m,lat+2,6);
          m[6]=0;
       }

       degrees=atof(d);
       min=atof(m);
       t=t+degrees;
       if(min>0.0)
         {b=min;}
       ret=(long)(t*360000.0);

      ret=ret+(long)(b*6000.0);
      }

   }

return(ret);
}

long convert_long_string(char *lng)
{

long ret;

 float degrees;
 float min;
 float t;
 float b;

 char d[4];
 char m[8];

 ret=0l;
 t=180.0;
 b=0.0;
 if(lng[5]=='.' && (toupper(lng[8])=='W' || toupper(lng[9])=='W'))
   {strncpy(d,lng,3);
    d[3]=0;
    if (toupper(lng[8])=='W') {
       strncpy(m,lng+3,5);
       m[5]=0;
    } else {
       /* do high precision */  
       strncpy(m,lng+3,6);
       m[6]=0;
    }

    degrees=atof(d);
    min=atof(m);
    t=t-degrees;
    if(min>0.0)
      {b=60.0;
       t=t-1;
       b=b-min;
      }
   ret=(long)(t*360000.0);

   ret=ret+(long)(b*6000.0);

   }
   else
   {if(lng[5]=='.' && (toupper(lng[8])=='E' || toupper(lng[9])=='E'))
      {strncpy(d,lng,3);
       d[3]=0;
       if (toupper(lng[8])=='E') {
          strncpy(m,lng+3,5);
          m[5]=0;
       } else {
          /* do high precision */   
          strncpy(m,lng+3,6);
          m[6]=0;
       }

       degrees=atof(d);
       min=atof(m);
       t=t+degrees;
       if(min>0.0)
         {b=min;
	 }
       ret=(long)(t*360000.0);

       ret=ret+(long)(b*6000.0);

      }
   }
return(ret);
}


double convert_lat_string_to_angle_radians(char *lat)
{float ret;

 double degrees;
 double min;

 char d[4];
 char m[8];

 ret=0.0;

 if(lat[4]=='.' && toupper(lat[7])=='N')
   {strncpy(d,lat,2);
    d[2]=0;
    strncpy(m,lat+2,5);
    m[5]=0;

    degrees=atof(d);
    min=atof(m);
    ret=(degrees+(min/60.0))*(PI/180.0);

   }
   else
   {if(lat[4]=='.' && toupper(lat[7])=='S')
      {strncpy(d,lat,2);
       d[2]=0;
       strncpy(m,lat+2,5);
       m[5]=0;

       degrees=(-1)*atof(d);
       min=(-1)*atof(m);
       ret=(degrees+(min/60.0))*(PI/180.0);
      }

   }

return(ret);
}

double convert_long_string_to_angle_radians(char *lng)
{double ret;

 double degrees;
 double min;

 char d[4];
 char m[8];

 ret=0.0;

 if(lng[5]=='.' && toupper(lng[8])=='W')
   {strncpy(d,lng,3);
    d[3]=0;
    strncpy(m,lng+3,5);
    m[5]=0;

    degrees=atof(d);
    min=atof(m);

    ret=(degrees+(min/60.0))*(PI/180.0);
   }
   else
   {if(lng[5]=='.' && toupper(lng[8])=='E')
      {strncpy(d,lng,3);
       d[3]=0;
       strncpy(m,lng+3,5);
       m[5]=0;

       degrees=(-1)*atof(d);
       min=(-1)*atof(m);

      ret=(degrees+(min/60.0))*(PI/180.0);

      }
   }
return(ret);
}


double distance_between_string(char *lat_point1, char *long_point1, char *lat_point2, char *long_point2, char *course_deg)
{float ret;

 double r_lat1,r_lat2;
 double r_long1,r_long2;
 double r_d,r_c;

  r_lat1 = convert_lat_string_to_angle_radians(lat_point1);
  r_lat2 = convert_lat_string_to_angle_radians(lat_point2);
  r_long1 = convert_long_string_to_angle_radians(long_point1);
  r_long2 = convert_long_string_to_angle_radians(long_point2);
  r_d = acos(sin(r_lat1) * sin(r_lat2) + cos(r_lat1) * cos(r_lat2) * cos(r_long1-r_long2));

  if(cos(r_lat1) < 0.0000000001)
    {if(r_lat1>0.0)
       {r_c=PI;}
       else
       {r_c=0.0;}
    }
    else
    {
     if(sin((r_long2-r_long1))<0.0)
       {r_c = acos((sin(r_lat2)-sin(r_lat1)*cos(r_d))/(sin(r_d)*cos(r_lat1)));}
       else
       {r_c = (2*PI) - acos((sin(r_lat2)-sin(r_lat1)*cos(r_d))/(sin(r_d)*cos(r_lat1)));}

    }


  sprintf(course_deg,"%.1f",(180.0/PI)*r_c);

  ret = r_d*180*60/PI;
return(ret);
}

/*** end these work world wide **/



void create_image(Widget w)
{
 Dimension       width, height, margin_width, margin_height;
 char str_lat[20];
 char str_long[20];
 long lat_offset_temp;
 long long_offset_temp;
 char temp1[20];
 char temp2[20];
 char temp3[20];
 char temp_course[20];
 unsigned char   unit_type;
 unsigned char medium_dashed[2] = {5,5};

 busy_cursor(w);

 if (debug_level & 4)
    printf("Create image start\n");

 /* First get the various dimensions */
 XtVaGetValues(w,
              XmNwidth, &width,
	      XmNheight, &height,
	      XmNmarginWidth, &margin_width,
	      XmNmarginHeight, &margin_height,
	      XmNunitType, &unit_type,
	      0);


 XSetDashes(XtDisplay(w), gc, 0, medium_dashed , 2);

 screen_width=width;
 screen_height=height;

 long_offset_temp = x_long_offset=mid_x_long_offset - (screen_width*size/2);
 y_lat_offset=mid_y_lat_offset - (screen_height*size/2);
 lat_offset_temp=mid_y_lat_offset;

 /* map default background color */ 
 switch(map_background_color){
      case 0 :
        colors[0xfd] = GetPixelByName(appshell,"gray73");      
              break;
      case 1 :
        colors[0xfd] = GetPixelByName(w,"MistyRose");   
              break;
      case 2 :
        colors[0xfd] = GetPixelByName(w,"NavyBlue");   
              break;
      case 3 :
        colors[0xfd] = GetPixelByName(w,"SteelBlue");   
              break;
      case 4 :
        colors[0xfd] = GetPixelByName(w,"MediumSeaGreen");   
              break;
      case 5 :
        colors[0xfd] = GetPixelByName(w,"PaleGreen");   
              break;
      case 6 :
        colors[0xfd] = GetPixelByName(w,"PaleGoldenrod");   
              break;
      case 7 :
        colors[0xfd] = GetPixelByName(w,"LightGoldenrodYellow");   
              break;
      case 8 :
        colors[0xfd] = GetPixelByName(w,"RosyBrown");   
              break;
      case 9 :
        colors[0xfd] = GetPixelByName(w,"firebrick");   
              break;
      case 10 :
        colors[0xfd] = GetPixelByName(w,"white");      
              break;

      default:
        colors[0xfd] = GetPixelByName(appshell,"gray73");
        map_background_color=0;
              break;
 }

 current_trail_color = 0x01;    /* Have to start somewhere... */
 max_trail_colors = 32;       

 XSetForeground(XtDisplay(w),gc,colors[0xfd]); 
 XSetBackground(XtDisplay(w),gc,colors[0xfd]); 

 XFillRectangle(XtDisplay(w), pixmap,gc,0,0,screen_width,screen_height);

 XmTextFieldSetString(text,langcode("BBARSTA003"));
 
 if (display_up_first!=0) {
    
    if (wx_alert_style) 
       load_alert_maps(w, ALERT_MAP_DIR);

    if(map_auto_maps)
      load_auto_maps(w,AUTO_MAP_DIR);
     else
      load_maps(w); 

    if (!wx_alert_style) 
       load_alert_maps(w, ALERT_MAP_DIR);

    wx_alert_update_list();
 }

 XmTextFieldSetString(text,langcode("BBARSTA004"));

 /* copy over map to final pixmap */
 XCopyArea(XtDisplay(w),pixmap,pixmap_final,gc,0,0,screen_width,screen_height,0,0);

 /* Compute distance */
 if(lat_offset_temp<0l)
   {lat_offset_temp=0l;}
   else
   {if(lat_offset_temp>64800000l)
     {lat_offset_temp=64800000l;}
   }

 if(long_offset_temp<0l)
   {long_offset_temp=0l;}
   else
   {if(long_offset_temp>129600000l)
      {long_offset_temp=129600000l;}
   }

 convert_long_lat_string(lat_offset_temp,str_lat);
 str_lat[2]=0;
 sprintf(temp1,"%s%s",str_lat,str_lat+3);

 convert_long_long_string(long_offset_temp,str_long);
 str_long[3]=0;
 sprintf(temp2,"%s%s",str_long,str_long+4);

 long_offset_temp = long_offset_temp+(50*size);

 if(lat_offset_temp<0l)
   {lat_offset_temp=0l;}
   else
   {if(lat_offset_temp>64799999l)
     {lat_offset_temp=64799999l;}
   }

 if(long_offset_temp<0l)
   {long_offset_temp=0l;}
   else
   {if(long_offset_temp>129599999l)
      {long_offset_temp=129599999l;}
   }

 convert_long_long_string(long_offset_temp,str_long);
 str_long[3]=0;
 sprintf(temp3,"%s%s",str_long,str_long+4);

 x_screen_distance= distance_between_string(temp1,temp2,temp1,temp3,temp_course);


 /* display icons */
 display_file(w);

 /* set last time of screen redraw*/
 last_alert_redraw=sec_now();

 if (debug_level & 4)
    printf("Create image stop\n");
}

void redraw_symbols(Widget w)
{
/* copy over map to final pixmap */
if(!wait_to_redraw) 
  {XCopyArea(XtDisplay(w),pixmap,pixmap_final,gc,0,0,screen_width,screen_height,0,0);
   /*New station*/
   display_file(w);
   XCopyArea(XtDisplay(w),pixmap_final,XtWindow(w),gc,0,0,screen_width,screen_height,0,0);
  }
}

static void TrackMouse(Widget w, XtPointer clientData, XEvent *event, Boolean *flag)
{char text[100];
 char str_lat[20];
 char str_long[20];
 long x,y;

 Widget textarea = (Widget) clientData;

 x=(mid_x_long_offset - ((screen_width*size)/2) + (event->xmotion.x*size));
 y=(mid_y_lat_offset - ((screen_height*size)/2) + (event->xmotion.y*size));
 if(x<0)
   {x=0l;}
 if(x>129600000l)
   {x=129600000l;}
 if(y<0)
   {y=0l;}
 if(y>64800000l)
   {y=64800000l;}
 
 convert_long_lat_string_hp(y,str_lat);
 convert_long_long_string_hp(x,str_long);

 sprintf(text,"%s %s",str_lat,str_long);
 XmTextFieldSetString(textarea,text);
 XtManageChild(textarea);  
}

static void ClearTrackMouse(Widget w, XtPointer clientData, XEvent *event, Boolean *flag)
{
 Widget textarea = (Widget) clientData;
 XmTextFieldSetString(textarea," ");
 XtManageChild(textarea);  
}

void create_appshell (Display *display, char *app_name, int app_argc, char ** app_argv)
{
        Atom WM_DELETE_WINDOW;
	Widget children[8];      /* Children to manage */
	Arg al[64];                    /* Arg List */
	register int ac = 0;           /* Arg Count */
        static char appLabel[81];
        
char zoom[30];
/*popup menu widgets */
Widget zoom_in, zoom_out, zoom_sub, zoom_level, zl1, zl2, zl3, zl4, zl5, zl6, pan_up, pan_down, pan_left, pan_right, last_loc, station_info;

/*menu widgets */
Widget sep;
Widget filepane, configpane, mappane, viewpane, displaypane, messagepane, ifacepane, qpane, helppane;
Widget station_clear_button, tracks_clear_button,
       save_button,file_button, open_file_button, exit_button,
       view_button, bullet_button, packet_data_button, mobile_button, stations_button,  localstations_button, 
       weather_button, wx_station_button, locate_button, jump_button, alert_button,
       config_button, defaults_button, station_button,
       map_button, Option_Map_auto_Pane, map_auto_button, map_chooser_button, map_grid_button, Option_Grid_Pane, map_levels_button, Option_Map_Levels_Pane,
       Map_background_color_Pane, map_background_button, 
       Map_station_label_Pane, map_station_label_button,
       Map_wx_alerts_Pane, map_wx_alerts_button,
       display_button, alt_pane, alt_button, course_pane, course_button, speed_pane, speed_button, dist_course_pane, dist_course_button,  units_choice_pane, units_choice_button,  
       device_config_button, 
       iface_button, iface_connect_button, tnc_logging_pane, tnc_logging, iface_transmit_now,
       net_logging_pane, net_logging,
       igate_logging_pane, igate_logging,
       wx_logging_pane, wx_logging,
       test_button, aa_button, 
       auto_msg_pane, auto_msg_button, auto_msg_set_button,
       message_button, send_message_to_button, open_messages_group_button, clear_messages_button,
       station_trails_pane, station_trails_button,
       station_phg_pane, station_phg_button,
       wx_pane, wx_button,
       track_button,
       queries_button, General_q_button, IGate_q_button, WX_q_button, 
       help_button, help_about, help_help;
 
       if(debug_level & 8)
         {printf("Create appshell start\n");}

        appLabel[80] = '\0';
        strncpy(appLabel, VERSIONLABEL, 79);
        strcat(appLabel, " ");
        gethostname(&appLabel[strlen(appLabel)], 80-strlen(appLabel));

        colors[0xff] = GetPixelByName(hidden_shell,"gray73");   

        XtSetArg(al[ac], XmNallowShellResize, TRUE); ac++;
	XtSetArg(al[ac], XmNtitle, appLabel); ac++;
	XtSetArg(al[ac], XmNargv, app_argv); ac++;
        XtSetArg(al[ac], XmNminWidth, 640); ac++;
        XtSetArg(al[ac], XmNminHeight,380); ac++;
        XtSetArg(al[ac], XmNdefaultPosition, False); ac++;
       
       appshell= XtCreatePopupShell (app_name,topLevelShellWidgetClass,hidden_shell, al, ac);
       
       form = XtVaCreateManagedWidget("Form",xmFormWidgetClass,appshell,
                                      XmNbackground, colors[0xff],
                                      NULL);

       /* Menu Bar */
       ac=0;
       XtSetArg(al[ac],XmNshadowThickness,1); ac++;
       XtSetArg(al[ac],XmNalignment,XmALIGNMENT_BEGINNING); ac++;
       XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM); ac++;
       XtSetArg(al[ac],XmNtopAttachment,XmATTACH_FORM); ac++;
       XtSetArg(al[ac],XmNrightAttachment,XmATTACH_FORM); ac++;
       XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_NONE); ac++;
       XtSetArg(al[ac],XmNbackground, colors[0xff]); ac++;      
       menubar = XmCreateMenuBar(form,"menubar",al,ac);

       /*set args for color */ 
       ac=0;
       XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      

       filepane = XmCreatePulldownMenu(menubar,"filepane",al,ac);
       viewpane = XmCreatePulldownMenu(menubar,"viewpane",al, ac); 
       configpane = XmCreatePulldownMenu(menubar,"configpane",al, ac);
       mappane = XmCreatePulldownMenu(menubar,"mappane",al, ac);
       displaypane = XmCreatePulldownMenu(menubar,"displaymap",al, ac);
       messagepane = XmCreatePulldownMenu(menubar,"messagepane",al, ac);
       ifacepane = XmCreatePulldownMenu(menubar,"ifacepane",al, ac);
       qpane = XmCreatePulldownMenu(menubar,"qpane",al, ac);
       helppane = XmCreatePulldownMenu(menubar,"helppane",al, ac);

       

       /* menu bar */
       file_button = XtVaCreateManagedWidget(langcode("MENUTB0001"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId, filepane,XmNmnemonic,langcode_hotkey("MENUTB0001"),XmNbackground,colors[0xff],NULL);
       view_button = XtVaCreateManagedWidget(langcode("MENUTB0002"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,viewpane,XmNmnemonic,langcode_hotkey("MENUTB0002"),XmNbackground,colors[0xff],NULL);
       config_button = XtVaCreateManagedWidget(langcode("MENUTB0003"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,configpane,XmNmnemonic,langcode_hotkey("MENUTB0003"),XmNbackground,colors[0xff],NULL);
       map_button = XtVaCreateManagedWidget(langcode("MENUTB0004"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,mappane,XmNmnemonic,langcode_hotkey("MENUTB0004"),XmNbackground,colors[0xff],NULL);
       display_button = XtVaCreateManagedWidget(langcode("MENUTB0005"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,displaypane,XmNmnemonic,langcode_hotkey("MENUTB0005"),XmNbackground,colors[0xff],NULL);
       message_button = XtVaCreateManagedWidget(langcode("MENUTB0006"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,messagepane,XmNmnemonic,langcode_hotkey("MENUTB0006"),XmNbackground,colors[0xff],NULL);
       iface_button = XtVaCreateManagedWidget(langcode("MENUTB0010"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,ifacepane,XmNmnemonic,langcode_hotkey("MENUTB0010"),XmNbackground,colors[0xff],NULL);
       queries_button = XtVaCreateManagedWidget(langcode("MENUTB0011"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,qpane,XmNmnemonic,langcode_hotkey("MENUTB0011"),XmNbackground,colors[0xff],NULL);
       help_button = XtVaCreateManagedWidget(langcode("MENUTB0009"),xmCascadeButtonGadgetClass,menubar,XmNsubMenuId,helppane,XmNmnemonic,langcode_hotkey("MENUTB0009"),XmNbackground,colors[0xff],NULL);
       XtVaSetValues (menubar,XmNmenuHelpWidget,help_button,NULL);
       /* end bar */


       /* Queries */ 
       General_q_button = XtVaCreateManagedWidget(langcode("PULDQUS001"),xmPushButtonGadgetClass,qpane,XmNmnemonic,langcode_hotkey("PULDQUS001"),XmNbackground,colors[0xff],NULL);
       IGate_q_button = XtVaCreateManagedWidget(langcode("PULDQUS002"),xmPushButtonGadgetClass,qpane,XmNmnemonic,langcode_hotkey("PULDQUS002"),XmNbackground,colors[0xff],NULL);
       WX_q_button = XtVaCreateManagedWidget(langcode("PULDQUS003"),xmPushButtonGadgetClass,qpane,XmNmnemonic,langcode_hotkey("PULDQUS003"),XmNbackground,colors[0xff],NULL);

       /* File */ 
       station_clear_button = XtVaCreateManagedWidget(langcode("PULDNFI001"),xmPushButtonGadgetClass,filepane,XmNmnemonic,langcode_hotkey("PULDNFI001"),XmNbackground,colors[0xff],NULL);
       tracks_clear_button = XtVaCreateManagedWidget(langcode("PULDNFI005"),xmPushButtonGadgetClass,filepane,XmNbackground,colors[0xff],NULL);
       open_file_button = XtVaCreateManagedWidget(langcode("PULDNFI002"),xmPushButtonGadgetClass,filepane,XmNmnemonic,langcode_hotkey("PULDNFI002"),XmNbackground,colors[0xff],NULL);
       save_button = XtVaCreateManagedWidget(langcode("PULDNFI006"),xmPushButtonGadgetClass,filepane,XmNmnemonic,langcode_hotkey("PULDNFI006"),XmNbackground,colors[0xff],NULL);
       XtVaCreateManagedWidget("sep1",xmSeparatorGadgetClass,filepane,XmNbackground,colors[0xff],NULL);
       test_button = XtVaCreateManagedWidget(langcode("PULDNFI003"),xmPushButtonWidgetClass,filepane,XmNmnemonic,langcode_hotkey("PULDNFI003"),XmNbackground,colors[0xff],NULL);
       exit_button = XtVaCreateManagedWidget(langcode("PULDNFI004"),xmPushButtonWidgetClass,filepane,XmNmnemonic,langcode_hotkey("PULDNFI004"),XmNbackground,colors[0xff],NULL);
       
       /*XtSetSensitive(test_button,False);*/

       /* View */
       bullet_button = XtVaCreateManagedWidget(langcode("PULDNVI001"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI001"),XmNbackground,colors[0xff],NULL);
       packet_data_button = XtVaCreateManagedWidget(langcode("PULDNVI002"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI002"),XmNbackground,colors[0xff],NULL);
       mobile_button = XtVaCreateManagedWidget(langcode("PULDNVI003"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI003"),XmNbackground,colors[0xff],NULL);
       stations_button = XtVaCreateManagedWidget(langcode("PULDNVI004"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI004"),XmNbackground,colors[0xff],NULL);
       localstations_button = XtVaCreateManagedWidget(langcode("PULDNVI009"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI009"),XmNbackground,colors[0xff],NULL);
       weather_button = XtVaCreateManagedWidget(langcode("PULDNVI005"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI005"),XmNbackground,colors[0xff],NULL);
       wx_station_button = XtVaCreateManagedWidget(langcode("PULDNVI008"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI008"),XmNbackground,colors[0xff],NULL);
       alert_button = XtVaCreateManagedWidget(langcode("PULDNVI007"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI007"),XmNbackground,colors[0xff],NULL);
       locate_button = XtVaCreateManagedWidget(langcode("PULDNVI006"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI006"),XmNbackground,colors[0xff],NULL);
       jump_button = XtVaCreateManagedWidget(langcode("PULDNVI010"),xmPushButtonGadgetClass,viewpane,XmNmnemonic,langcode_hotkey("PULDNVI010"),XmNbackground,colors[0xff],NULL);

       XtSetSensitive(bullet_button,False);

       /* Configure */
       defaults_button = XtVaCreateManagedWidget(langcode("PULDNCF001"),xmPushButtonGadgetClass,configpane,XmNmnemonic,langcode_hotkey("PULDNCF001"),XmNbackground,colors[0xff],NULL);
       device_config_button = XtVaCreateManagedWidget(langcode("PULDNCF008"),xmPushButtonGadgetClass,configpane,XmNmnemonic,langcode_hotkey("PULDNCF008"),XmNbackground,colors[0xff],NULL);
       station_button = XtVaCreateManagedWidget(langcode("PULDNCF004"),xmPushButtonGadgetClass,configpane,XmNmnemonic,langcode_hotkey("PULDNCF004"),XmNbackground,colors[0xff],NULL);
       aa_button = XtVaCreateManagedWidget(langcode("PULDNCF006"),xmPushButtonGadgetClass,configpane,XmNmnemonic,langcode_hotkey("PULDNCF006"),XmNbackground,colors[0xff],NULL);
       

       units_choice_pane = XmCreatePulldownMenu(configpane,"units_choice",al,ac);
       units_choice_button = XtVaCreateManagedWidget(langcode("PULDNDP006"),xmCascadeButtonWidgetClass,configpane,XmNsubMenuId,units_choice_pane,XmNmnemonic,langcode_hotkey("PULDNDP006"),XmNbackground,colors[0xff],NULL); 
       units_choice_on = XtVaCreateManagedWidget(langcode("PULDNUT001"),xmPushButtonGadgetClass,units_choice_pane,XmNbackground,colors[0xff],NULL);
       units_choice_off = XtVaCreateManagedWidget(langcode("PULDNUT002"),xmPushButtonGadgetClass,units_choice_pane,XmNbackground,colors[0xff],NULL);


       /* Maps */
       map_chooser_button = XtVaCreateManagedWidget(langcode("PULDNMP001"),xmPushButtonGadgetClass,mappane,XmNmnemonic,langcode_hotkey("PULDNMP001"),XmNbackground,colors[0xff],NULL);
       Option_Map_auto_Pane = XmCreatePulldownMenu(mappane,"auto_map_levels",al,ac);
       map_auto_button = XtVaCreateManagedWidget(langcode("PULDNMP002"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId,Option_Map_auto_Pane,XmNmnemonic,langcode_hotkey("PULDNMP002"),XmNbackground,colors[0xff],NULL);
       map_auto_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,Option_Map_auto_Pane,XmNbackground,colors[0xff],NULL);
       map_auto_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,Option_Map_auto_Pane,XmNbackground,colors[0xff],NULL);


       XtVaCreateManagedWidget("sep2",xmSeparatorGadgetClass,mappane,XmNbackground,colors[0xff],NULL);
       Option_Grid_Pane = XmCreatePulldownMenu(mappane,"Option_grid",al,ac);
       map_grid_button = XtVaCreateManagedWidget(langcode("PULDNMP003"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId, Option_Grid_Pane,XmNmnemonic,langcode_hotkey("PULDNMP003"),XmNbackground,colors[0xff],NULL);
       grid_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,Option_Grid_Pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       grid_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,Option_Grid_Pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);

       Option_Map_Levels_Pane = XmCreatePulldownMenu(mappane,"Option_map_levels",al,ac);
       map_levels_button = XtVaCreateManagedWidget(langcode("PULDNMP004"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId, Option_Map_Levels_Pane,XmNmnemonic,langcode_hotkey("PULDNMP004"),XmNbackground,colors[0xff],NULL);
       map_levels_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,Option_Map_Levels_Pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       map_levels_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,Option_Map_Levels_Pane,XmNmnemonic,langcode_hotkey("PULDNOT002"),XmNbackground,colors[0xff],NULL);

       Map_background_color_Pane = XmCreatePulldownMenu(mappane,"map_background_color",al,ac);
       map_background_button = XtVaCreateManagedWidget(langcode("PULDNMP005"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId, Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMP005"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[10] = XtVaCreateManagedWidget(langcode("PULDNMBC11"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC11"),XmNbackground,colors[0xff],NULL);      
       map_bgcolor[0] = XtVaCreateManagedWidget(langcode("PULDNMBC01"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC01"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[1] = XtVaCreateManagedWidget(langcode("PULDNMBC02"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC02"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[2] = XtVaCreateManagedWidget(langcode("PULDNMBC03"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC03"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[3] = XtVaCreateManagedWidget(langcode("PULDNMBC04"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC04"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[4] = XtVaCreateManagedWidget(langcode("PULDNMBC05"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC05"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[5] = XtVaCreateManagedWidget(langcode("PULDNMBC06"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC06"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[6] = XtVaCreateManagedWidget(langcode("PULDNMBC07"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC07"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[7] = XtVaCreateManagedWidget(langcode("PULDNMBC08"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC08"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[8] = XtVaCreateManagedWidget(langcode("PULDNMBC09"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC09"),XmNbackground,colors[0xff],NULL);
       map_bgcolor[9] = XtVaCreateManagedWidget(langcode("PULDNMBC10"),xmPushButtonGadgetClass,Map_background_color_Pane,XmNmnemonic,langcode_hotkey("PULDNMBC10"),XmNbackground,colors[0xff],NULL);

       Map_station_label_Pane = XmCreatePulldownMenu(mappane,"map_station_label",al,ac);
       map_station_label_button = XtVaCreateManagedWidget(langcode("PULDNMP006"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId, Map_station_label_Pane,XmNmnemonic,langcode_hotkey("PULDNMP006"),XmNbackground,colors[0xff],NULL);
       map_station_label0 = XtVaCreateManagedWidget(langcode("PULDNMSL01"),xmPushButtonGadgetClass,Map_station_label_Pane,XmNmnemonic,langcode_hotkey("PULDNSLC01"),XmNbackground,colors[0xff],NULL);
       map_station_label1 = XtVaCreateManagedWidget(langcode("PULDNMSL02"),xmPushButtonGadgetClass,Map_station_label_Pane,XmNmnemonic,langcode_hotkey("PULDNMSL02"),XmNbackground,colors[0xff],NULL);

       Map_wx_alerts_Pane = XmCreatePulldownMenu(mappane,"map_wx_alerts",al,ac);
       map_wx_alerts_button = XtVaCreateManagedWidget(langcode("PULDNMP007"),xmCascadeButtonWidgetClass,mappane,XmNsubMenuId, Map_wx_alerts_Pane,XmNmnemonic,langcode_hotkey("PULDNMP007"),XmNbackground,colors[0xff],NULL);
       map_wx_alerts_0 = XtVaCreateManagedWidget(langcode("PULDNMAT01"),xmPushButtonGadgetClass,Map_wx_alerts_Pane,XmNmnemonic,langcode_hotkey("PULDNMAT01"),XmNbackground,colors[0xff],NULL);
       map_wx_alerts_1 = XtVaCreateManagedWidget(langcode("PULDNMAT02"),xmPushButtonGadgetClass,Map_wx_alerts_Pane,XmNmnemonic,langcode_hotkey("PULDNMAT02"),XmNbackground,colors[0xff],NULL);

       /* Display */
       alt_pane = XmCreatePulldownMenu(displaypane,"Altitudepane",al,ac);
       track_button = XtVaCreateManagedWidget(langcode("PULDNDP001"),xmPushButtonGadgetClass,displaypane,XmNmnemonic,langcode_hotkey("PULDNDP001"),XmNbackground,colors[0xff],NULL);

       alt_button = XtVaCreateManagedWidget(langcode("PULDNDP002"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,alt_pane,XmNmnemonic,langcode_hotkey("PULDNDP002"),XmNbackground,colors[0xff],NULL);
       alt_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,alt_pane,XmNbackground,colors[0xff],NULL);
       alt_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,alt_pane,XmNbackground,colors[0xff],NULL);

       course_pane = XmCreatePulldownMenu(displaypane,"Coursepane",al,ac);
       course_button = XtVaCreateManagedWidget(langcode("PULDNDP003"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,course_pane,XmNmnemonic,langcode_hotkey("PULDNDP003"),XmNbackground,colors[0xff],NULL); 
       course_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,course_pane,XmNbackground,colors[0xff],NULL);
       course_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,course_pane,XmNbackground,colors[0xff],NULL);

       speed_pane = XmCreatePulldownMenu(displaypane,"speedpane",al,ac);
       speed_button = XtVaCreateManagedWidget(langcode("PULDNDP004"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,speed_pane,XmNmnemonic,langcode_hotkey("PULDNDP004"),XmNbackground,colors[0xff],NULL); 
       speed_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,speed_pane,XmNbackground,colors[0xff],NULL);
       speed_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,speed_pane,XmNbackground,colors[0xff],NULL);

       dist_course_pane = XmCreatePulldownMenu(displaypane,"my_dist_coursepane",al,ac);
       dist_course_button = XtVaCreateManagedWidget(langcode("PULDNDP005"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,dist_course_pane,XmNmnemonic,langcode_hotkey("PULDNDP005"),XmNbackground,colors[0xff],NULL);
       dist_course_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,dist_course_pane,XmNbackground,colors[0xff],NULL);
       dist_course_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,dist_course_pane,XmNbackground,colors[0xff],NULL);

       station_trails_pane = XmCreatePulldownMenu(displaypane,"station_trails",al,ac);
       station_trails_button = XtVaCreateManagedWidget(langcode("PULDNDP007"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,station_trails_pane,XmNmnemonic,langcode_hotkey("PULDNDP007"),XmNbackground,colors[0xff],NULL); 
       trails_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,station_trails_pane,XmNbackground,colors[0xff],NULL);
       trails_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,station_trails_pane,XmNbackground,colors[0xff],NULL);

       station_phg_pane = XmCreatePulldownMenu(displaypane,"station_phg",al,ac);
       station_phg_button = XtVaCreateManagedWidget(langcode("PULDNDP008"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,station_phg_pane,XmNmnemonic,langcode_hotkey("PULDNDP008"),XmNbackground,colors[0xff],NULL); 
       phg_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,station_phg_pane,XmNbackground,colors[0xff],NULL);
       phg_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,station_phg_pane,XmNbackground,colors[0xff],NULL);

       wx_pane = XmCreatePulldownMenu(displaypane,"station_wx",al,ac);
       wx_button = XtVaCreateManagedWidget(langcode("PULDNDP009"),xmCascadeButtonWidgetClass,displaypane,XmNsubMenuId,wx_pane,XmNmnemonic,langcode_hotkey("PULDNDP009"),XmNbackground,colors[0xff],NULL); 
       wx_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,wx_pane,XmNbackground,colors[0xff],NULL);
       wx_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,wx_pane,XmNbackground,colors[0xff],NULL);


       /* Messages */
       send_message_to_button = XtVaCreateManagedWidget(langcode("PULDNMG001"),xmPushButtonGadgetClass,messagepane,XmNmnemonic,langcode_hotkey("PULDNMG001"),XmNbackground,colors[0xff],NULL);       
       open_messages_group_button = XtVaCreateManagedWidget(langcode("PULDNMG002"),xmPushButtonGadgetClass,messagepane,XmNmnemonic,langcode_hotkey("PULDNMG002"),XmNbackground,colors[0xff],NULL);
       clear_messages_button= XtVaCreateManagedWidget(langcode("PULDNMG003"),xmPushButtonGadgetClass,messagepane,XmNmnemonic,langcode_hotkey("PULDNMG003"),XmNbackground,colors[0xff],NULL);
       auto_msg_set_button = XtVaCreateManagedWidget(langcode("PULDNMG004"),xmPushButtonGadgetClass,messagepane,XmNmnemonic,langcode_hotkey("PULDNMG004"),XmNbackground,colors[0xff],NULL);

       auto_msg_pane = XmCreatePulldownMenu(messagepane,"auto_msg",al,ac);
       auto_msg_button = XtVaCreateManagedWidget(langcode("PULDNMG005"),xmCascadeButtonWidgetClass,messagepane,XmNsubMenuId,auto_msg_pane,XmNmnemonic,langcode_hotkey("PULDNMG005"),XmNbackground,colors[0xff],NULL);
       auto_msg_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonWidgetClass,auto_msg_pane,XmNbackground,colors[0xff],NULL);
       auto_msg_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonWidgetClass,auto_msg_pane,XmNbackground,colors[0xff],NULL);

 
      /* Interface */
       iface_transmit_now = XtVaCreateManagedWidget(langcode("PULDNTNC01"),xmPushButtonGadgetClass,ifacepane,XmNmnemonic,langcode_hotkey("PULDNTNC01"),XmNbackground,colors[0xff],NULL);       
       iface_connect_button = XtVaCreateManagedWidget(langcode("PULDNTNT04"),xmPushButtonGadgetClass,ifacepane,XmNmnemonic,langcode_hotkey("PULDNTNT04"),XmNbackground,colors[0xff],NULL);

       tnc_logging_pane = XmCreatePulldownMenu(ifacepane,"tnc_logging_pane",al,ac);
       tnc_logging = XtVaCreateManagedWidget(langcode("PULDNTNT02"),xmCascadeButtonWidgetClass,ifacepane,XmNsubMenuId, tnc_logging_pane,XmNmnemonic,langcode_hotkey("PULDNTNT02"),XmNbackground,colors[0xff],NULL);     
       tnc_logging_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,tnc_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       tnc_logging_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,tnc_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT002"),XmNbackground,colors[0xff],NULL);

       net_logging_pane = XmCreatePulldownMenu(ifacepane,"net_logging_pane",al,ac);
       net_logging = XtVaCreateManagedWidget(langcode("PULDNTNT03"),xmCascadeButtonWidgetClass,ifacepane,XmNsubMenuId, net_logging_pane,XmNmnemonic,langcode_hotkey("PULDNTNT03"),XmNbackground,colors[0xff],NULL);
       net_logging_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,net_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       net_logging_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,net_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT002"),XmNbackground,colors[0xff],NULL);

       igate_logging_pane = XmCreatePulldownMenu(ifacepane,"igate_logging_pane",al,ac);
       igate_logging = XtVaCreateManagedWidget(langcode("PULDNTNT05"),xmCascadeButtonWidgetClass,ifacepane,XmNsubMenuId, igate_logging_pane,XmNmnemonic,langcode_hotkey("PULDNTNT05"),XmNbackground,colors[0xff],NULL);
       igate_logging_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,igate_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       igate_logging_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,igate_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT002"),XmNbackground,colors[0xff],NULL);

       wx_logging_pane = XmCreatePulldownMenu(ifacepane,"wx_logging_pane",al,ac);
       wx_logging = XtVaCreateManagedWidget(langcode("PULDNTNT06"),xmCascadeButtonWidgetClass,ifacepane,XmNsubMenuId, wx_logging_pane,XmNmnemonic,langcode_hotkey("PULDNTNT06"),XmNbackground,colors[0xff],NULL);
       wx_logging_on = XtVaCreateManagedWidget(langcode("PULDNOT001"),xmPushButtonGadgetClass,wx_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT001"),XmNbackground,colors[0xff],NULL);
       wx_logging_off = XtVaCreateManagedWidget(langcode("PULDNOT002"),xmPushButtonGadgetClass,wx_logging_pane,XmNmnemonic,langcode_hotkey("PULDNOT002"),XmNbackground,colors[0xff],NULL);

       /* Help*/
       help_about = XtVaCreateManagedWidget(langcode("PULDNHEL01"),xmPushButtonGadgetClass,helppane,XmNmnemonic,langcode_hotkey("PULDNHEL01"),XmNbackground,colors[0xff],NULL);
       help_help = XtVaCreateManagedWidget(langcode("PULDNHEL02"),xmPushButtonGadgetClass,helppane,XmNmnemonic,langcode_hotkey("PULDNHEL02"),XmNbackground,colors[0xff],NULL);
             
       /* view */
       XtAddCallback(stations_button, XmNactivateCallback,Station_List,"0");   
       XtAddCallback(mobile_button, XmNactivateCallback,Station_List,"1");   
       XtAddCallback(weather_button, XmNactivateCallback,Station_List,"2");   
       XtAddCallback(localstations_button, XmNactivateCallback,Station_List,"3");   
       
       /* button callbacks */
       XtAddCallback(General_q_button, XmNactivateCallback,General_query,"");
       XtAddCallback(IGate_q_button, XmNactivateCallback,IGate_query,NULL);
       XtAddCallback(WX_q_button, XmNactivateCallback,WX_query,NULL);
 
       XtAddCallback(station_clear_button, XmNactivateCallback,Stations_Clear,NULL); 

       XtAddCallback(tracks_clear_button, XmNactivateCallback,Tracks_All_Clear,NULL);

       XtAddCallback(exit_button, XmNactivateCallback,Menu_Quit,NULL); 

       on_off_switch(long_lat_grid,grid_on,grid_off);
       XtAddCallback(grid_on, XmNactivateCallback,Grid_on,NULL); 
       XtAddCallback(grid_off, XmNactivateCallback,Grid_off,NULL);

       on_off_switch(map_auto_maps,map_auto_on,map_auto_off);
       XtAddCallback(map_auto_on, XmNactivateCallback,Map_auto_on,NULL); 
       XtAddCallback(map_auto_off, XmNactivateCallback,Map_auto_off,NULL); 

       XtAddCallback(map_chooser_button, XmNactivateCallback,Map_chooser,NULL);

       on_off_switch(map_color_levels,map_levels_on,map_levels_off);
       XtAddCallback(map_levels_on, XmNactivateCallback,Map_levels_on,NULL); 
       XtAddCallback(map_levels_off, XmNactivateCallback,Map_levels_off,NULL); 


       XtSetSensitive(map_bgcolor[map_background_color],False);

       XtAddCallback(map_bgcolor[10], XmNactivateCallback,Map_background,"10"); 
       XtAddCallback(map_bgcolor[0], XmNactivateCallback,Map_background,"0"); 
       XtAddCallback(map_bgcolor[1], XmNactivateCallback,Map_background,"1"); 
       XtAddCallback(map_bgcolor[2], XmNactivateCallback,Map_background,"2"); 
       XtAddCallback(map_bgcolor[3], XmNactivateCallback,Map_background,"3"); 
       XtAddCallback(map_bgcolor[4], XmNactivateCallback,Map_background,"4"); 
       XtAddCallback(map_bgcolor[5], XmNactivateCallback,Map_background,"5"); 
       XtAddCallback(map_bgcolor[6], XmNactivateCallback,Map_background,"6"); 
       XtAddCallback(map_bgcolor[7], XmNactivateCallback,Map_background,"7"); 
       XtAddCallback(map_bgcolor[8], XmNactivateCallback,Map_background,"8"); 
       XtAddCallback(map_bgcolor[9], XmNactivateCallback,Map_background,"9"); 

       on_off_switch(letter_style,map_station_label1,map_station_label0);
       XtAddCallback(map_station_label0, XmNactivateCallback,Map_station_label,"0"); 
       XtAddCallback(map_station_label1, XmNactivateCallback,Map_station_label,"1"); 

       on_off_switch(wx_alert_style,map_wx_alerts_1,map_wx_alerts_0);
       XtAddCallback(map_wx_alerts_0, XmNactivateCallback,Map_wx_alerts,"0"); 
       XtAddCallback(map_wx_alerts_1, XmNactivateCallback,Map_wx_alerts,"1"); 

       XtAddCallback(defaults_button, XmNactivateCallback,Configure_defaults,NULL);
       XtAddCallback(aa_button, XmNactivateCallback,Configure_audio_alarms,NULL);
       XtAddCallback(station_button, XmNactivateCallback,Configure_station,NULL);

 
       XtAddCallback(help_about, XmNactivateCallback,Help_About,NULL); 
       XtAddCallback(help_help, XmNactivateCallback,Help_Index,NULL);

       /* Interface */
       XtAddCallback(device_config_button, XmNactivateCallback,Configure_interface,NULL); 
       
       /* TNC */
       XtAddCallback(iface_transmit_now, XmNactivateCallback,TNC_Transmit_now,NULL); 
       on_off_switch(log_tnc_data,tnc_logging_on,tnc_logging_off);
       XtAddCallback(tnc_logging_on, XmNactivateCallback,TNC_Logging_on,NULL); 
       XtAddCallback(tnc_logging_off, XmNactivateCallback,TNC_Logging_off,NULL); 


       /* NET */
       on_off_switch(log_net_data,net_logging_on,net_logging_off);
       XtAddCallback(net_logging_on, XmNactivateCallback,Net_Logging_on,NULL); 
       XtAddCallback(net_logging_off, XmNactivateCallback,Net_Logging_off,NULL); 

       /* igate */
       on_off_switch(log_igate,igate_logging_on,igate_logging_off);
       XtAddCallback(igate_logging_on, XmNactivateCallback,IGate_Logging_on,NULL); 
       XtAddCallback(igate_logging_off, XmNactivateCallback,IGate_Logging_off,NULL); 

       /* igate */
       on_off_switch(log_wx,wx_logging_on,wx_logging_off);
       XtAddCallback(wx_logging_on, XmNactivateCallback,WX_Logging_on,NULL); 
       XtAddCallback(wx_logging_off, XmNactivateCallback,WX_Logging_off,NULL); 

       /* Display */ 
       on_off_switch(symbol_alt_display,alt_on,alt_off);
       XtAddCallback(alt_on, XmNactivateCallback,Alt_on,NULL); 
       XtAddCallback(alt_off, XmNactivateCallback,Alt_off,NULL);
    
       on_off_switch(symbol_course_display,course_on,course_off);
       XtAddCallback(course_on, XmNactivateCallback,Course_on,NULL); 
       XtAddCallback(course_off, XmNactivateCallback,Course_off,NULL); 

       on_off_switch(symbol_speed_display,speed_on,speed_off);
       XtAddCallback(speed_on, XmNactivateCallback,Speed_on,NULL); 
       XtAddCallback(speed_off, XmNactivateCallback,Speed_off,NULL); 

       on_off_switch(symbol_dist_course_display,dist_course_on,dist_course_off);
       XtAddCallback(dist_course_on, XmNactivateCallback,Dist_course_on,NULL); 
       XtAddCallback(dist_course_off, XmNactivateCallback,Dist_course_off,NULL); 

       on_off_switch(units_english_metric,units_choice_on,units_choice_off);
       XtAddCallback(units_choice_on, XmNactivateCallback,Units_choice_on,NULL); 
       XtAddCallback(units_choice_off, XmNactivateCallback,Units_choice_off,NULL); 

       on_off_switch(station_trails,trails_on,trails_off);
       XtAddCallback(trails_on, XmNactivateCallback,Station_trails_on,NULL); 
       XtAddCallback(trails_off, XmNactivateCallback,Station_trails_off,NULL);

       on_off_switch(show_phg,phg_on,phg_off);
       XtAddCallback(phg_on, XmNactivateCallback,Station_phg_on,NULL); 
       XtAddCallback(phg_off, XmNactivateCallback,Station_phg_off,NULL); 
      
       XtAddCallback(track_button, XmNactivateCallback,Track_station,NULL); 

       on_off_switch(symbol_weather_display,wx_on,wx_off);
       XtAddCallback(wx_on, XmNactivateCallback,WX_on,NULL); 
       XtAddCallback(wx_off, XmNactivateCallback,WX_off,NULL); 

       on_off_switch(auto_reply,auto_msg_on,auto_msg_off);
       XtAddCallback(auto_msg_on, XmNactivateCallback,Auto_msg_on,NULL); 
       XtAddCallback(auto_msg_off, XmNactivateCallback,Auto_msg_off,NULL);
       XtAddCallback(auto_msg_set_button,XmNactivateCallback,Auto_msg_set,NULL);

       XtAddCallback(iface_connect_button, XmNactivateCallback,control_interface,NULL); 

       XtAddCallback(open_file_button, XmNactivateCallback,Read_File_Selection,NULL);  

       XtAddCallback(packet_data_button, XmNactivateCallback,Display_data,NULL);  
       XtAddCallback(locate_button, XmNactivateCallback,Locate_station,NULL);  
       XtAddCallback(alert_button, XmNactivateCallback,Display_Wx_Alert,NULL);
       XtAddCallback(wx_station_button, XmNactivateCallback,WX_station,NULL);
       XtAddCallback(jump_button, XmNactivateCallback,Jump_location,NULL);  

       XtAddCallback(send_message_to_button, XmNactivateCallback,Send_message,NULL);  
       XtAddCallback(open_messages_group_button, XmNactivateCallback,Send_message,"*");  
       XtAddCallback(clear_messages_button, XmNactivateCallback,Clear_messages,NULL);  
       XtAddCallback(save_button, XmNactivateCallback,Save_Config,NULL);  


       XtAddCallback(test_button, XmNactivateCallback,Test,NULL);
      
//       XtSetSensitive(test_button, False);

#define FONT_WIDTH 7

       /* Create bottom text area */ 

       text = XtVaCreateManagedWidget("text_output", xmTextFieldWidgetClass, form,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
				      XmNshadowThickness,	1,
                                      XmNcolumns,30,
                                      XmNwidth,((30*FONT_WIDTH)+2),
                                      XmNbackground, colors[0xff],
                                      XmNtopOffset,4,
                                      XmNtopAttachment,XmATTACH_NONE,
                                      XmNbottomAttachment,XmATTACH_FORM,                                
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      XmNbackground, colors[0xff],        
                                      NULL);



       text2 = XtVaCreateManagedWidget("text_output2", xmTextFieldWidgetClass, form,
                                       XmNeditable,   False,
                                       XmNcursorPositionVisible, False,
                                       XmNsensitive, STIPPLE,
                                       XmNshadowThickness,	1,
                                       XmNcolumns,21,
                                       XmNwidth,((21*FONT_WIDTH)+2),
                                       XmNbackground, colors[0xff],
                                       XmNtopAttachment,XmATTACH_NONE,
                                       XmNbottomAttachment,XmATTACH_FORM,                                
                                       XmNleftAttachment, XmATTACH_WIDGET,
                                       XmNleftWidget,text,
                                       XmNrightAttachment,XmATTACH_NONE,
                                       XmNbackground, colors[0xff],        
                                       NULL);

       text3 = XtVaCreateManagedWidget("text_output3", xmTextFieldWidgetClass, form,
                                       XmNeditable,   False,
                                       XmNcursorPositionVisible, False,
                                       XmNsensitive, STIPPLE,
                                       XmNshadowThickness,	1,
                                       XmNcolumns,14,
                                       XmNwidth,((14*FONT_WIDTH)+2),
                                       XmNbackground, colors[0xff],
                                       XmNtopAttachment,XmATTACH_NONE,
                                       XmNbottomAttachment,XmATTACH_FORM,                                
                                       XmNleftAttachment, XmATTACH_WIDGET,
                                       XmNleftWidget,text2,
                                       XmNrightAttachment,XmATTACH_NONE,
                                       XmNbackground, colors[0xff],        
                                       NULL);

       text4 = XtVaCreateManagedWidget("text_output4", xmTextFieldWidgetClass, form,
                                       XmNeditable,   False,
                                       XmNcursorPositionVisible, False,
                                       XmNsensitive, STIPPLE,
                                       XmNshadowThickness,	1,
                                       XmNcolumns,12,
                                       XmNwidth,((12*FONT_WIDTH)+2),
                                       XmNbackground, colors[0xff],
                                       XmNtopAttachment,XmATTACH_NONE,
                                       XmNbottomAttachment,XmATTACH_FORM,                                
                                       XmNleftAttachment, XmATTACH_WIDGET,
                                       XmNleftWidget,text3,
                                       XmNrightAttachment,XmATTACH_NONE,
                                       XmNbackground, colors[0xff],        
                                       NULL);

       iface_da = XtVaCreateManagedWidget("iface", xmDrawingAreaWidgetClass,form,
                                   XmNwidth, 20*(MAX_IFACE_DEVICES/2),
                                   XmNheight, 20,
                                   XmNunitType, XmPIXELS,
                                   XmNtopAttachment,XmATTACH_NONE,
                                   XmNbottomAttachment,XmATTACH_FORM,
                                   XmNbottomOffset, 5,                                
                                   XmNleftAttachment, XmATTACH_WIDGET,
                                   XmNleftWidget,text4,
                                   XmNrightAttachment,XmATTACH_NONE,
                                   XmNbackground, colors[0xff],        
                                   NULL);

       sep = XtVaCreateManagedWidget("sep", xmSeparatorGadgetClass,form,
                                      XmNorientation, XmHORIZONTAL,
                                      XmNtopAttachment,XmATTACH_NONE,
                                      XmNbottomAttachment,XmATTACH_WIDGET,
                                      XmNbottomWidget,text,                                
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment,XmATTACH_FORM,
                                      XmNbackground, colors[0xff],        
                                      NULL);

      /* Do drawing map area */

      da = XtVaCreateManagedWidget("da", xmDrawingAreaWidgetClass,form,
                                   XmNwidth, screen_width,
                                   XmNheight, screen_height,
                                   XmNunitType, XmPIXELS,
                                   XmNtopAttachment, XmATTACH_WIDGET,
                                   XmNtopWidget, menubar,
                                   XmNbottomAttachment, XmATTACH_WIDGET,
                                   XmNbottomWidget, sep,
                                   XmNleftAttachment, XmATTACH_FORM,
                                   XmNrightAttachment, XmATTACH_FORM,
                                   XmNbackground, colors[0xff],        
                                   NULL);
       
        /* Menu popup */
        menu_popup = XmCreatePopupMenu(da,"Menu Popup",al,ac);
        XtVaSetValues(menu_popup, XmNwhichButton, 1, NULL);
        XtCreateManagedWidget(langcode("POPUPMA001"),xmLabelWidgetClass,menu_popup,al,ac);
        XtCreateManagedWidget("sep",xmSeparatorWidgetClass,menu_popup,al,ac);
        zoom_in=XtCreateManagedWidget(langcode("POPUPMA002"),xmPushButtonGadgetClass,menu_popup,al,ac); 
        XtAddCallback(zoom_in,XmNactivateCallback,Zoom_in,NULL);
        zoom_out=XtCreateManagedWidget(langcode("POPUPMA003"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(zoom_out,XmNactivateCallback,Zoom_out,NULL);
        zoom_sub=XmCreatePulldownMenu(menu_popup,"zoom sub",al,ac);
        zoom_level=XtVaCreateManagedWidget(langcode("POPUPMA004"),xmCascadeButtonGadgetClass,menu_popup,XmNsubMenuId,zoom_sub,XmNbackground,colors[0xff],NULL);
        zl1=XtCreateManagedWidget(langcode("POPUPMA005"),xmPushButtonGadgetClass,zoom_sub,al,ac);
        XtAddCallback(zl1,XmNactivateCallback,Zoom_level,"1"); 
        zl2=XtCreateManagedWidget(langcode("POPUPMA006"),xmPushButtonGadgetClass,zoom_sub,al,ac);
        XtAddCallback(zl2,XmNactivateCallback,Zoom_level,"2");  
        zl3=XtCreateManagedWidget(langcode("POPUPMA007"),xmPushButtonGadgetClass,zoom_sub,al,ac);
        XtAddCallback(zl3,XmNactivateCallback,Zoom_level,"3");  
        zl4=XtCreateManagedWidget(langcode("POPUPMA008"),xmPushButtonGadgetClass,zoom_sub,al,ac);
        XtAddCallback(zl4,XmNactivateCallback,Zoom_level,"4");  
        zl5=XtCreateManagedWidget(langcode("POPUPMA009"),xmPushButtonGadgetClass,zoom_sub,al,ac);
        XtAddCallback(zl5,XmNactivateCallback,Zoom_level,"5");  
        zl6=XtCreateManagedWidget(langcode("POPUPMA010"),xmPushButtonGadgetClass,zoom_sub,al,ac); 
        XtAddCallback(zl6,XmNactivateCallback,Zoom_level,"6"); 
        pan_up=XtCreateManagedWidget(langcode("POPUPMA011"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(pan_up,XmNactivateCallback,Pan_up,NULL);  
        pan_down=XtCreateManagedWidget(langcode("POPUPMA012"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(pan_down,XmNactivateCallback,Pan_down,NULL);  
        pan_left=XtCreateManagedWidget(langcode("POPUPMA013"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(pan_left,XmNactivateCallback,Pan_left,NULL);  
        pan_right=XtCreateManagedWidget(langcode("POPUPMA014"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(pan_right,XmNactivateCallback,Pan_right,NULL);  

        last_loc=XtCreateManagedWidget(langcode("POPUPMA016"),xmPushButtonGadgetClass,menu_popup,al,ac);
        XtAddCallback(last_loc,XmNactivateCallback,Last_location,NULL);
  
        station_info=XtCreateManagedWidget(langcode("POPUPMA015"),xmPushButtonGadgetClass,menu_popup,NULL,0);
        XtAddCallback(station_info,XmNactivateCallback,Station_info,NULL);  

        /* mouse tracking */
        XtAddEventHandler(da,LeaveWindowMask,FALSE,ClearTrackMouse,(XtPointer)text2);
        XtAddEventHandler(da,PointerMotionMask,FALSE,TrackMouse,(XtPointer)text2);

        ac = 0;    
        children[ac++] = text;
        children[ac++] = text2;
        children[ac++] = menubar;
        children[ac++] = da;

	XtManageChildren(children, ac);
	ac = 0;
	XtManageChild(form);
       
        WM_DELETE_WINDOW = XmInternAtom(XtDisplay(appshell),"WM_DELETE_WINDOW", False);
        XmAddWMProtocolCallback(appshell, WM_DELETE_WINDOW, Window_Quit, (XtPointer) NULL);

        XmTextFieldSetString(text,"");
        XtManageChild(text);  

        if(track_station_on!=1)
          {sprintf(zoom,langcode("BBARZM0001"),size);}
          else
          {sprintf(zoom,langcode("BBARZM0002"),size);}
        XmTextFieldSetString(text4,zoom);
        XtManageChild(text);  

        XtCreateManagedWidget("MAIN",xmMainWindowWidgetClass,appshell,NULL,0);

	XtRealizeWidget (appshell);
        XtVaSetValues(appshell,XmNx,1,XmNy,1,NULL);
        XtPopup(appshell,XtGrabNone);

        create_gc(da);
        XSetForeground(XtDisplay(da),gc,colors[0xff]); 
        XSetBackground(XtDisplay(da),gc,colors[0xff]); 

        XFillRectangle(XtDisplay(appshell),XtWindow(da),gc,0,0,screen_width,screen_height);

        XtVaSetValues(appshell,
                      XmNwidth,screen_width,
                      XmNheight,(screen_height+60),
                      NULL);

	XtAddCallback (da, XmNinputCallback, da_input,NULL);
        XtAddCallback (da, XmNresizeCallback, da_resize,NULL);     
	XtAddCallback (da, XmNexposeCallback, da_expose,(XtPointer)text);
        UpdateTime( (XtPointer) da , (XtIntervalId) NULL );

        if(debug_level & 8)
          {printf("Create appshell stop\n");}

}



void create_gc(Widget w)
{
 XGCValues       values;
 Display        *display = XtDisplay(w);
 int             mask = 0;
 Pixmap          pix;

 if (debug_level & 8)
    printf("Create gc start\n");

 if (gc != 0)
    return;

 /* Allocate colors */
 colors[0x00] = GetPixelByName(w,"darkgreen");
 colors[0x01] = GetPixelByName(w,"purple");
 colors[0x02] = GetPixelByName(w,"darkgreen");
 colors[0x03] = GetPixelByName(w,"cyan");
 colors[0x04] = GetPixelByName(w,"brown");
 colors[0x05] = GetPixelByName(w,"plum"); /* light magenta */
 colors[0x06] = GetPixelByName(w,"orange");
 colors[0x07] = GetPixelByName(w,"darkgray");
 colors[0x08] = GetPixelByName(w,"black");
 colors[0x09] = GetPixelByName(w,"blue");
 colors[0x0a] = GetPixelByName(w,"green");
 colors[0x0b] = GetPixelByName(w,"mediumorchid"); /* light purple */
 colors[0x0c] = GetPixelByName(w,"red");
 colors[0x0d] = GetPixelByName(w,"magenta");
 colors[0x0e] = GetPixelByName(w,"yellow");
 colors[0x0f] = GetPixelByName(w,"white");
 colors[0x10] = GetPixelByName(w,"black");
 colors[0x11] = GetPixelByName(w,"black");
 colors[0x12] = GetPixelByName(w,"black");
 colors[0x13] = GetPixelByName(w,"black");
 colors[0x14] = GetPixelByName(w,"lightgray");
 colors[0x15] = GetPixelByName(w,"magenta");
 colors[0x16] = GetPixelByName(w,"mediumorchid"); /* light purple */
 colors[0x17] = GetPixelByName(w,"lightblue");
 colors[0x18] = GetPixelByName(w,"purple");
 colors[0x19] = GetPixelByName(w,"orange2"); /* light orange */
 colors[0x20] = GetPixelByName(w,"white");
 colors[0x40] = GetPixelByName(w,"yellow");
 colors[0x41] = GetPixelByName(w,"DarkOrange3");
 colors[0x42] = GetPixelByName(w,"purple");
 colors[0x43] = GetPixelByName(w,"gray80");
 colors[0x44] = GetPixelByName(w,"red3");
 colors[0x45] = GetPixelByName(w,"brown1");
 colors[0x46] = GetPixelByName(w,"brown3");
 colors[0x47] = GetPixelByName(w,"blue4");
 colors[0x48] = GetPixelByName(w,"DeepSkyBlue");
 colors[0x49] = GetPixelByName(w,"DarkGreen");
 colors[0x4a] = GetPixelByName(w,"red2");
 colors[0x4b] = GetPixelByName(w,"green3");
 colors[0x4c] = GetPixelByName(w,"MediumBlue");
 colors[0x4d] = GetPixelByName(w,"white");
 colors[0x4e] = GetPixelByName(w,"gray53");
 colors[0x4f] = GetPixelByName(w,"gray35");
 colors[0x50] = GetPixelByName(w,"gray27");
 colors[0x51] = GetPixelByName(w,"black");
 colors[0x52] = GetPixelByName(w,"LimeGreen");
 colors[0xfe] = GetPixelByName(w,"pink");

 /* map solid colors */
 colors[0x60] = GetPixelByName(w,"HotPink");
 colors[0x61] = GetPixelByName(w,"RoyalBlue");
 colors[0x62] = GetPixelByName(w,"orange3");
 colors[0x63] = GetPixelByName(w,"yellow2");
 colors[0x64] = GetPixelByName(w,"ForestGreen");
 colors[0x65] = GetPixelByName(w,"DodgerBlue");
 colors[0x66] = GetPixelByName(w,"cyan2");
 colors[0x67] = GetPixelByName(w,"plum2");
 colors[0x68] = GetPixelByName(w,"blue3");
 colors[0x69] = GetPixelByName(w,"gray86");

 /* tracking trail colors */
 trail_colors[0x00] = GetPixelByName(w,"black");       /* Reserved for all my_callsign* stations */
 trail_colors[0x01] = GetPixelByName(w,"blue");
 trail_colors[0x02] = GetPixelByName(w,"green");
 trail_colors[0x03] = GetPixelByName(w,"red");
 trail_colors[0x04] = GetPixelByName(w,"magenta");
 trail_colors[0x05] = GetPixelByName(w,"yellow");
 trail_colors[0x06] = GetPixelByName(w,"white");
 trail_colors[0x07] = GetPixelByName(w,"DarkOrchid");
 trail_colors[0x08] = GetPixelByName(w,"purple");
 trail_colors[0x09] = GetPixelByName(w,"OrangeRed");
 trail_colors[0x0a] = GetPixelByName(w,"brown");
 trail_colors[0x0b] = GetPixelByName(w,"darkgreen");
 trail_colors[0x0c] = GetPixelByName(w,"MediumBlue");
 trail_colors[0x0d] = GetPixelByName(w,"ForestGreen");
 trail_colors[0x0e] = GetPixelByName(w,"chartreuse");
 trail_colors[0x0f] = GetPixelByName(w,"cornsilk");
 trail_colors[0x10] = GetPixelByName(w,"LightCyan");
 trail_colors[0x11] = GetPixelByName(w,"cyan");
 trail_colors[0x12] = GetPixelByName(w,"DarkSlateGray");
 trail_colors[0x13] = GetPixelByName(w,"NavyBlue");
 trail_colors[0x14] = GetPixelByName(w,"DarkOrange3");
 trail_colors[0x15] = GetPixelByName(w,"gray27");
 trail_colors[0x16] = GetPixelByName(w,"RoyalBlue");
 trail_colors[0x17] = GetPixelByName(w,"yellow2");
 trail_colors[0x18] = GetPixelByName(w,"DodgerBlue");
 trail_colors[0x19] = GetPixelByName(w,"cyan2");
 trail_colors[0x1a] = GetPixelByName(w,"blue3");
 trail_colors[0x1b] = GetPixelByName(w,"gray86");
 trail_colors[0x1c] = GetPixelByName(w,"SteelBlue");
 trail_colors[0x1d] = GetPixelByName(w,"PaleGreen");
 trail_colors[0x1e] = GetPixelByName(w,"RosyBrown");
 trail_colors[0x1f] = GetPixelByName(w,"DeepSkyBlue");

 values.background=GetPixelByName(w,"darkgray");
           
 gc = XCreateGC(display, XtWindow(w), mask, &values);
 
 pix =  XCreatePixmap(XtDisplay(w), RootWindowOfScreen(XtScreen(w)), 20, 20, 1); 
 values.function = GXcopy;
 gc2 = XCreateGC(XtDisplay(w), pix,GCForeground|GCBackground|GCFunction, &values);

 pixmap=XCreatePixmap(XtDisplay(w),
                       DefaultRootWindow(XtDisplay(w)),
                       screen_width,screen_height,
                       DefaultDepthOfScreen(XtScreen(w)));

 pixmap_final=XCreatePixmap(XtDisplay(w),
                        DefaultRootWindow(XtDisplay(w)),
                        screen_width,screen_height,
                        DefaultDepthOfScreen(XtScreen(w)));

 display_up=1;

 wait_to_redraw=0;

 if (debug_level & 8)
    printf("Create gc stop\n");
}




void da_expose(Widget w, XtPointer client_data, XtPointer call_data)
{
Dimension       width, height, margin_width, margin_height;

XmDrawingAreaCallbackStruct *db = (XmDrawingAreaCallbackStruct *)call_data;

XExposeEvent *event = (XExposeEvent *) db->event;



unsigned char   unit_type;


/* Call a routine to create a Graphics Context */
create_gc(w);

/*printf("Expose event\n");*/
	
/* First get the various dimensions */
XtVaGetValues(w,
              XmNwidth, &width,
	      XmNheight, &height,
	      XmNmarginWidth, &margin_width,
	      XmNmarginHeight, &margin_height,
	      XmNunitType, &unit_type,
	      0);


XCopyArea(XtDisplay(w),pixmap_final,XtWindow(w),gc,event->x,event->y,event->width,event->height,event->x,event->y);
}




void  da_resize(Widget w, XtPointer client_data, XtPointer call_data)
{
Dimension       width, height;

 if (XtIsRealized(w)){
  /* First get the various dimensions */
  XtVaGetValues(w,
      XmNwidth, &width,
      XmNheight, &height,
      0);
  screen_width=width;
  screen_height=height;
  XtVaSetValues(w,
      XmNwidth,screen_width,
      XmNheight,screen_height,
      NULL);

/*  printf("Size x:%ld, y:%ld\n",screen_width,screen_height);*/

  if (pixmap)
     XFreePixmap(XtDisplay(w),pixmap);

  if(pixmap_final)
     XFreePixmap(XtDisplay(w),pixmap_final);

  pixmap=XCreatePixmap(XtDisplay(w),
             DefaultRootWindow(XtDisplay(w)),
             width,height,
             DefaultDepthOfScreen(XtScreen(w)));

  pixmap_final=XCreatePixmap(XtDisplay(w),
                   DefaultRootWindow(XtDisplay(w)),
                   width,height,
                   DefaultDepthOfScreen(XtScreen(w)));
   
  create_image(w);
  XCopyArea(XtDisplay(w),pixmap_final,XtWindow(w),gc,0,0,screen_width,screen_height,0,0);      
 }
}

void da_input(Widget w, XtPointer client_data, XtPointer call_data)
{

XEvent         *event = ((XmDrawingAreaCallbackStruct *) call_data)->event;
Dimension       width, height;
int redraw;

XtVaGetValues(w,
              XmNwidth, &width,
	      XmNheight, &height,
	      0);

/*printf("input event %d %d\n",event->type,ButtonPress);*/
redraw=0;
	
        input_x = event->xbutton.x;
        input_y = event->xbutton.y;

	if(event->type == ButtonPress)
          {/*printf("ButtonPress %d %d\n",event->xbutton.button,Button3);*/
           if(event->xbutton.button == Button1)
              {menu_x=input_x;
               menu_y=input_y;
               if(menu_popup!=NULL)
                 {XmMenuPosition(menu_popup,(XButtonPressedEvent *)event);
                  XtManageChild(menu_popup);
		 }                           
	      }

           if(event->xbutton.button == Button2)
             {
	     }
            
           if(event->xbutton.button == Button3)
              {
	      }

           if(redraw)
             {create_image(w);
              XCopyArea(XtDisplay(w),pixmap_final,XtWindow(w),gc,0,0,screen_width,screen_height,0,0);
              /*printf("Current x %ld y %ld\n",mid_x_long_offset,mid_y_lat_offset);*/
             }

	} 
        else 
        {if(event->type == MotionNotify) 
           {input_x = event->xmotion.x;
            input_y = event->xmotion.y;
            /*printf("da_input2 x %d y %d\n",input_x,input_y);*/
 	   }
	}
}


void UpdateTime( XtPointer clientData, XtIntervalId id )
{
Widget w = (Widget) clientData;
time_t nexttime;
int do_time;
int max;
static int last_alert_on_screen;

do_time=0;
nexttime = 10;

sound_done();

if(display_up)
  {if(display_up_first==0)
     {/* do startup */
      display_up_first=1; 
      load_pixmap_symbol_file("symbols.dat");
      /*printf("Symbols Loaded %d\n",symbols_loaded);*/

      my_station_add(my_callsign,my_group,my_symbol,my_long,my_lat,my_phg,my_comment);   
      
      /* start interfaces */
      startup_all_or_defined_port(-1);

      /* make sure the size is right after startup */
      da_resize(w, NULL,NULL);

      /* set last position to first location on startup */
      set_last_position();
      }
      else
      {
       /* check popup time outs */
       popup_time_out_check();

       /* Check Messages */
       if (new_message_data>0)
          update_messages(); 

       /* check on Redraw requests */
       if ( ( (redraw_on_new_data>1) || (redraw_on_new_data && (sec_now() > next_redraw)) || (sec_now() > next_redraw+30)  ) && !wait_to_redraw) {
          /*printf("Redraw on new data \n");*/

       /* check if alert_redraw is set and it has been xx seconds since last redraw then redraw */ 
	  if (alert_redraw_on_new_data && (sec_now()-last_alert_redraw)>WX_ALERTS_REFRESH_TIME) {
	     create_image(da);
             XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
	     if ((alert_redraw_on_new_data = alert_on_screen()) > last_alert_on_screen) 
	         play_sound(sound_command, sound_wx_alert_message);
                 
	     last_alert_on_screen = alert_redraw_on_new_data;
             alert_redraw_on_new_data = 0;
	  } 

	  redraw_on_new_data=0;
	  redraw_symbols(w);
	  next_redraw=sec_now()+30;
	  if (alert_display_request()) alert_redraw_on_new_data = redraw_on_new_data = 1;
       }

       /* Look for packet data and check port status */
       if (delay_time>15) {
          display_packet_data();
          interface_status(w); 
          delay_time=0;
          /* check station lists */
          update_station_scroll_list();
       }  
       delay_time++;

       /* check gps start up, GPS on GPSPORT */
       if(sec_now() > sec_next_gps)
         {sec_next_gps=sec_now()+gps_time;
          /*printf("Check GPS\n");*/
          /* set dtr lines down */
          dtr_all_set(1);
          if(gps_stop_now)
            {gps_stop_now=0;}
	 }

       /* Check to restablish a connection */
       if(sec_now() > net_next_time)
         {net_last_time = sec_now();
          net_next_time = net_last_time+120; 
          /*printf("Checking for reconnects\n");*/
          check_ports();
	 }


       /* Check to see if it time to spit out data */
       if(!wait_to_redraw)
         {if(last_time == 0)
            {/* first update */
             next_time = 120;
             last_time = sec_now();
             do_time=1;  
	    }
            else
            {/* check for update */
             /*printf("Checking --- time %ld time to update %ld\n",sec_now(),last_time+next_time);*/
             if(sec_now() >= (last_time + next_time))
               {next_time += next_time;
                if(next_time>max_transmit_time)
                  {next_time=max_transmit_time;}
                last_time = sec_now();
                do_time=1; 
	       }
	    }
          
           if(do_time || transmit_now)
             {transmit_now=0;
              /* output to ALL net/tnc ports */
              /*printf("Output data\n");*/
              output_my_aprs_data(); 
	     }
	 }
       /* END- Check to see if it time to spit out data */


       /* Time to put out raw WX data ? */
       if(sec_now() > sec_next_raw_wx)
         {sec_next_raw_wx=sec_now()+600;
          if (transmit_raw_wx) 
             tx_raw_wx_data();
          /* check wx data last recieved */
          wx_last_data_check();
	 }
       /* end raw wx data */
 
       /* is it time to spit out messages? */  
       check_and_transmit_messages(sec_now());

       /* get data from interface */
       max=0;
       while (max < 20 && !XtAppPending(app_context)) {
            pthread_mutex_lock(&data_lock);
            if (data_avail) {
              
               switch (port_data[data_port].device_type) {

                      /* NET Data stream */   
                      case DEVICE_NET_STREAM:
                           if (log_net_data) {
                              log_data(LOGFILE_NET,data);
			   }

                           packet_data_add(langcode("WPUPDPD006"),data);
                           serial_decode(data,'I',data_port);        
                           break;


                      /* TNC Devices */ 
                      case DEVICE_SERIAL_TNC:
                      case DEVICE_AX25_TNC:
                           if (log_tnc_data) {
                              log_data(LOGFILE_TNC,data);
			   }

                           packet_data_add(langcode("WPUPDPD005"),data);
                           serial_decode(data,'T',data_port);  
                           break;

                      case DEVICE_SERIAL_TNC_HSP_GPS:
                           if (port_data[data_port].dtr==1) {
                              /* get GPS data */
                              gps_data_find(data,data_port);
			   } else {
                             /* get TNC data */
                             if (log_tnc_data) {
                                log_data(LOGFILE_TNC,data);
			     }

                             packet_data_add(langcode("WPUPDPD005"),data);
                             serial_decode(data,'T',data_port);  
			   }
                           break;

                      /* GPS Devices */
                      case DEVICE_SERIAL_GPS:
                      case DEVICE_NET_GPSD:
                           /*printf("GPS Data <%s>\n",data);*/
                           gps_data_find(data,data_port);
                           break;

                      /* WX Devices */
                      case DEVICE_SERIAL_WX:
                      case DEVICE_NET_WX:
                           if (log_wx) {
                              log_data(LOGFILE_WX,data);
			     }
                           wx_decode(data,data_port);
                           break;

                      default:printf("Data from unknown source\n");
                          break;
	       }

               data_avail=0;
               max++;
	    }
            else
            {max=20;}
           
            pthread_mutex_unlock(&data_lock);

       }
       /* END- get data from interface */
       
       /* READ FILE IF OPENED */
       if(read_file)
         {if(read_file_delay_count>=read_file_delay)
            {read_file_line(read_file_ptr);
             redraw_on_new_data=2;
             read_file_delay_count=0;
	    }
            else
            {read_file_delay_count++;}
	 }
       /* END- READ FILE IF OPENED */

      }
  }

XtAppAddTimeOut(XtWidgetToApplicationContext(w),nexttime,(XtTimerCallbackProc)UpdateTime,(XtPointer)w);
}



void quit(int sig)
{
if(debug_level & 15)
  {printf("Caught %d\n",sig);}

save_data();

/* shutdown all interfaces */
shutdown_all_active_or_defined_port(-1);

if(debug_level & 1)
  {printf("Exiting..\n");}
exit (sig);
}


/* handle segfault signal */
void segfault(int sig)
{
 printf("Caught Segfault! Xastir will terminate\n");
 quit(-1);
}


/*********************  dialog position *************************/

void pos_dialog(Widget w)
{static Position x,y;

 Dimension wd, ht;

 XtVaGetValues(appshell, XmNx, &x, XmNy, &y, NULL);
 XtVaGetValues(appshell, XmNwidth, &wd, XmNheight, &ht, NULL);

 if(last_popup_x < x  || last_popup_y < y)
   {last_popup_x=x+20;
    last_popup_y=y+20;
   }
   else
   {last_popup_x+=10;
    last_popup_y+=10;
   }

 if((last_popup_y+50)>(y+(ht/4)))
   {last_popup_x=(x+20);
    last_popup_y=(y+20);
   }
 if((last_popup_x+50)>(x+(wd/4)))
   {last_popup_x=(x+10);
    last_popup_y=(y+10);
   }
  XtVaSetValues(w,XmNx,last_popup_x,XmNy,last_popup_y,NULL);
}


/*********************  fix dialog size *************************/

void fix_dialog_size(Widget w)
{Dimension wd, ht;

 if (XtIsRealized(w)){
    XtVaGetValues(w,
                  XmNwidth, &wd,
                  XmNheight, &ht,
                  0);
 
    XtVaSetValues(w,
                  XmNminWidth,wd,
                  XmNminHeight,ht,
                  XmNmaxWidth,wd,
                  XmNmaxHeight,ht,
                  NULL);          
 }
}


void fix_dialog_vsize(Widget w)
{Dimension ht;

 if (XtIsRealized(w)){
    XtVaGetValues(w,
                  XmNheight, &ht,
                  0);
 
    XtVaSetValues(w,
                  XmNminHeight,ht,
                  XmNmaxHeight,ht,
                  NULL);          
 }
}

/**************************************** Button CallBacks *************************************/
/***********************************************************************************************/

void on_off_switch(int switchpos, Widget first, Widget second)
{
if(switchpos)
  {XtSetSensitive(first,False);
   XtSetSensitive(second,True);
  }
  else
  {XtSetSensitive(first,True);
   XtSetSensitive(second,False);
  }
}

void Zoom_in(Widget w, XtPointer clientData, XtPointer calldata)
{
Dimension       width, height;
char zoom[30];

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_x_long_offset = mid_x_long_offset - ((width*size)/2) + (menu_x*size);
   mid_y_lat_offset = mid_y_lat_offset - ((height*size)/2) + (menu_y*size);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   size=size/2;
   if(size<=0)
     {size=1;}

  if(track_station_on!=1)
    {sprintf(zoom,langcode("BBARZM0001"),size);}
     else
    {sprintf(zoom,langcode("BBARZM0002"),size);}
  XmTextFieldSetString(text4,zoom);
  create_image(da);
  XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}

void Zoom_out(Widget w, XtPointer clientData, XtPointer calldata)
{
Dimension       width, height;
char zoom[30];

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_x_long_offset = mid_x_long_offset - ((width*size)/2) + (menu_x*size);
   mid_y_lat_offset = mid_y_lat_offset - ((height*size)/2) + (menu_y*size);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   size=size*2;
   if(size>=327680)
     {size=327680;}

   if(track_station_on!=1)
     {sprintf(zoom,langcode("BBARZM0001"),size);}
     else
     {sprintf(zoom,langcode("BBARZM0002"),size);}
   XmTextFieldSetString(text4,zoom);
   create_image(da);
   XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}


void Zoom_level(Widget w, XtPointer clientData, XtPointer calldata)
{Dimension       width, height;
int level;
char zoom[30];

level=atoi((char *)clientData);
if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_x_long_offset = mid_x_long_offset - ((width*size)/2) + (menu_x*size);
   mid_y_lat_offset = mid_y_lat_offset - ((height*size)/2) + (menu_y*size);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   switch(level)
         {case(1):size=1;
                  break;

          case(2):size=32;
                  break;

          case(3):size=128;
                  break;

          case(4):size=2048;
                  break;

          case(5):size=8192;
                  break;

          case(6):size=32768;
                  break;
          

          default:break;
	 }

  if(track_station_on!=1)
    {sprintf(zoom,langcode("BBARZM0001"),size);}
    else
    {sprintf(zoom,langcode("BBARZM0002"),size);}
  XmTextFieldSetString(text4,zoom);
  create_image(da);
  XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}

void Pan_up(Widget w, XtPointer clientData, XtPointer calldata)
{Dimension       width, height;

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_y_lat_offset-=(height*size/4);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   create_image(da);
   XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0); 
  }
}

void Pan_down(Widget w, XtPointer clientData, XtPointer calldata)
{Dimension       width, height;

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_y_lat_offset+=(height*size/4);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   create_image(da);
   XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}

void Pan_left(Widget w, XtPointer clientData, XtPointer calldata)
{Dimension       width, height;

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_x_long_offset-=(width*size/4);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   create_image(da);
   XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}

void Pan_right(Widget w, XtPointer clientData, XtPointer calldata)
{Dimension       width, height;

if(display_up)
  {set_last_position();
   XtVaGetValues(da,XmNwidth, &width,XmNheight, &height,0);
   mid_x_long_offset+=(width*size/4);

   /* do range checking */ 
   if((mid_x_long_offset+((width*size)/2))<0) 
     {mid_x_long_offset=0l-((width*size)/2);}

   if((mid_x_long_offset-((width*size)/2))>129600000l)
     {mid_x_long_offset=129600000l+((width*size)/2);}

   if((mid_y_lat_offset+((height*size)/2))<0) 
     {mid_y_lat_offset=0l-((height*size)/2);}

   if((mid_y_lat_offset-((height*size)/2))>64800000l)
     {mid_y_lat_offset=64800000l+((height*size)/2);}

   create_image(da);
   XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}

void Window_Quit(Widget w, XtPointer client, XtPointer calldata)
{
    quit(0);
}


void Menu_Quit(Widget w, XtPointer clientData, XtPointer calldata)
{
   quit(0);
}



void Grid_on(Widget w, XtPointer clientData, XtPointer calldata)
{
long_lat_grid=1;              /* toggle use of map lat and long grid */
XmTextFieldSetString (text,langcode("BBARSTA005"));
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
on_off_switch(long_lat_grid,grid_on,grid_off);
}

void Grid_off(Widget w, XtPointer clientData, XtPointer calldata)
{
long_lat_grid=0;              /* toggle use of map lat and long grid */   
XmTextFieldSetString (text,langcode("BBARSTA006"));
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
on_off_switch(long_lat_grid,grid_on,grid_off);
}


void Map_auto_on(Widget w, XtPointer clientData, XtPointer calldata)
{
map_auto_maps=1;              /* toggle use of auto maps */   
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
XmTextFieldSetString (text,langcode("BBARSTA007"));
on_off_switch(map_auto_maps,map_auto_on,map_auto_off);
}


void Map_auto_off(Widget w, XtPointer clientData, XtPointer calldata)
{
map_auto_maps=0;              /* toggle use of auto map */ 
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);  
XmTextFieldSetString (text,langcode("BBARSTA008"));
on_off_switch(map_auto_maps,map_auto_on,map_auto_off);
}

void Map_levels_on(Widget w, XtPointer clientData, XtPointer calldata)
{
map_color_levels=1;              /* toggle use of map color levels */   
XmTextFieldSetString (text,langcode("BBARSTA009"));
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
on_off_switch(map_color_levels,map_levels_on,map_levels_off);
}


void Map_levels_off(Widget w, XtPointer clientData, XtPointer calldata)
{
map_color_levels=0;              /* toggle use of map color levels */   
XmTextFieldSetString (text,langcode("BBARSTA010"));
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
on_off_switch(map_color_levels,map_levels_on,map_levels_off);
}

void Map_background(Widget w, XtPointer clientData, XtPointer calldata)
{int bgcolor;
 int i;

 bgcolor=atoi((char *)clientData);

 if(display_up){
    for(i=0;i<11;i++){
      if (i == bgcolor){
         XtSetSensitive(map_bgcolor[i],False);
      } else {
         XtSetSensitive(map_bgcolor[i],True);
      }
    }

    map_background_color=bgcolor;
    create_image(da);
    XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
  }
}


void Map_station_label(Widget w, XtPointer clientData, XtPointer calldata)
{int style;

 style=atoi((char *)clientData);

 if(display_up){
  letter_style = style;
  on_off_switch(letter_style,map_station_label1,map_station_label0);
  create_image(da);
  XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
 }
}

void Map_wx_alerts(Widget w, XtPointer clientData, XtPointer calldata)
{int style;

 style=atoi((char *)clientData);

 if(display_up){
  wx_alert_style = style;
  on_off_switch(wx_alert_style,map_wx_alerts_1,map_wx_alerts_0);
  create_image(da);
  XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
 }
}

void TNC_Logging_on(Widget w, XtPointer clientData, XtPointer calldata)
{
log_tnc_data=1;              /* toggle logging of TNC Data */
on_off_switch(log_tnc_data,tnc_logging_on,tnc_logging_off);         
}

void TNC_Logging_off(Widget w, XtPointer clientData, XtPointer calldata)
{
log_tnc_data=0;              /* toggle logging of TNC Data */
on_off_switch(log_tnc_data,tnc_logging_on,tnc_logging_off);      
}

void TNC_Transmit_now(Widget w, XtPointer clientData, XtPointer calldata)
{
transmit_now=1;              /* toggle transmission of station now*/
}



void Net_Logging_on(Widget w, XtPointer clientData, XtPointer calldata)
{
log_net_data=1;              /* toggle logging of net Data */
on_off_switch(log_net_data,net_logging_on,net_logging_off);         
}

void Net_Logging_off(Widget w, XtPointer clientData, XtPointer calldata)
{
log_net_data=0;              /* toggle logging of net Data */
on_off_switch(log_net_data,net_logging_on,net_logging_off);      
}


void IGate_Logging_on(Widget w, XtPointer clientData, XtPointer calldata)
{
log_igate=1;              /* toggle logging of igate Data */
on_off_switch(log_igate,igate_logging_on,igate_logging_off);         
}

void IGate_Logging_off(Widget w, XtPointer clientData, XtPointer calldata)
{
log_igate=0;              /* toggle logging of igate Data */
on_off_switch(log_igate,igate_logging_on,igate_logging_off);      
}

void WX_Logging_on(Widget w, XtPointer clientData, XtPointer calldata)
{
log_wx=1;              /* toggle logging of igate Data */
on_off_switch(log_wx,wx_logging_on,wx_logging_off);         
}

void WX_Logging_off(Widget w, XtPointer clientData, XtPointer calldata)
{
log_wx=0;              /* toggle logging of igate Data */
on_off_switch(log_wx,wx_logging_on,wx_logging_off);      
}

void Alt_on(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_alt_display=1;              /* toggle alt of station */   
redraw_on_new_data=2;
on_off_switch(symbol_alt_display,alt_on,alt_off);
}

void Alt_off(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_alt_display=0;              /* toggle alt of station */   
redraw_on_new_data=2;
on_off_switch(symbol_alt_display,alt_on,alt_off);
}


void Course_on(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_course_display=1;              /* toggle course of station */   
redraw_on_new_data=2;
on_off_switch(symbol_course_display,course_on,course_off);
}

void Course_off(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_course_display=0;              /* toggle course of station */   
redraw_on_new_data=2;
on_off_switch(symbol_course_display,course_on,course_off);
}

void Speed_on(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_speed_display=1;              /* toggle speed of station */   
redraw_on_new_data=2;
on_off_switch(symbol_speed_display,speed_on,speed_off);
}

void Speed_off(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_speed_display=0;              /* toggle speed of station */   
redraw_on_new_data=2;
on_off_switch(symbol_speed_display,speed_on,speed_off);
}

void Dist_course_on(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_dist_course_display=1;              /* toggle my distance and course to station */   
redraw_on_new_data=2;
on_off_switch(symbol_dist_course_display,dist_course_on,dist_course_off);
}

void Dist_course_off(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_dist_course_display=0;              /* toggle my distance and course to station */   
redraw_on_new_data=2;
on_off_switch(symbol_dist_course_display,dist_course_on,dist_course_off);
}

void Units_choice_on(Widget w, XtPointer clientData, XtPointer calldata)
{
units_english_metric=1;              /* toggle units to english*/   
redraw_on_new_data=2;
fill_wx_data(); 
on_off_switch(units_english_metric,units_choice_on,units_choice_off);
}

void Units_choice_off(Widget w, XtPointer clientData, XtPointer calldata)
{
units_english_metric=0;              /* toggle units to metric */   
redraw_on_new_data=2;
fill_wx_data();
on_off_switch(units_english_metric,units_choice_on,units_choice_off);
}


void Station_trails_on(Widget w, XtPointer clientData, XtPointer calldata)
{
station_trails=1;              /* toggle display of station trails*/   
redraw_on_new_data=2;
on_off_switch(station_trails,trails_on,trails_off);
}

void Station_trails_off(Widget w, XtPointer clientData, XtPointer calldata)
{
station_trails=0;              /* toggle display of station trails */   
redraw_on_new_data=2;
on_off_switch(station_trails,trails_on,trails_off);
}


void Station_phg_on(Widget w, XtPointer clientData, XtPointer calldata)
{
show_phg=1;              /* toggle display of station pg*/   
redraw_on_new_data=2;
on_off_switch(show_phg,phg_on,phg_off);
}

void Station_phg_off(Widget w, XtPointer clientData, XtPointer calldata)
{
show_phg=0;              /* toggle display of station pg */   
redraw_on_new_data=2;
on_off_switch(show_phg,phg_on,phg_off);
}

void WX_on(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_weather_display=1;              /* toggle display of station WX*/   
redraw_on_new_data=2;
on_off_switch(symbol_weather_display,wx_on,wx_off);
}

void WX_off(Widget w, XtPointer clientData, XtPointer calldata)
{
symbol_weather_display=0;              /* toggle display of station WX */   
redraw_on_new_data=2;
on_off_switch(symbol_weather_display,wx_on,wx_off);
}

void Help_About(Widget w, XtPointer clientData, XtPointer callData)
{static Widget dialog = NULL;
 char vers[1200];
  Arg al[4];                    /* Arg List */
  register int ac = 0;           /* Arg Count */ 

 sprintf(vers,"%s\n\n%s\n\n%s\n",VERSIONMESSAGE,ABOUTGNUL,XmVERSION_STRING);


if(!dialog)
   {/*set args for color */ 
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]);ac++;      
    dialog = XmCreateInformationDialog(hidden_shell,"About Message",al,ac);
    XtVaSetValues(dialog,XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL,
                  XmNdefaultPosition, True,
                  XmNnoResize, True,
                  XmNtitle,"Help About",
                  XmNdialogTitle,"Help About",
                  XtVaTypedArg,XmNmessageString,XmRString,
                  vers, strlen(vers)+1,
                  NULL);
   
   /*pos_dialog(dialog);*/

   XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
   XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
   }

 
 XtManageChild(dialog);
 /*fix_dialog_size(dialog);*/
}


Widget GetTopShell(Widget w)
{while(w && !XtIsWMShell(w))
      {w = XtParent(w);}
 return(w);
}

/*********************** Display incoming data*******************************/
/****************************************************************************/


void Display_data_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 Display_data_dialog = (Widget)NULL;
}

void  Display_packet_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {Display_packet_data_type = atoi(which);}
  else
   {Display_packet_data_type = 0;}
redraw_on_new_packet_data=1;
}


void Display_data(Widget w, XtPointer clientData, XtPointer callData)
{Widget pane, form, button_close, option_box, tnc_data, net_data, tnc_net_data;
 int n; 
 Arg args[20];
 Atom delw;

 if(!Display_data_dialog)   
   {Display_data_dialog = XtVaCreatePopupShell(langcode("WPUPDPD001"),xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, Display_data_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5,
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1,
                            NULL);

    n=0;
    XtSetArg(args[n], XmNrows, 15); n++;
    XtSetArg(args[n], XmNcolumns, 100); n++;
    XtSetArg(args[n], XmNeditable, False); n++;
    XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg(args[n], XmNwordWrap, True); n++;
    XtSetArg(args[n], XmNbackground, colors[0xff]); n++;
    XtSetArg(args[n], XmNscrollHorizontal, True); n++;
    XtSetArg(args[n], XmNscrollVertical, False); n++;
    XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
    XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNtopOffset, 5); n++;
    XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
    XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNleftOffset, 5); n++;
    XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg(args[n], XmNrightOffset, 5); n++;

    Display_data_text=NULL;
    Display_data_text = XmCreateScrolledText(form,"text",args,n);

    /* set colors */
    n=0;
    XtSetArg(args[n],XmNbackground, colors[0xff]); n++;
    option_box = XmCreateRadioBox(form,"option box",args,n);

    XtVaSetValues(option_box,
                  XmNpacking, XmPACK_TIGHT,
                  XmNorientation, XmHORIZONTAL,
                  XmNtopAttachment,XmATTACH_WIDGET,
                  XmNtopWidget, XtParent(Display_data_text),
		  XmNbottomAttachment,XmATTACH_NONE,
		  XmNleftAttachment, XmATTACH_FORM,
                  XmNleftOffset, 5,
                  XmNrightAttachment,XmATTACH_FORM,
                  XmNrightOffset, 5,
                  NULL);

     tnc_data = XtVaCreateManagedWidget(langcode("WPUPDPD002"),xmToggleButtonGadgetClass,
			                                option_box,
                                                        XmNbackground, colors[0xff],
       	                                                NULL);

     XtAddCallback(tnc_data,XmNvalueChangedCallback,Display_packet_toggle,"1");  

     net_data = XtVaCreateManagedWidget(langcode("WPUPDPD003"),xmToggleButtonGadgetClass,
			                                option_box,
                                                        XmNbackground, colors[0xff],
       	                                                NULL);

     XtAddCallback(net_data,XmNvalueChangedCallback,Display_packet_toggle,"2");  

     tnc_net_data = XtVaCreateManagedWidget(langcode("WPUPDPD004"),xmToggleButtonGadgetClass,
			                                option_box,
                                                        XmNbackground, colors[0xff],
	                                                NULL);

     XtAddCallback(tnc_net_data,XmNvalueChangedCallback,Display_packet_toggle,"0");  




     button_close = XtVaCreateManagedWidget(langcode("UNIOP00003"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, option_box,
                                      XmNtopOffset, 2,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 3,
                                      XmNbackground, colors[0xff],
                                      NULL);
 
    XtAddCallback(button_close, XmNactivateCallback, Display_data_destroy_shell, Display_data_dialog);

    pos_dialog(Display_data_dialog);

    delw = XmInternAtom(XtDisplay(Display_data_dialog),"WM_DELETE_WINDOW", False);
    XmAddWMProtocolCallback(Display_data_dialog, delw, Display_data_destroy_shell, (XtPointer)Display_data_dialog);

    switch(Display_packet_data_type)
          {case(0):XmToggleButtonSetState(tnc_net_data,True,False);
	           break;
           case(1):XmToggleButtonSetState(tnc_data,True,False);
	           break;
           case(2):XmToggleButtonSetState(net_data,True,False);
	           break;
           default:XmToggleButtonSetState(tnc_net_data,True,False);
	           break;

	  }

    XtManageChild(form);
    XtManageChild(option_box); 
    XtManageChild(Display_data_text); 
    XtVaSetValues(Display_data_text, XmNbackground, colors[0x0f], NULL); 
    XtManageChild(pane);

    redraw_on_new_packet_data=1;
    XtPopup(Display_data_dialog,XtGrabNone);
    fix_dialog_size(Display_data_dialog);    
   }  else {
     XRaiseWindow(XtDisplay(Display_data_dialog), XtWindow(Display_data_dialog));
  }
}


/****************************** Help menu ***********************************/
/****************************************************************************/


void help_view_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 help_view_dialog = (Widget)NULL;
}

void help_index_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;

 if (help_view_dialog)
    help_view_destroy_shell(help_view_dialog, help_view_dialog, NULL);

 XtPopdown(shell);
 XtDestroyWidget(shell);
 help_index_dialog = (Widget)NULL;
}



void help_view(Widget w, XtPointer clientData, XtPointer callData)
{Widget pane, form, button_close,help_text;
 int i,x,y,n; 
 char *temp;
 char title[200];
 char temp2[200];
 char temp3[200];
 FILE *f;
 XmString *list;
 int open;
 Arg args[20];
 int data_on,pos;
 int found;
 Atom delw;

 data_on=0;
 pos=0;
 found=0;

 XtVaGetValues(help_list,
               XmNitemCount,&i,
               XmNitems,&list,
               NULL);

 for(x=1; x<=i;x++)
    {if(XmListPosSelected(help_list,x))
       {found=1; 
        if(XmStringGetLtoR(list[(x-1)],XmFONTLIST_DEFAULT_TAG,&temp))
          {x=i+1;}	 
       }
    }
 open=0;

 if(found)
   {if(help_view_dialog)   
      {XtVaGetValues(help_view_dialog, XmNx, &x, XmNy, &y, NULL);
       help_view_destroy_shell(help_view_dialog, help_view_dialog, NULL);
       open=1;
      }

    if(!help_view_dialog)   
      {sprintf(title,"Help - %s",temp);
       help_view_dialog = XtVaCreatePopupShell(title,xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);
       pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, help_view_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

       form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane, 
                            XmNfractionBase, 5,
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1,
                            NULL);

       n=0;
       XtSetArg(args[n], XmNrows, 20); n++;
       XtSetArg(args[n], XmNcolumns, 80); n++;
       XtSetArg(args[n], XmNeditable, False); n++;
       XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
       XtSetArg(args[n], XmNwordWrap, True); n++;
       XtSetArg(args[n], XmNbackground, colors[0xff]); n++;
       XtSetArg(args[n], XmNscrollHorizontal, False); n++;
       XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
       XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
       XtSetArg(args[n], XmNtopOffset, 5); n++;
       XtSetArg(args[n], XmNbottomAttachment, XmATTACH_NONE); n++;
       XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
       XtSetArg(args[n], XmNleftOffset, 5); n++;
       XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
       XtSetArg(args[n], XmNrightOffset, 5); n++;

       help_text=NULL;
       help_text = XmCreateScrolledText(form,"help text",args,n);

       f=fopen(HELP_FILE,"r");
       if(f!=NULL)
         {while(!feof(f))
               {get_line(f,temp2,200);
                if(strncmp(temp2,"HELP-INDEX>",11)==0)
                  {if(strcmp((temp2+11),temp)==0)
                     {data_on=1;}
                     else
                     {data_on=0;}
		  }
                  else
                  {if(data_on)
                     {sprintf(temp3,"%s\n",temp2);
                      XmTextInsert(help_text,pos,temp3);
                      pos=pos+strlen(temp3);
                      XmTextShowPosition(help_text,0);
		     }
		  }
   
	       }
          fclose(f);
	 }
       

       button_close = XtVaCreateManagedWidget(langcode("UNIOP00003"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, XtParent(help_text),
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 3,
                                      NULL);
 
       XtAddCallback(button_close, XmNactivateCallback, help_view_destroy_shell, help_view_dialog);

       if(!open)
         {pos_dialog(help_view_dialog);}
         else
         {XtVaSetValues(help_view_dialog, XmNx, x, XmNy, y, NULL);}

       delw = XmInternAtom(XtDisplay(help_view_dialog),"WM_DELETE_WINDOW", False);
       XmAddWMProtocolCallback(help_view_dialog, delw, help_view_destroy_shell, (XtPointer)help_view_dialog);

       XtManageChild(form);
       XtManageChild(help_text); 
       XtVaSetValues(help_text, XmNbackground, colors[0x0f], NULL);
       XtManageChild(pane);
  
       XtPopup(help_view_dialog,XtGrabNone);
       fix_dialog_size(help_view_dialog);
       XmTextShowPosition(help_text,0);
      }
   }

}


void Help_Index(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane,form, button_ok, button_cancel;
 int n;
 char temp[600];
 FILE *f;
 Arg al[20];           /* Arg List */
 int ac = 0;           /* Arg Count */  

 Atom delw;
 XmString str_ptr;

 if(!help_index_dialog)   
   {help_index_dialog = XtVaCreatePopupShell(langcode("WPUPHPI001"),xmDialogShellWidgetClass,appshell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  XmNresize, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, help_index_dialog,
                          XmNbackground, colors[0xff],
                          NULL);
  
    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5, 
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1,
                            NULL);

    /*set args for color */ 
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      
    XtSetArg(al[ac], XmNvisibleItemCount, 11); ac++;      
    XtSetArg(al[ac], XmNtraversalOn, False); ac++;      
    XtSetArg(al[ac], XmNshadowThickness, 3); ac++;      
    XtSetArg(al[ac], XmNbackground, colors[0x0ff]); ac++;      
    XtSetArg(al[ac], XmNselectionPolicy, XmSINGLE_SELECT); ac++;      
    XtSetArg(al[ac], XmNscrollBarPlacement, XmBOTTOM_RIGHT); ac++;      
    XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;      
    XtSetArg(al[ac], XmNtopOffset, 5); ac++;      
    XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_NONE); ac++;      
    XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;      
    XtSetArg(al[ac], XmNrightOffset, 5); ac++;      
    XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;      
    XtSetArg(al[ac], XmNleftOffset, 5); ac++;      
                 
    help_list = XmCreateScrolledList(form,"list",al,ac);

    n=1;
    f=fopen(HELP_FILE,"r");
    if (f!=NULL) {
       while (!feof(f)) {
             get_line(f,temp,600);
             if (strncmp(temp,"HELP-INDEX>",11)==0) {
	        XmListAddItem(help_list, str_ptr = XmStringCreateLtoR((temp+11),XmFONTLIST_DEFAULT_TAG),n++);
	        XmStringFree(str_ptr);
	     }
       }
       fclose(f);
    }
 
    button_ok = XtVaCreateManagedWidget(langcode("WPUPHPI002"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, XtParent(help_list),
                                      XmNtopOffset,5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset,5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 1,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 2,
                                      NULL);

    button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00003"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, XtParent(help_list),
                                      XmNtopOffset,5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset,5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 3,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 4,
                                      NULL);
 
   XtAddCallback(button_cancel, XmNactivateCallback, help_index_destroy_shell, help_index_dialog);
   XtAddCallback(button_ok, XmNactivateCallback, help_view, NULL);

   pos_dialog(help_index_dialog); 

   delw = XmInternAtom(XtDisplay(help_index_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(help_index_dialog, delw, help_index_destroy_shell, (XtPointer)help_index_dialog);

   XtManageChild(form);
   XtManageChild(help_list);
   XtVaSetValues(help_list, XmNbackground, colors[0x0f], NULL);
   XtManageChild(pane);
    
   XtPopup(help_index_dialog,XtGrabNone);
   fix_dialog_size(help_index_dialog);
   } else {
     XtPopup(help_index_dialog,XtGrabNone);
     XRaiseWindow(XtDisplay(help_index_dialog), XtWindow(help_index_dialog));
  }
}


/************************** Clear stations *******************************/
/*************************************************************************/

void Stations_Clear(Widget w, XtPointer clientData, XtPointer callData)
{
clean_data_file();
my_station_add(my_callsign,my_group,my_symbol,my_long,my_lat,my_phg,my_comment);   
redraw_on_new_data=2;
}



/************************* Map Chooser ***********************************/
/*************************************************************************/

void map_chooser_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 map_chooser_dialog = (Widget)NULL;
}

void map_chooser_select_maps(Widget widget, XtPointer clientData, XtPointer callData)
{int i,x; 
 char *temp;
 XmString *list;
 FILE *f;

 XtVaGetValues(map_list,
               XmNitemCount,&i,
               XmNitems,&list,
               NULL);

f=fopen(WIN_MAP_DATA,"w");
if(f!=NULL)
  {for(x=1; x<=i;x++)
      {if(XmListPosSelected(map_list,x))
         {if(XmStringGetLtoR(list[(x-1)],XmFONTLIST_DEFAULT_TAG,&temp))
            {fprintf(f,"%s\n",temp);}
	 }
      }
  }
if(f!=NULL) 
  {fclose(f);}
map_chooser_destroy_shell(widget,clientData,callData);
create_image(da);
XCopyArea(XtDisplay(da),pixmap_final,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
}

void dir_sort(char *dir)
{struct dirent *dl;
 DIR *dm;
 char fullpath[8000];
 struct stat nfile;
 const time_t *ftime; 

 int size;
 size=strlen(WIN_MAP_DIR)+1;

 dm = opendir(dir);
 if(!dm)
   {perror("map");}
   else
   {while((dl = readdir(dm)))
         {sprintf(fullpath,"%s/%s",dir,dl->d_name);
          if(stat(fullpath,&nfile)==0)
            {ftime=&nfile.st_ctime;
             switch(nfile.st_mode & S_IFMT)
                   {case(S_IFDIR):/*printf("file %c letter %c\n",dl->d_name[0],letter);*/
                                  if((strcmp(dl->d_name,".") !=0) && (strcmp(dl->d_name,"..") !=0))
                                    {dir_sort(fullpath);}
                                  break;
                    case(S_IFREG):/*printf("FILE %d<%s>\n",*item,fullpath+size);*/
                                  sort_input_database(get_user_base_dir("data/sort_maps_db.dat"),(fullpath+size),1000);
                                  break;
                    default:break;
		   }
	    }
		  
	 }
      }
 closedir(dm);
}

void sort_list(char *filename,int size, Widget list, int *item)
{FILE *f_data;
 FILE *f_pointer;
 char fill[2000];
 long file_ptr;
 long i;
 long ptr;
 char ptr_filename[400];
 XmString str_ptr;

 sprintf(ptr_filename,"%s-ptr",filename);
 i=0l;
 f_pointer=fopen(ptr_filename,"r");
 f_data=fopen(filename,"r");
 if(f_pointer!=NULL && f_data !=NULL)
   {while(!feof(f_pointer))
         {ptr=ftell(f_pointer);
	  if(fread(&file_ptr,sizeof(file_ptr),1,f_pointer)==1)
	    {fseek(f_data,file_ptr,SEEK_SET);
	     if(fread(fill,size,1,f_data)==1)
	       {XmListAddItem(list, str_ptr = XmStringCreateLtoR(fill,XmFONTLIST_DEFAULT_TAG),*item);
	        XmStringFree(str_ptr);
                *item=*item+1;
	       }
	     i++;
	    }
	 }
   }
 if(f_pointer!=NULL)
   {fclose(f_pointer);}
 if(f_data!=NULL)
   {fclose(f_data);}
}                                                      

void Map_chooser(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane, form, button_ok, button_cancel, mess;
 Atom delw;
 int n,i;
 FILE *f;
 char temp[600];
 XmString str_ptr;
 Arg al[10];                    /* Arg List */
 register int ac = 0;           /* Arg Count */     

 i=0;
 if(!map_chooser_dialog)   
   {map_chooser_dialog = XtVaCreatePopupShell(langcode("WPUPMCP001"),xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, map_chooser_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5,
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1,  
                            NULL);
   
    /*set args for color */ 
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      
    XtSetArg(al[ac], XmNvisibleItemCount, 13); ac++;      
    XtSetArg(al[ac], XmNtraversalOn, False); ac++;      
    XtSetArg(al[ac], XmNshadowThickness, 3); ac++;      
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      
    XtSetArg(al[ac], XmNselectionPolicy, XmMULTIPLE_SELECT); ac++;      
    XtSetArg(al[ac], XmNscrollBarPlacement, XmBOTTOM_RIGHT); ac++;      

    map_list = XmCreateScrolledList(form,"list",al,ac);

    n=1;
    clear_sort_file(get_user_base_dir("data/sort_maps_db.dat"));
    dir_sort(WIN_MAP_DIR);
    sort_list(get_user_base_dir("data/sort_maps_db.dat"),1000,map_list,&n);
    f=fopen(WIN_MAP_DATA,"r");
    if (f!=NULL) {
       while(!feof(f)) {
            {get_line(f,temp,600);
             i=XmListItemPos(map_list, str_ptr = XmStringCreateLtoR(temp,XmFONTLIST_DEFAULT_TAG));
	     XmStringFree(str_ptr);
             if (i!=0)
                XmListSelectPos(map_list,i,True);
	    }
       }
       fclose(f);
    }

    mess = XtVaCreateManagedWidget(langcode("WPUPMCP002"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_FORM,
                                      XmNrightOffset, 5,
                                      XmNbackground, colors[0xff], 
                                      NULL);

    XtVaSetValues(XtParent(map_list),
                  XmNtopAttachment, XmATTACH_WIDGET,
                  XmNtopWidget,mess,
                  XmNtopOffset, 5,
                  XmNbottomAttachment, XmATTACH_NONE,
                  XmNrightAttachment, XmATTACH_FORM,
                  XmNrightOffset, 5,
                  XmNleftAttachment, XmATTACH_FORM,
                  XmNleftOffset, 5,
                  NULL);
  
    button_ok = XtVaCreateManagedWidget(langcode("UNIOP00001"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget,XtParent(map_list),
                                      XmNtopOffset,5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 1,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 2,
                                      XmNbackground, colors[0xff], 
                                      NULL);

    button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget,XtParent(map_list),
                                      XmNtopOffset,5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 3,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 4,
                                      XmNbackground, colors[0xff], 
                                      NULL);
 
   XtAddCallback(button_cancel, XmNactivateCallback, map_chooser_destroy_shell, map_chooser_dialog);
   XtAddCallback(button_ok, XmNactivateCallback, map_chooser_select_maps, map_chooser_dialog);

   pos_dialog(map_chooser_dialog);  

   delw = XmInternAtom(XtDisplay(map_chooser_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(map_chooser_dialog, delw, map_chooser_destroy_shell, (XtPointer)map_chooser_dialog);

   XtManageChild(form);
   XtManageChild(map_list);   
   XtVaSetValues(map_list, XmNbackground, colors[0x0f], NULL);
   XtManageChild(pane);

   XtPopup(map_chooser_dialog,XtGrabNone);
   fix_dialog_size(map_chooser_dialog);
  } else {
     XRaiseWindow(XtDisplay(map_chooser_dialog), XtWindow(map_chooser_dialog));
  }
}


/****** Read in file **********/

void read_file_selection_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtDestroyWidget(shell);
 read_selection_dialog = (Widget)NULL;
}

void read_file_selection_now(Widget w, XtPointer clientData, XtPointer callData)
{
char *file;
XmFileSelectionBoxCallbackStruct *cbs =(XmFileSelectionBoxCallbackStruct*)callData;


if(XmStringGetLtoR(cbs->value,XmFONTLIST_DEFAULT_TAG,&file))
  {/*printf("FILE is %s\n",file);*/
   if(!read_file)
     {/* do read file start */
      read_file_ptr=fopen(file,"r");
      if(read_file_ptr!=NULL)
        {read_file=1;
         read_file_delay_count=0;
         read_file_delay=0;   
	}
        else
        {fclose(read_file_ptr);}
     }
  }
read_file_selection_destroy_shell(w, clientData, callData);
}


void Read_File_Selection(Widget w, XtPointer clientData, XtPointer callData)
{Arg al[10];                    /* Arg List */
 register int ac = 0;           /* Arg Count */
 Widget fs;

 if(read_selection_dialog!=NULL)
   {read_file_selection_destroy_shell(read_selection_dialog, read_selection_dialog, NULL);}
 
 if(read_selection_dialog==NULL)
   {/*set args for color */ 
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      
    read_selection_dialog = XmCreateFileSelectionDialog(hidden_shell,"filesb",al,ac);

    fs=XmFileSelectionBoxGetChild(read_selection_dialog,XmDIALOG_TEXT);
    XtVaSetValues(fs,XmNbackground, colors[0x0f],NULL); 

    fs=XmFileSelectionBoxGetChild(read_selection_dialog,XmDIALOG_FILTER_TEXT);
    XtVaSetValues(fs,XmNbackground, colors[0x0f],NULL); 

    fs=XmFileSelectionBoxGetChild(read_selection_dialog,XmDIALOG_DIR_LIST);
    XtVaSetValues(fs,XmNbackground, colors[0x0f],NULL); 

    fs=XmFileSelectionBoxGetChild(read_selection_dialog,XmDIALOG_LIST);
    XtVaSetValues(fs,XmNbackground, colors[0x0f],NULL); 

    XtAddCallback(read_selection_dialog, XmNcancelCallback,read_file_selection_destroy_shell,read_selection_dialog);     
    XtAddCallback(read_selection_dialog, XmNokCallback,read_file_selection_now,read_selection_dialog);     

    XtManageChild(read_selection_dialog);
  }
}


/****** TEST **********/

void Test(Widget w, XtPointer clientData, XtPointer callData)
{

}

/****** Save Config data **********/

void Save_Config(Widget w, XtPointer clientData, XtPointer callData)
{
save_data();
}


/**** DEFAULTS CONFIGURE ******/

void defaults_old_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {old_station_item = atoi(which);}
  else
   {old_station_item = 0;}

}

void defaults_clear_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {clear_station_item = atoi(which);}
  else
   {clear_station_item = 0;}

}

void Configure_defaults_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 configure_defaults_dialog = (Widget)NULL;
}


void Configure_defaults_change_data(Widget widget, XtPointer clientData, XtPointer callData)
{
switch(old_station_item)
      {case(1):sec_old=900l;
               break;

       case(2):sec_old=1800l;
               break;

       case(3):sec_old=2700l;
               break;

       case(4):sec_old=3600l;
               break;

       case(5):sec_old=5400l;
               break;

       case(6):sec_old=7200l;
               break;

       default:break;
      }

switch(clear_station_item)
      {case(1):sec_clear=21600l;
               break;

       case(2):sec_clear=43200l;
               break;

       case(3):sec_clear=86400l;
               break;

       case(4):sec_clear=172800l;
               break;

       case(5):sec_clear=604800l;
               break;

       default:break;
      }

switch(GPS_time)
      {case(10):gps_time=10l;
               break;

       case(20):gps_time=20l;
               break;

       case(30):gps_time=30l;
               break;

       case(60):gps_time=60l;
               break;

       case(120):gps_time=120l;
               break;

       case(300):gps_time=300l;
               break;

       case(600):gps_time=600l;
               break;

       default:gps_time=30l;
               break;
      }

output_station_type=Station_transmit_type;

if(XmToggleButtonGetState(raw_wx_tx))
  {transmit_raw_wx=1;}
  else
  {transmit_raw_wx=0;} 

if(XmToggleButtonGetState(compressed_posit_tx))
  {transmit_compressed_posit=1;}
  else
  {transmit_compressed_posit=0;} 

operate_as_an_igate=Igate_type;

redraw_on_new_data=2;
Configure_defaults_destroy_shell(widget,clientData,callData);
}

/* GPS sample time radio buttons */
void gps_time_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {GPS_time = atoi(which);}
  else
   {GPS_time = 30l;}

}

/* Station_transmit type radio buttons */
void station_type_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {Station_transmit_type = atoi(which);}
  else
   {Station_transmit_type = 0;}

}

/* Igate type radio buttons */
void igate_type_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {Igate_type = atoi(which);}
  else
   {Igate_type = 0;}

}

void Configure_defaults(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane, form, button_ok, button_cancel, frame, frame2, frame3, frame4, frame5,
                time, time_box, 
                time_15, time_30, time_45, time_60, time_90, time_120,
                time2, time_box2,
                time_6h, time_12h, time_24h, time_48h, time_168h,
                time3, time_box3,
                toption1, toption2, toption3, toption4, toption5, toption6, toption7,
                station_type, type_box,
                styp1, styp2, styp3, styp4, styp5, styp6,
                igate_option, igate_box, 
                igtyp0, igtyp1, igtyp2;

 Atom delw;
 Arg al[2];                    /* Arg List */
 register int ac = 0;           /* Arg Count */  

 if(!configure_defaults_dialog)   
   {configure_defaults_dialog = XtVaCreatePopupShell(langcode("WPUPCFD001"),xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, configure_defaults_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5, 
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1,
                            NULL);


    frame = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_FORM,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    time = XtVaCreateManagedWidget(langcode("WPUPCFD002"),xmLabelWidgetClass,frame,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);
    /*set args for color */ 
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++;      

    time_box = XmCreateRadioBox(frame,"Time box",al,ac);
   
    XtVaSetValues(time_box,
                  XmNnumColumns,4,
                  NULL);


    time_15 = XtVaCreateManagedWidget(langcode("WPUPCFD003"),xmToggleButtonGadgetClass,
                                        time_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_15,XmNvalueChangedCallback,defaults_old_toggle,"1");


    time_30 = XtVaCreateManagedWidget(langcode("WPUPCFD004"),xmToggleButtonGadgetClass,
                                        time_box, 
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_30,XmNvalueChangedCallback,defaults_old_toggle,"2");


    time_45 = XtVaCreateManagedWidget(langcode("WPUPCFD005"),xmToggleButtonGadgetClass,
                                        time_box, 
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_45,XmNvalueChangedCallback,defaults_old_toggle,"3");


    time_60 = XtVaCreateManagedWidget(langcode("WPUPCFD006"),xmToggleButtonGadgetClass,
                                        time_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_60,XmNvalueChangedCallback,defaults_old_toggle,"4");


    time_90 = XtVaCreateManagedWidget(langcode("WPUPCFD007"),xmToggleButtonGadgetClass,
                                        time_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_90,XmNvalueChangedCallback,defaults_old_toggle,"5");


    time_120 = XtVaCreateManagedWidget(langcode("WPUPCFD008"),xmToggleButtonGadgetClass,
                                        time_box, 
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_120,XmNvalueChangedCallback,defaults_old_toggle,"6");


    frame2 = XtVaCreateManagedWidget("frame2", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, frame,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    time2 = XtVaCreateManagedWidget(langcode("WPUPCFD009"),xmLabelWidgetClass,frame2,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    time_box2 = XmCreateRadioBox(frame2,"Time box2",al,ac);

    XtVaSetValues(time_box2,
                  XmNnumColumns,5,
                  NULL);

    time_6h = XtVaCreateManagedWidget(langcode("WPUPCFD010"),xmToggleButtonGadgetClass,
                                        time_box2,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_6h,XmNvalueChangedCallback,defaults_clear_toggle,"1");


    time_12h = XtVaCreateManagedWidget(langcode("WPUPCFD011"),xmToggleButtonGadgetClass,
                                        time_box2, 
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_12h,XmNvalueChangedCallback,defaults_clear_toggle,"2");


    time_24h = XtVaCreateManagedWidget(langcode("WPUPCFD012"),xmToggleButtonGadgetClass,
                                        time_box2, 
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_24h,XmNvalueChangedCallback,defaults_clear_toggle,"3");


    time_48h = XtVaCreateManagedWidget(langcode("WPUPCFD013"),xmToggleButtonGadgetClass,
                                        time_box2,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_48h,XmNvalueChangedCallback,defaults_clear_toggle,"4");


    time_168h = XtVaCreateManagedWidget(langcode("WPUPCFD014"),xmToggleButtonGadgetClass,
                                        time_box2,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(time_168h,XmNvalueChangedCallback,defaults_clear_toggle,"5");


    frame3 = XtVaCreateManagedWidget("frame3", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, frame2,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    time3  = XtVaCreateManagedWidget(langcode("WPUPCFG008"),xmLabelWidgetClass,frame3,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    time_box3 = XmCreateRadioBox(frame3,"Time Options box",al,ac);

    XtVaSetValues(time_box3,
                  XmNnumColumns,4,
                  NULL);

    toption1 = XtVaCreateManagedWidget(langcode("WPUPCFG009"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption1,XmNvalueChangedCallback,gps_time_toggle,"10");


    toption2 = XtVaCreateManagedWidget(langcode("WPUPCFG010"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption2,XmNvalueChangedCallback,gps_time_toggle,"20");


    toption3 = XtVaCreateManagedWidget(langcode("WPUPCFG011"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption3,XmNvalueChangedCallback,gps_time_toggle,"30");

    toption4 = XtVaCreateManagedWidget(langcode("WPUPCFG012"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption4,XmNvalueChangedCallback,gps_time_toggle,"60");

    toption5 = XtVaCreateManagedWidget(langcode("WPUPCFG013"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption5,XmNvalueChangedCallback,gps_time_toggle,"120");

    toption6 = XtVaCreateManagedWidget(langcode("WPUPCFG014"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption6,XmNvalueChangedCallback,gps_time_toggle,"300");

    toption7 = XtVaCreateManagedWidget(langcode("WPUPCFG015"),xmToggleButtonGadgetClass,
                                        time_box3,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(toption7,XmNvalueChangedCallback,gps_time_toggle,"600");


    frame4 = XtVaCreateManagedWidget("frame4", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, frame3,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    station_type  = XtVaCreateManagedWidget(langcode("WPUPCFD015"),xmLabelWidgetClass,frame4,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    type_box = XmCreateRadioBox(frame4,"Transmit Options box",al,ac);

    XtVaSetValues(type_box,
                  XmNnumColumns,2,
                  NULL);

    styp1 = XtVaCreateManagedWidget(langcode("WPUPCFD016"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp1,XmNvalueChangedCallback,station_type_toggle,"0");

    styp2 = XtVaCreateManagedWidget(langcode("WPUPCFD017"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp2,XmNvalueChangedCallback,station_type_toggle,"1");

    styp3 = XtVaCreateManagedWidget(langcode("WPUPCFD018"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp3,XmNvalueChangedCallback,station_type_toggle,"2");

    styp4 = XtVaCreateManagedWidget(langcode("WPUPCFD019"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp4,XmNvalueChangedCallback,station_type_toggle,"3");

    styp5 = XtVaCreateManagedWidget(langcode("WPUPCFD021"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp5,XmNvalueChangedCallback,station_type_toggle,"4");

    styp6 = XtVaCreateManagedWidget(langcode("WPUPCFD022"),xmToggleButtonGadgetClass,
                                        type_box,
                                        XmNbackground, colors[0xff],
                                        NULL);
    XtAddCallback(styp6,XmNvalueChangedCallback,station_type_toggle,"5");


    compressed_posit_tx = XtVaCreateManagedWidget(langcode("WPUPCFD024"),xmToggleButtonWidgetClass,form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, frame4,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset,10,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    raw_wx_tx  = XtVaCreateManagedWidget(langcode("WPUPCFD023"),xmToggleButtonWidgetClass,form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, compressed_posit_tx,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset,10,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    frame5 = XtVaCreateManagedWidget("frame5", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, raw_wx_tx,
                                    XmNtopOffset,3,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    igate_option  = XtVaCreateManagedWidget(langcode("IGPUPCF000"),xmLabelWidgetClass,frame5,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    igate_box = XmCreateRadioBox(frame5,"Igate Options box",al,ac);

    XtVaSetValues(igate_box,
                  XmNnumColumns,2,
                  NULL);

    igtyp0 = XtVaCreateManagedWidget(langcode("IGPUPCF001"),xmToggleButtonGadgetClass,
                                        igate_box,
                                        XmNbackground, colors[0xff],
                                        NULL);

    XtAddCallback(igtyp0,XmNvalueChangedCallback,igate_type_toggle,"0");

    igtyp1 = XtVaCreateManagedWidget(langcode("IGPUPCF002"),xmToggleButtonGadgetClass,
                                        igate_box,
                                        XmNbackground, colors[0xff],
                                        NULL);

    XtAddCallback(igtyp1,XmNvalueChangedCallback,igate_type_toggle ,"1");

    igtyp2 = XtVaCreateManagedWidget(langcode("IGPUPCF003"),xmToggleButtonGadgetClass,
                                        igate_box,
                                        XmNbackground, colors[0xff],
                                        NULL);

    XtAddCallback(igtyp2,XmNvalueChangedCallback,igate_type_toggle,"2");

    button_ok = XtVaCreateManagedWidget(langcode("UNIOP00001"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, frame5, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 1,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 2,
                                      XmNbackground, colors[0xff],
                                      NULL);


    button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, frame5, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 3,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 4,
                                      XmNbackground, colors[0xff],
                                      NULL);

   XtAddCallback(button_ok, XmNactivateCallback, Configure_defaults_change_data, configure_defaults_dialog);
   XtAddCallback(button_cancel, XmNactivateCallback, Configure_defaults_destroy_shell, configure_defaults_dialog);

   switch(sec_old)
         {case(900l):XmToggleButtonSetState(time_15,True,False);
                     old_station_item=1; 
                     break;

          case(1800l):XmToggleButtonSetState(time_30,True,False);
                     old_station_item=2; 
                     break;

          case(2700l):XmToggleButtonSetState(time_45,True,False);
                     old_station_item=3; 
                     break;

          case(3600l):XmToggleButtonSetState(time_60,True,False);
                     old_station_item=4;  
                     break;

          case(5400l):XmToggleButtonSetState(time_90,True,False);
                     old_station_item=5; 
                     break;

          case(7200l):XmToggleButtonSetState(time_120,True,False);
                     old_station_item=6; 
                     break;

          default:XmToggleButtonSetState(time_120,True,False);
                  old_station_item=6; 
                  break;
	 }

   switch(sec_clear)
         {case(21600l):XmToggleButtonSetState(time_6h,True,False);
                     clear_station_item=1; 
                     break;

          case(43200l):XmToggleButtonSetState(time_12h,True,False);
                     clear_station_item=2; 
                     break;

          case(86400l):XmToggleButtonSetState(time_24h,True,False);
                     clear_station_item=3; 
                     break;

          case(172800l):XmToggleButtonSetState(time_48h,True,False);
                     clear_station_item=4;  
                     break;

          case(604800l):XmToggleButtonSetState(time_168h,True,False);
                     clear_station_item=5; 
                     break;

          default:XmToggleButtonSetState(time_168h,True,False);
                  clear_station_item=5; 
                  break;
	 }

   switch(gps_time)
         {case(10):XmToggleButtonSetState(toption1,True,False);
                     GPS_time=10l;
                     break;

          case(20):XmToggleButtonSetState(toption2,True,False);
                     GPS_time=20l;
                     break;

          case(30):XmToggleButtonSetState(toption3,True,False);
                     GPS_time=30l;
                     break;

          case(60):XmToggleButtonSetState(toption4,True,False);
                     GPS_time=60l;
                     break;

          case(120):XmToggleButtonSetState(toption5,True,False);
                     GPS_time=120l;
                     break;

          case(300):XmToggleButtonSetState(toption6,True,False);
                     GPS_time=300l;
                     break;

          case(600):XmToggleButtonSetState(toption7,True,False);
                     GPS_time=600l;
                     break;

          default:XmToggleButtonSetState(toption1,True,False);
                  GPS_time=30;
                  break;
	 }

   switch(output_station_type)
         {case(0):XmToggleButtonSetState(styp1,True,False);
                  Station_transmit_type=0;
                  break;

          case(1):XmToggleButtonSetState(styp2,True,False);
                  Station_transmit_type=1;
                  break;

          case(2):XmToggleButtonSetState(styp3,True,False);
                  Station_transmit_type=2;
                  break;

          case(3):XmToggleButtonSetState(styp4,True,False);
                  Station_transmit_type=3;
                  break;

          case(4):XmToggleButtonSetState(styp5,True,False);
                  Station_transmit_type=4;
                  break;

          case(5):XmToggleButtonSetState(styp6,True,False);
                  Station_transmit_type=5;
                  break;

          default:XmToggleButtonSetState(styp1,True,False);
                  Station_transmit_type=0;
                  break;

	 }

   if(transmit_raw_wx)
     {XmToggleButtonSetState(raw_wx_tx,True,False);}
     else
     {XmToggleButtonSetState(raw_wx_tx,False,False);}

   if(transmit_compressed_posit)
     {XmToggleButtonSetState(compressed_posit_tx,True,False);}
     else
     {XmToggleButtonSetState(compressed_posit_tx,False,False);}

   switch(operate_as_an_igate)
         {case(0):XmToggleButtonSetState(igtyp0,True,False);
                  Igate_type=0;
                  break;

          case(1):XmToggleButtonSetState(igtyp1,True,False);
                  Igate_type=1;
                  break;

          case(2):XmToggleButtonSetState(igtyp2,True,False);
                  Igate_type=2;
                  break;

          default:XmToggleButtonSetState(igtyp0,True,False);
                  Igate_type=0;
                  break;
	 }
   pos_dialog(configure_defaults_dialog);

   delw = XmInternAtom(XtDisplay(configure_defaults_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(configure_defaults_dialog, delw, Configure_defaults_destroy_shell, (XtPointer)configure_defaults_dialog);

   XtManageChild(form);  
   XtManageChild(time_box);  
   XtManageChild(time_box2);  
   XtManageChild(time_box3);  
   XtManageChild(type_box);  
   XtManageChild(igate_box);  
   XtManageChild(pane);  

   XtPopup(configure_defaults_dialog,XtGrabNone);
   fix_dialog_size(configure_defaults_dialog);

  } else {
     XRaiseWindow(XtDisplay(configure_defaults_dialog), XtWindow(configure_defaults_dialog));
  }
}



/**** AUDIO ALARM CONFIGURE ******/

void Configure_audio_alarm_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 configure_audio_alarm_dialog = (Widget)NULL;
}


void Configure_audio_alarm_change_data(Widget widget, XtPointer clientData, XtPointer callData)
{
strcpy(sound_command,XmTextFieldGetString(audio_alarm_config_play_data));
remove_trailing_spaces(sound_command);
strcpy(sound_new_station,XmTextFieldGetString(audio_alarm_config_play_ons_data));
remove_trailing_spaces(sound_new_station);
strcpy(sound_new_message,XmTextFieldGetString(audio_alarm_config_play_onm_data));
remove_trailing_spaces(sound_new_message);
strcpy(sound_prox_message,XmTextFieldGetString(audio_alarm_config_play_onpx_data));
remove_trailing_spaces(sound_prox_message);
strcpy(prox_min,XmTextFieldGetString(prox_min_data));
remove_trailing_spaces(prox_min);
strcpy(prox_max,XmTextFieldGetString(prox_max_data));
remove_trailing_spaces(prox_max);
strcpy(sound_band_open_message,XmTextFieldGetString(audio_alarm_config_play_onbo_data));
remove_trailing_spaces(sound_band_open_message);
strcpy(bando_min,XmTextFieldGetString(bando_min_data));
remove_trailing_spaces(bando_min);
strcpy(bando_max,XmTextFieldGetString(bando_max_data));
remove_trailing_spaces(bando_max);
strcpy(sound_wx_alert_message,XmTextFieldGetString(audio_alarm_config_wx_alert_data));
remove_trailing_spaces(sound_wx_alert_message);

if(XmToggleButtonGetState(audio_alarm_config_play_on_new_station))
  {sound_play_new_station=1;}
  else
  {sound_play_new_station=0;}

if(XmToggleButtonGetState(audio_alarm_config_play_on_new_message))
  {sound_play_new_message=1;}
  else
  {sound_play_new_message=0;}

if(XmToggleButtonGetState(audio_alarm_config_play_on_prox))
  {sound_play_prox_message=1;}
  else
  {sound_play_prox_message=0;}

if(XmToggleButtonGetState(audio_alarm_config_play_on_bando))
  {sound_play_band_open_message=1;}
  else
  {sound_play_band_open_message=0;}

if(XmToggleButtonGetState(audio_alarm_config_play_on_wx_alert))
  {sound_play_wx_alert_message=1;}
  else
  {sound_play_wx_alert_message=0;}

Configure_audio_alarm_destroy_shell(widget,clientData,callData);
}


void Configure_audio_alarms(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane, form, button_ok, button_cancel, 
                audio_play, file1, file2, 
                min1, min2, max1, max2,
                minb1, minb2, maxb1, maxb2,
                sep;

 Atom delw;

 if(!configure_audio_alarm_dialog)   
   {configure_audio_alarm_dialog = XtVaCreatePopupShell(langcode("WPUPCFA001"),xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, configure_audio_alarm_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5,
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1, 
                            NULL);

    audio_play = XtVaCreateManagedWidget(langcode("WPUPCFA002"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    audio_alarm_config_play_data = XtVaCreateManagedWidget("Play Command", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 25,
                                      XmNwidth, ((25*7)+2),
                                      XmNmaxLength, 80,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, audio_play,
                                      XmNrightAttachment,XmATTACH_FORM,
                                      XmNrightOffset, 10,
                                      NULL);


    file1 = XtVaCreateManagedWidget(langcode("WPUPCFA003"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_play,
                                      XmNtopOffset, 20,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    file2 = XtVaCreateManagedWidget(langcode("WPUPCFA004"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_play,
                                      XmNtopOffset, 20,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    audio_alarm_config_play_on_new_station  = XtVaCreateManagedWidget(langcode("WPUPCFA005"),xmToggleButtonWidgetClass,form,  
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, file1,
                                      XmNtopOffset, 10, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset ,10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);  

     
    audio_alarm_config_play_ons_data = XtVaCreateManagedWidget("Play Command NS", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 25,
                                      XmNwidth, ((25*7)+2),
                                      XmNmaxLength, 80,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, file2,
                                      XmNtopOffset, 5, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,                                
                                      XmNleftWidget, audio_alarm_config_play_on_new_station,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    audio_alarm_config_play_on_new_message  = XtVaCreateManagedWidget(langcode("WPUPCFA006"),xmToggleButtonWidgetClass,form,  
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_new_station,
                                      XmNtopOffset, 10, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset ,10,                                
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);  

    audio_alarm_config_play_onm_data = XtVaCreateManagedWidget("Play Command NM", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 25,
                                      XmNwidth, ((25*7)+2),
                                      XmNmaxLength, 80,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_new_station,
                                      XmNtopOffset, 5, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    audio_alarm_config_play_on_prox  = XtVaCreateManagedWidget(langcode("WPUPCFA007"),xmToggleButtonWidgetClass,form,  
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_new_message,
                                      XmNtopOffset, 10, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset ,10,                                
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);  

    audio_alarm_config_play_onpx_data = XtVaCreateManagedWidget("Play Command PROX", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 25,
                                      XmNwidth, ((25*7)+2),
                                      XmNmaxLength, 80,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_new_message,
                                      XmNtopOffset, 5, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);


    min1 = XtVaCreateManagedWidget(langcode("WPUPCFA009"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_prox,
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset, 30,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    prox_min_data = XtVaCreateManagedWidget("prox min", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 10,
                                      XmNwidth, ((10*7)+2),
                                      XmNmaxLength, 20,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_onpx_data,
                                      XmNtopOffset, 2, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    min2 = XtVaCreateManagedWidget(units_english_metric?langcode("UNIOP00004"):langcode("UNIOP00005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_prox,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, prox_min_data,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    max1 = XtVaCreateManagedWidget(langcode("WPUPCFA010"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, min1,
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset, 30,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    prox_max_data = XtVaCreateManagedWidget("prox max", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 10,
                                      XmNwidth, ((10*7)+2),
                                      XmNmaxLength, 20,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, prox_min_data,
                                      XmNtopOffset, 2, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    max2 = XtVaCreateManagedWidget(units_english_metric?langcode("UNIOP00004"):langcode("UNIOP00005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, min1,
                                      XmNtopOffset, 14,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, prox_max_data,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    audio_alarm_config_play_on_bando  = XtVaCreateManagedWidget(langcode("WPUPCFA008"),xmToggleButtonWidgetClass,form,  
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, max1,
                                      XmNtopOffset, 12, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset ,10,                                
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);  

    audio_alarm_config_play_onbo_data = XtVaCreateManagedWidget("Play Command BAND", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 25,
                                      XmNwidth, ((25*7)+2),
                                      XmNmaxLength, 80,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, prox_max_data,
                                      XmNtopOffset, 5, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);


    minb1 = XtVaCreateManagedWidget(langcode("WPUPCFA009"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_bando,
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset, 30,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    bando_min_data = XtVaCreateManagedWidget("bando min", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 12,
                                      XmNwidth, ((10*7)+2),
                                      XmNmaxLength, 20,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_onbo_data,
                                      XmNtopOffset, 2, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    minb2 = XtVaCreateManagedWidget(units_english_metric?langcode("UNIOP00004"):langcode("UNIOP00005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_bando,
                                      XmNtopOffset, 14,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, bando_min_data,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    maxb1 = XtVaCreateManagedWidget(langcode("WPUPCFA010"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, minb1,
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 0,
                                      XmNleftOffset, 30,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    bando_max_data = XtVaCreateManagedWidget("bando max", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 10,
                                      XmNwidth, ((10*7)+2),
                                      XmNmaxLength, 20,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, bando_min_data,
                                      XmNtopOffset, 2, 
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    maxb2 = XtVaCreateManagedWidget(units_english_metric?langcode("UNIOP00004"):langcode("UNIOP00005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, minb1,
                                      XmNtopOffset, 14,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, bando_max_data,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    audio_alarm_config_play_on_wx_alert  = XtVaCreateManagedWidget(langcode("WPUPCFA011"),xmToggleButtonWidgetClass,form,
		                      XmNtopAttachment, XmATTACH_WIDGET,
		                      XmNtopWidget, maxb2,
		                      XmNtopOffset, 12,
		                      XmNbottomAttachment, XmATTACH_NONE,
		                      XmNleftAttachment, XmATTACH_POSITION,
		                      XmNleftPosition, 0,
		                      XmNleftOffset, 10,
		                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
		                      NULL);

    audio_alarm_config_wx_alert_data = XtVaCreateManagedWidget("Play Command WxAlert", xmTextFieldWidgetClass, form,
		                      XmNeditable, True,
		                      XmNcursorPositionVisible, True,
		                      XmNsensitive, True,
				      XmNshadowThickness, 1,
		                      XmNcolumns, 25,
		                      XmNwidth, ((25*7)+2),
		                      XmNmaxLength, 80,
		                      XmNbackground, colors[0x0f],
		                      XmNtopAttachment,XmATTACH_WIDGET,
		                      XmNtopWidget, bando_max_data,
		                      XmNtopOffset, 5,
		                      XmNbottomAttachment, XmATTACH_NONE,
		                      XmNleftAttachment, XmATTACH_POSITION,
		                      XmNleftPosition, 2,
		                      XmNrightAttachment, XmATTACH_NONE,
		                      NULL);


    sep = XtVaCreateManagedWidget("sep", xmSeparatorGadgetClass,form,
                                      XmNorientation, XmHORIZONTAL,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, audio_alarm_config_play_on_wx_alert,
                                      XmNtopOffset, 20,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment,XmATTACH_FORM,
                                      XmNbackground, colors[0xff],
                                      NULL);



    button_ok = XtVaCreateManagedWidget(langcode("UNIOP00001"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, sep, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 1,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 2,
                                      XmNbackground, colors[0xff],
                                      NULL);

    button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, sep, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 3,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 4,
                                      XmNbackground, colors[0xff],
                                      NULL);

   XtAddCallback(button_ok, XmNactivateCallback, Configure_audio_alarm_change_data, configure_audio_alarm_dialog);
   XtAddCallback(button_cancel, XmNactivateCallback, Configure_audio_alarm_destroy_shell, configure_audio_alarm_dialog);

   pos_dialog(configure_audio_alarm_dialog); 

   delw = XmInternAtom(XtDisplay(configure_audio_alarm_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(configure_audio_alarm_dialog, delw, Configure_audio_alarm_destroy_shell, (XtPointer)configure_audio_alarm_dialog);

   XmTextFieldSetString(audio_alarm_config_play_data,sound_command);
   XmTextFieldSetString(audio_alarm_config_play_ons_data,sound_new_station);
   XmTextFieldSetString(audio_alarm_config_play_onm_data,sound_new_message);
   XmTextFieldSetString(audio_alarm_config_play_onpx_data,sound_prox_message);
   XmTextFieldSetString(prox_min_data,prox_min);
   XmTextFieldSetString(prox_max_data,prox_max);
   XmTextFieldSetString(audio_alarm_config_play_onbo_data,sound_band_open_message);
   XmTextFieldSetString(bando_min_data,bando_min);
   XmTextFieldSetString(bando_max_data,bando_max);
   XmTextFieldSetString(audio_alarm_config_wx_alert_data, sound_wx_alert_message);


   if(sound_play_new_station)
     {XmToggleButtonSetState(audio_alarm_config_play_on_new_station,True,False);}
     else
     {XmToggleButtonSetState(audio_alarm_config_play_on_new_station,False,False);}

   if(sound_play_new_message)
     {XmToggleButtonSetState(audio_alarm_config_play_on_new_message,True,False);}
     else
     {XmToggleButtonSetState(audio_alarm_config_play_on_new_message,False,False);}

   if(sound_play_prox_message)
     {XmToggleButtonSetState(audio_alarm_config_play_on_prox,True,False);}
     else
     {XmToggleButtonSetState(audio_alarm_config_play_on_prox,False,False);}

   if(sound_play_band_open_message)
     {XmToggleButtonSetState(audio_alarm_config_play_on_bando,True,False);}
     else
     {XmToggleButtonSetState(audio_alarm_config_play_on_bando,False,False);}

   if (sound_play_wx_alert_message)
     XmToggleButtonSetState(audio_alarm_config_play_on_wx_alert, True, False);
   else
     XmToggleButtonSetState(audio_alarm_config_play_on_wx_alert, False, False);

   XtManageChild(form);  
   XtManageChild(pane);  
      
   XtPopup(configure_audio_alarm_dialog,XtGrabNone);
   fix_dialog_size(configure_audio_alarm_dialog);
  } else {
     XRaiseWindow(XtDisplay(configure_audio_alarm_dialog), XtWindow(configure_audio_alarm_dialog));
  }
}


/**** STATION CONFIGURE *******/

void Configure_station_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 configure_station_dialog = (Widget)NULL;
}

void  Configure_station_toggle(Widget widget, XtPointer clientData, XtPointer callData)
{char *which = (char *)clientData;
 XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *)callData;

 if(state->set)
   {Configure_station_pos_amb = atoi(which);}
  else
   {Configure_station_pos_amb = 0;}
}


void Configure_station_change_data(Widget widget, XtPointer clientData, XtPointer callData)
{char temp[40];
 char old_callsign[MAX_CALL+1];

strcpy(old_callsign,my_callsign);
/*printf("Changing Configure station data\n");*/
 
sprintf(my_callsign,"%s",XmTextFieldGetString(station_config_call_data));
remove_trailing_spaces(my_callsign);
to_upper(my_callsign);

strcpy(temp,XmTextFieldGetString(station_config_slat_data_ns));
if(toupper(temp[0])=='S')
  {temp[0]='S';}
  else
  {temp[0]='N';}
sprintf(my_lat,"%02d%05.2f%c",atoi(XmTextFieldGetString(station_config_slat_data_deg)),atof(XmTextFieldGetString(station_config_slat_data_min)),temp[0]);

strcpy(temp,XmTextFieldGetString(station_config_slong_data_ew));
if(toupper(temp[0])=='E')
  {temp[0]='E';}
  else
  {temp[0]='W';}
sprintf(my_long,"%03d%05.2f%c",atoi(XmTextFieldGetString(station_config_slong_data_deg)),atof(XmTextFieldGetString(station_config_slong_data_min)),temp[0]);

strcpy(temp,XmTextFieldGetString(station_config_group_data));
my_group=temp[0];
if(isalpha(my_group))
  {my_group=toupper(temp[0]);}

strcpy(temp,XmTextFieldGetString(station_config_symbol_data));
my_symbol=temp[0];



strcpy(temp,XmTextFieldGetString(station_config_power_data));
strcat(temp,XmTextFieldGetString(station_config_height_data));
strcat(temp,XmTextFieldGetString(station_config_gain_data));
strcat(temp,XmTextFieldGetString(station_config_direction_data));
if(isdigit(temp[0]) && isdigit(temp[1]) && isdigit(temp[2]) && isdigit(temp[3]))
  {sprintf(my_phg,"PHG%s/",temp);}
  else
  {my_phg[0]=0;}

/* set station ambiguity*/
position_amb_chars = Configure_station_pos_amb;

strcpy(my_comment,XmTextFieldGetString(station_config_comment_data));
remove_trailing_spaces(my_comment);

/* TO DO: KILL only my station data? */
station_del(old_callsign);
my_station_add(my_callsign,my_group,my_symbol,my_long,my_lat,my_phg,my_comment);   
redraw_on_new_data=2;
Configure_station_destroy_shell(widget,clientData,callData);
}

void Configure_station(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane, form, form1, form2, button_ok, button_cancel, call, frame, frame1, frame2, 
                slat, 
                slat_deg,  slat_min, slat_ns, 
                slong, 
                slong_deg, slong_min, slong_ew, 
                sts, group, symbol, comment,
                pg, power, height, gain, direction,
                posamb,option_box,posamb0,posamb1,posamb2,posamb3,posamb4,
                sep;

 char temp_data[40];
 Atom delw;
 Arg al[2];                    /* Arg List */
 register int ac = 0;           /* Arg Count */  


 if(!configure_station_dialog)   
   {configure_station_dialog = XtVaCreatePopupShell(langcode("WPUPCFS001"),xmDialogShellWidgetClass,hidden_shell,
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, configure_station_dialog,
                          XmNbackground, colors[0xff],
                          NULL);

    form =  XtVaCreateWidget("form2",xmFormWidgetClass, pane,
                            XmNfractionBase, 5,
                            XmNbackground, colors[0xff],
                            XmNautoUnmanage, False,
                            XmNshadowThickness, 1, 
                            NULL);

    call = XtVaCreateManagedWidget(langcode("WPUPCFS002"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_call_data = XtVaCreateManagedWidget("call_data", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 15,
                                      XmNwidth, ((15*7)+2),
                                      XmNmaxLength, 15,
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);


    slat = XtVaCreateManagedWidget(langcode("WPUPCFS003"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, call,
                                      XmNtopOffset, 25, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    station_config_slat_data_deg = XtVaCreateManagedWidget("lat_deg", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 2,
                                      XmNmaxLength, 2,
                                      XmNtopOffset, 20, 
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, call,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slat_deg = XtVaCreateManagedWidget(langcode("WPUPCFS004"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, call,
                                      XmNtopOffset, 25, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, station_config_slat_data_deg,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    station_config_slat_data_min = XtVaCreateManagedWidget("lat_min", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 5,
                                      XmNmaxLength, 5,
                                      XmNtopOffset, 20, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, slat_deg,
                                      XmNleftOffset, 10,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, call,                                
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slat_min = XtVaCreateManagedWidget(langcode("WPUPCFS005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, call,
                                      XmNtopOffset, 25, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_slat_data_min,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    station_config_slat_data_ns = XtVaCreateManagedWidget("lat_ns", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 20, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, slat_min,
                                      XmNleftOffset, 10,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, call,                                
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slat_ns = XtVaCreateManagedWidget(langcode("WPUPCFS006"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, call,
                                      XmNtopOffset, 25, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_slat_data_ns,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    slong = XtVaCreateManagedWidget(langcode("WPUPCFS007"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, slat,
                                      XmNtopOffset, 20, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_slong_data_deg = XtVaCreateManagedWidget("long_deg", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 3,
                                      XmNmaxLength, 3,
                                      XmNtopOffset, 14, 
                                      XmNbackground, colors[0x0f],
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, slat,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slong_deg = XtVaCreateManagedWidget(langcode("WPUPCFS004"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, slat,
                                      XmNtopOffset, 20, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, station_config_slong_data_deg,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    station_config_slong_data_min = XtVaCreateManagedWidget("long_min", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 5,
                                      XmNmaxLength, 5,
                                      XmNtopOffset, 14, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, slong_deg,
                                      XmNleftOffset, 10,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, slat,                                
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slong_min = XtVaCreateManagedWidget(langcode("WPUPCFS005"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, slat,
                                      XmNtopOffset, 20, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_slong_data_min,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_slong_data_ew = XtVaCreateManagedWidget("long_ew", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1, 
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 14, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, slong_min,
                                      XmNleftOffset, 10,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, slat,                                
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    slong_ew = XtVaCreateManagedWidget(langcode("WPUPCFS008"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, slat,
                                      XmNtopOffset, 20, 
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_slong_data_ew,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);

    frame = XtVaCreateManagedWidget("frame1", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, slong_ew,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    sts  = XtVaCreateManagedWidget(langcode("WPUPCFS009"),xmLabelWidgetClass,frame,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    form1 =  XtVaCreateWidget("form1",xmFormWidgetClass, frame,
                            XmNfractionBase, 5, 
                            XmNbackground, colors[0xff],
                            NULL);

    group = XtVaCreateManagedWidget(langcode("WPUPCFS010"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 10,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 100,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_group_data = XtVaCreateManagedWidget("group", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, group,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,                    
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    symbol = XtVaCreateManagedWidget(langcode("WPUPCFS011"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_group_data,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_symbol_data = XtVaCreateManagedWidget("symbol", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, symbol,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,                    
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);


    frame1 = XtVaCreateManagedWidget("frame1", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, frame,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    pg  = XtVaCreateManagedWidget(langcode("WPUPCFS012"),xmLabelWidgetClass,frame1,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    form2 =  XtVaCreateWidget("form1",xmFormWidgetClass,frame1,
                            XmNfractionBase, 5, 
                            XmNbackground, colors[0xff],
                            NULL);


    power = XtVaCreateManagedWidget(langcode("WPUPCFS013"),xmLabelWidgetClass, form2,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset,10,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_power_data = XtVaCreateManagedWidget("power", xmTextFieldWidgetClass, form2,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, power,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    height = XtVaCreateManagedWidget(langcode("WPUPCFS014"),xmLabelWidgetClass, form2,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_power_data,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_height_data = XtVaCreateManagedWidget("power", xmTextFieldWidgetClass, form2,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, height,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    gain = XtVaCreateManagedWidget(langcode("WPUPCFS015"),xmLabelWidgetClass, form2,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_height_data,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_gain_data = XtVaCreateManagedWidget("gain", xmTextFieldWidgetClass, form2,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1, 
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, gain,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    direction = XtVaCreateManagedWidget(langcode("WPUPCFS016"),xmLabelWidgetClass, form2,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget,station_config_gain_data,
                                      XmNleftOffset, 20,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_direction_data = XtVaCreateManagedWidget("gain", xmTextFieldWidgetClass, form2,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 1,
                                      XmNmaxLength, 1,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, direction,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_FORM,
                                      XmNrightOffset, 5,
                                      NULL);

    comment = XtVaCreateManagedWidget(langcode("WPUPCFS017"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, frame1, 
                                      XmNtopOffset, 15,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNbackground, colors[0xff],
                                      NULL);


    station_config_comment_data = XtVaCreateManagedWidget("comment", xmTextFieldWidgetClass, form,
                                      XmNeditable,   True,
                                      XmNcursorPositionVisible, True,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 60,
                                      XmNwidth, ((60*7)+2), 
                                      XmNmaxLength, MAX_COMMENT,
                                      XmNtopOffset, 11, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, comment,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, frame1,                                
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    frame2 = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, comment,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    XmNbackground, colors[0xff],
                                    NULL);

    posamb  = XtVaCreateManagedWidget(langcode("WPUPCFS018"),xmLabelWidgetClass,frame2,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    XmNbackground, colors[0xff],
                                    NULL);

    /*set args for color */
    ac=0;
    XtSetArg(al[ac], XmNbackground, colors[0xff]); ac++; 

    option_box = XmCreateRadioBox(frame2,"Option box",al,ac);

    XtVaSetValues(option_box,
                  XmNnumColumns,4,
                  NULL);

    posamb0 = XtVaCreateManagedWidget(langcode("WPUPCFS019"),xmToggleButtonGadgetClass,
			              option_box,
                                      XmNbackground, colors[0xff],
       	                              NULL);
    XtAddCallback(posamb0,XmNvalueChangedCallback,Configure_station_toggle,"0");  


    posamb1 = XtVaCreateManagedWidget(units_english_metric?langcode("WPUPCFS020"):langcode("WPUPCFS024"),xmToggleButtonGadgetClass,
			              option_box,
                                      XmNbackground, colors[0xff],
       	                              NULL);
    XtAddCallback(posamb1,XmNvalueChangedCallback,Configure_station_toggle,"1");  
  

    posamb2 = XtVaCreateManagedWidget(units_english_metric?langcode("WPUPCFS021"):langcode("WPUPCFS025"),xmToggleButtonGadgetClass,
			              option_box,
                                      XmNbackground, colors[0xff],
       	                              NULL);
    XtAddCallback(posamb2,XmNvalueChangedCallback,Configure_station_toggle,"2");  

    posamb3 = XtVaCreateManagedWidget(units_english_metric?langcode("WPUPCFS022"):langcode("WPUPCFS026"),xmToggleButtonGadgetClass,
			              option_box,
                                      XmNbackground, colors[0xff],
       	                              NULL);
    XtAddCallback(posamb3,XmNvalueChangedCallback,Configure_station_toggle,"3");  

    posamb4 = XtVaCreateManagedWidget(units_english_metric?langcode("WPUPCFS023"):langcode("WPUPCFS027"),xmToggleButtonGadgetClass,
			              option_box,
                                      XmNbackground, colors[0xff],
       	                              NULL);
    XtAddCallback(posamb4,XmNvalueChangedCallback,Configure_station_toggle,"4");  


    sep = XtVaCreateManagedWidget("sep", xmSeparatorGadgetClass,form,
                                      XmNorientation, XmHORIZONTAL,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, frame2,
                                      XmNtopOffset, 14,
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNrightAttachment,XmATTACH_FORM,
                                      XmNbackground, colors[0xff],
                                      NULL);


    button_ok = XtVaCreateManagedWidget(langcode("UNIOP00001"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, sep, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 1,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 2,
                                      XmNbackground, colors[0xff],
                                      NULL);

    button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, sep, 
                                      XmNtopOffset, 5,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNbottomOffset, 5,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 3,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 4,
                                      XmNbackground, colors[0xff],
                                      NULL);

   XtAddCallback(button_ok, XmNactivateCallback, Configure_station_change_data, configure_station_dialog);
   XtAddCallback(button_cancel, XmNactivateCallback, Configure_station_destroy_shell, configure_station_dialog);

   XmTextFieldSetString(station_config_call_data,my_callsign);

   strncpy(temp_data,my_lat,2);
   temp_data[2]=0;
   XmTextFieldSetString(station_config_slat_data_deg,temp_data);
   strncpy(temp_data,my_lat+2,5);
   temp_data[5]=0;
   XmTextFieldSetString(station_config_slat_data_min,temp_data);
   strncpy(temp_data,my_lat+7,1);
   temp_data[1]=0;
   XmTextFieldSetString(station_config_slat_data_ns,temp_data);

   strncpy(temp_data,my_long,3);
   temp_data[3]=0;
   XmTextFieldSetString(station_config_slong_data_deg,temp_data);
   strncpy(temp_data,my_long+3,5);
   temp_data[5]=0;
   XmTextFieldSetString(station_config_slong_data_min,temp_data);
   strncpy(temp_data,my_long+8,1);
   temp_data[1]=0;
   XmTextFieldSetString(station_config_slong_data_ew,temp_data);

   temp_data[0]=my_group;
   temp_data[1]=0;
   XmTextFieldSetString(station_config_group_data,temp_data);

   temp_data[0]=my_symbol;
   temp_data[1]=0;
   XmTextFieldSetString(station_config_symbol_data,temp_data);


   if(my_phg[0]=='P')
     {temp_data[0]=my_phg[3];
      temp_data[1]=0;
      XmTextFieldSetString(station_config_power_data,temp_data);

      temp_data[0]=my_phg[4];
      temp_data[1]=0;
      XmTextFieldSetString(station_config_height_data,temp_data);

      temp_data[0]=my_phg[5];
      temp_data[1]=0;
      XmTextFieldSetString(station_config_gain_data,temp_data);

      temp_data[0]=my_phg[6];
      temp_data[1]=0;
      XmTextFieldSetString(station_config_direction_data,temp_data);
     }


   XmTextFieldSetString(station_config_comment_data,my_comment);

   Configure_station_pos_amb = position_amb_chars; 
   switch(Configure_station_pos_amb)
          {case(0):XmToggleButtonSetState(posamb0,True,False);
                   break;
           case(1):XmToggleButtonSetState(posamb1,True,False);
                   break;
           case(2):XmToggleButtonSetState(posamb2,True,False);
                   break;
           case(3):XmToggleButtonSetState(posamb3,True,False);
                   break;
           case(4):XmToggleButtonSetState(posamb4,True,False);
                   break;
           default:XmToggleButtonSetState(posamb0,True,False);
                   break;
	  }

   pos_dialog(configure_station_dialog); 

   delw = XmInternAtom(XtDisplay(configure_station_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(configure_station_dialog, delw, Configure_station_destroy_shell, (XtPointer)configure_station_dialog);

   XtManageChild(form);  
   XtManageChild(form1);  
   XtManageChild(form2);  
   XtManageChild(option_box);
   XtManageChild(pane);  


   XtPopup(configure_station_dialog,XtGrabNone);
   fix_dialog_size(configure_station_dialog);
  } else {
     XRaiseWindow(XtDisplay(configure_station_dialog), XtWindow(configure_station_dialog));
  }
}



int main(int argc, char *argv[])
{int ag, ag_error;
 uid_t user_id;
 struct passwd *user_info;
 static char lang_to_use_or[30];
 char temp[100];

 /* get User info */
 user_id=getuid();
 user_info=getpwuid(user_id);
 strcpy(user_dir,user_info->pw_dir);

 /*
 printf("User %s, Dir %s\n",user_info->pw_name,user_dir);
 printf("User dir %s\n",get_user_base_dir("")); 
 printf("Data dir %s\n",get_data_base_dir("")); 
 */

 /* check user directories */
 if (filethere(get_user_base_dir(""))!=1) {
    printf("Making user dir\n");
    mkdir(get_user_base_dir(""),S_IRWXU);
 }

 if (filethere(get_user_base_dir("config"))!=1) {
    printf("Making user config dir\n");
    mkdir(get_user_base_dir("config"),S_IRWXU);
 }

 if (filethere(get_user_base_dir("data"))!=1) {
    printf("Making user data dir\n");
    mkdir(get_user_base_dir("data"),S_IRWXU);
 }
 
 if (filethere(get_user_base_dir("logs"))!=1) {
    printf("Making user dir\n");
    mkdir(get_user_base_dir("logs"),S_IRWXU);
 }

 /* done checking user dirs */


 /* check fhs directories ?*/

 /* setup values */
 max_stations=0;
 current_trail_color=1;
 track_station_on=0;
 delay_time=0;
 packet_data_display=0;
 redraw_on_new_packet_data=0;
 stations=0;
 read_file=0;
 redraw_on_new_data=0;
 display_up=0;
 display_up_first=0;
 max_transmit_time=900l;
 sec_next_gps=0l;
 sec_next_raw_wx=0l;
 auto_reply = 0;
 last_time = 0l;
 next_time = 120l;
 net_last_time = 0l;
 net_next_time = 120l;
 wait_to_redraw=1;
 new_message_data=0;
 last_popup_x=0;
 last_popup_y=0;
 debug_level=0;
 message_counter=0;
 last_sound_pid=0;
 my_last_course=0;
 my_last_speed=0;
 my_last_altitude=0l;
 my_last_altitude_time=0l;
 fcc_data_available=0;
 rac_data_available=0;
 strcpy(wx_station_type,"");
 last_alert_redraw=0;
 igate_msgs_tx=0;

 /* clear wx data; */
 clear_rain_data();
 clear_local_wx_data();

 strcpy(lang_to_use_or,"");
 ag_error=0;
 while ((ag = getopt(argc, argv, "v:l:0123467890")) != EOF) {
        switch (ag) {
        case 'v':  printf ("debug");
                 if (optarg) {
                   debug_level=atoi(optarg);
                   printf (" level %d", debug_level);
		 }
                 printf ("\n");
                 break;  

        case 'l':  printf ("Language is");
                 if (optarg) {
                   if (strcasecmp(optarg,"ENGLISH")==0) {
                      strcpy(lang_to_use_or,"English");
		   } else if (strcasecmp(optarg,"DUTCH")==0) {
                          strcpy(lang_to_use_or,"Dutch");
		   } else if (strcasecmp(optarg,"FRENCH")==0) {
                          strcpy(lang_to_use_or,"French");
		   } else if (strcasecmp(optarg,"GERMAN")==0) {
                          strcpy(lang_to_use_or,"German");
                   } else if (strcasecmp(optarg,"SPANISH")==0) {
                          strcpy(lang_to_use_or,"Spanish"); 
		   } else {ag_error++;
                           printf(" INVALID");
		   }
                  if (!ag_error) {
                     printf(" %s", lang_to_use_or);

		  }

		 }
                 printf ("\n");
                 break;  

        default: ag_error++;
                 break;
	}
 }
 
 if (ag_error){
   printf("\nXastir Command line Options\n\n");
   printf("-v level    Set the debug level\n\n");
   printf("-lang English   Set English as the Language to use\n");
   printf("-lang French    Set French as the Language to use\n"); 
   printf("-lang German    Set German as the Language to use\n"); 
   printf("-lang Spanish   Set Spanish as the Language to use\n");

   printf("\n");
   exit(0);
 }

 next_redraw=sec_now()+60;

 /* init interface */
 /* Initialize the lock on port data */
 pthread_mutex_init(&port_data_lock, NULL);
 pthread_mutex_init(&output_data_lock, NULL);
 pthread_mutex_init(&data_lock, NULL);

 /* clear interface port data */
 clear_all_port_data(); 


 /* get default data */
 load_data_or_default();

 /* do language links */
 if (strlen(lang_to_use_or)>0) {
     strcpy(lang_to_use,lang_to_use_or);
 }


 sprintf(temp,"help/help-%s.dat",lang_to_use);
 unlink(get_user_base_dir("config/help.dat"));
 if (symlink(get_data_base_dir(temp),get_user_base_dir("config/help.dat"))==-1) {
   printf("Error creating database link\n");
   exit(0);
 }

 sprintf(temp,"config/language-%s.sys",lang_to_use);
 unlink(get_user_base_dir("config/language.sys"));
 if (symlink(get_data_base_dir(temp),get_user_base_dir("config/language.sys"))==-1) {
    printf("Error creating database link\n");
   exit(0);
 }


 /* set signal on stop */
 (void) signal(SIGINT,quit);
 (void) signal(SIGQUIT,quit);
 (void) signal(SIGTERM,quit);

 /* set sigpipe signal to ignore */
 (void) signal(SIGPIPE,SIG_IGN);
 
 /* set segfault signal to check */
 (void) signal(SIGSEGV,segfault);

 /* (NEW) set help file area */
 strcpy(HELP_FILE,get_user_base_dir("config/help.dat"));

 if (load_color_file()) {

    if (load_language_file(get_user_base_dir("config/language.sys"))) {
   
       /* set interface names */
       init_device_names();
 
       /* get symbol data */
        clear_symbol_data();
        symbols_loaded=0;

        clear_message_windows();
        clear_popup_message_windows();

        clean_data_file();
        mclean_data_file();
        reset_outgoing_messages();

        /* set language attribs */
	XtSetLanguageProc ( (XtAppContext) NULL, (XtLanguageProc) NULL, (XtPointer) NULL );
	XtToolkitInitialize ();

        hidden_shell=XtVaAppInitialize(&app_context,"Xastir", NULL, 0, 
                                       &argc, argv, NULL,
                                       XmNmappedWhenManaged, FALSE, 
                                       NULL); 

	display = XtOpenDisplay (app_context, NULL, argv[0], "XApplication",NULL, 0, &argc, argv);
	if (!display)
	{
	    printf("%s: can't open display, exiting...\n", argv[0]);
	    exit (-1);
	}

        XtRealizeWidget(hidden_shell);

	create_appshell ( display, argv[0], argc, argv );

        /* reset language attribs for numeric, program needs decimal in US for all data! */
        setlocale(LC_NUMERIC, "en_US");

        /* check for ham databases */
        check_fcc_data();
        check_rac_data();

        XtAppMainLoop(app_context);

    } else printf("Error in language file! Exiting...\n");
 } else printf("Error in Color file! Exiting...\n");

 quit(0);
 return 0;
}

